@@ -32,7 +32,7 @@ using std::mutex;
 
 namespace Extension {
 
-int gapped_filter(const SeedHit &hit, const LongScoreProfile *query_profile, const sequence &target, int band, int window, std::function<decltype(DP::ARCH_GENERIC::scan_diags128)> f) {	
+int gapped_filter(const SeedHit &hit, const LongScoreProfile *query_profile, const Sequence &target, int band, int window, std::function<decltype(DP::ARCH_GENERIC::scan_diags128)> f) {	
 	const int slen = (int)target.length();
 	const int d = std::max(hit.diag() - band / 2, -(slen - 1)),
 		j0 = std::max(hit.j - window, 0),
@@ -42,13 +42,13 @@ int gapped_filter(const SeedHit &hit, const LongScoreProfile *query_profile, con
 	return DP::diag_alignment(scores, band);
 }
 
-bool gapped_filter(const SeedHit *begin, const SeedHit *end, const LongScoreProfile *query_profile, uint32_t target_block_id, Statistics &stat, const Parameters &params) {
+bool gapped_filter(FlatArray<SeedHit>::ConstIterator begin, FlatArray<SeedHit>::ConstIterator end, const LongScoreProfile *query_profile, uint32_t target_block_id, Statistics &stat, const Search::Config &params) {
 	constexpr int window1 = 100, MIN_STAGE2_QLEN = 100;
 		
 	const int qlen = (int)query_profile->length();
-	const sequence target = ref_seqs::get()[target_block_id];
+	const Sequence target = params.target->seqs()[target_block_id];
 	const int slen = (int)target.length();
-	for (const SeedHit* hit = begin; hit < end; ++hit) {
+	for (FlatArray<SeedHit>::ConstIterator hit = begin; hit < end; ++hit) {
 		stat.inc(Statistics::GAPPED_FILTER_HITS1);
 		const int f1 = gapped_filter(*hit, query_profile, target, 64, window1, DP::scan_diags64);
 		if (f1 > params.cutoff_gapped1_new(qlen, slen)) {
@@ -63,7 +63,7 @@ bool gapped_filter(const SeedHit *begin, const SeedHit *end, const LongScoreProf
 	return false;
 }
 
-void gapped_filter_worker(size_t i, size_t thread_id, const LongScoreProfile *query_profile, const FlatArray<SeedHit>* seed_hits, const uint32_t* target_block_ids, FlatArray<SeedHit>* out, vector<uint32_t> *target_ids_out, mutex* mtx, const Parameters *params) {
+void gapped_filter_worker(size_t i, size_t thread_id, const LongScoreProfile *query_profile, const FlatArray<SeedHit>* seed_hits, const uint32_t* target_block_ids, FlatArray<SeedHit>* out, vector<uint32_t> *target_ids_out, mutex* mtx, const Search::Config *params) {
 	thread_local Statistics stat;
 	if (gapped_filter(seed_hits->begin(i), seed_hits->end(i), query_profile, target_block_ids[i], stat, *params)) {
 		std::lock_guard<mutex> guard(*mtx);
@@ -72,12 +72,12 @@ void gapped_filter_worker(size_t i, size_t thread_id, const LongScoreProfile *qu
 	}
 }
 
-void gapped_filter(const sequence* query, const Bias_correction* query_cbs, FlatArray<SeedHit>& seed_hits, std::vector<uint32_t>& target_block_ids, Statistics& stat, int flags, const Parameters &params) {
+void gapped_filter(const Sequence* query, const Bias_correction* query_cbs, FlatArray<SeedHit>& seed_hits, std::vector<uint32_t>& target_block_ids, Statistics& stat, DP::Flags flags, const Search::Config &params) {
 	if (seed_hits.size() == 0)
 		return;
 	vector<LongScoreProfile> query_profile;
 	query_profile.reserve(align_mode.query_contexts);
-	for (unsigned i = 0; i < align_mode.query_contexts; ++i)
+	for (int i = 0; i < align_mode.query_contexts; ++i)
 		if(Stats::CBS::hauser(config.comp_based_stats))
 			query_profile.emplace_back(query[i], query_cbs[i]);
 		else
@@ -86,7 +86,7 @@ void gapped_filter(const sequence* query, const Bias_correction* query_cbs, Flat
 	FlatArray<SeedHit> hits_out;
 	vector<uint32_t> target_ids_out;
 	
-	if(flags & DP::PARALLEL) {
+	if(flag_any(flags, DP::Flags::PARALLEL)) {
 		mutex mtx;
 		Util::Parallel::scheduled_thread_pool_auto(config.threads_, seed_hits.size(), gapped_filter_worker, query_profile.data(), &seed_hits, target_block_ids.data(), &hits_out, &target_ids_out, &mtx, &params);
 	}
