@@ -23,20 +23,21 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../data/queries.h"
 #include "../util/sequence/sequence.h"
 #include "../../dp/ungapped.h"
+#include "../../util/util.h"
 
 using std::mutex;
 using std::vector;
 
 namespace Extension { namespace GlobalRanking {
 
-uint16_t recompute_overflow_scores(const SeedHit* begin, const SeedHit* end, size_t query_id, uint32_t target_id) {
-	const auto query = query_seqs::get()[query_id];
-	const auto target = ref_seqs::get()[target_id];
+uint16_t recompute_overflow_scores(FlatArray<SeedHit>::ConstIterator begin, FlatArray<SeedHit>::ConstIterator end, size_t query_id, uint32_t target_id, const Search::Config& cfg) {
+	const auto query = cfg.query->seqs()[query_id];
+	const auto target = cfg.target->seqs()[target_id];
 	int score = 0;
 	for (auto it = begin; it < end; ++it) {
 		if (it->score != UCHAR_MAX)
 			continue;
-		const sequence query_clipped = Util::Sequence::clip(query.data() + it->i - config.ungapped_window, config.ungapped_window * 2, config.ungapped_window);
+		const Sequence query_clipped = Util::Seq::clip(query.data() + it->i - config.ungapped_window, config.ungapped_window * 2, config.ungapped_window);
 		const ptrdiff_t window_left = query.data() + it->i - query_clipped.data();
 		const int s = ungapped_window(query_clipped.data(), target.data() + it->j - window_left, (int)query_clipped.length());
 		score = std::max(score, s);
@@ -44,11 +45,11 @@ uint16_t recompute_overflow_scores(const SeedHit* begin, const SeedHit* end, siz
 	return (uint16_t)std::min(score, USHRT_MAX);
 }
 
-std::vector<Extension::Match> ranking_list(size_t query_id, std::vector<TargetScore>::iterator begin, std::vector<TargetScore>::iterator end, std::vector<uint32_t>::const_iterator target_block_ids, const FlatArray<SeedHit>& seed_hits) {
+std::vector<Extension::Match> ranking_list(size_t query_id, std::vector<TargetScore>::iterator begin, std::vector<TargetScore>::iterator end, std::vector<uint32_t>::const_iterator target_block_ids, const FlatArray<SeedHit>& seed_hits, const Search::Config& cfg) {
 	size_t overflows = 0;
 	for (auto it = begin; it < end && it->score >= UCHAR_MAX; ++it)
 		if (it->score == UCHAR_MAX) {
-			it->score = recompute_overflow_scores(seed_hits.begin(it->target), seed_hits.end(it->target), query_id, target_block_ids[it->target]);
+			it->score = recompute_overflow_scores(seed_hits.begin(it->target), seed_hits.end(it->target), query_id, target_block_ids[it->target], cfg);
 			++overflows;
 		}
 	if (overflows > 0)
@@ -58,7 +59,7 @@ std::vector<Extension::Match> ranking_list(size_t query_id, std::vector<TargetSc
 	vector<Match> r;
 	r.reserve(std::min(size_t(end - begin), config.global_ranking_targets));
 	for (auto i = begin; i < end && n < config.global_ranking_targets; ++i, ++n) {
-		r.emplace_back(target_block_ids[i->target], i->score);
+		r.emplace_back(target_block_ids[i->target], Sequence(), Stats::TargetMatrix(), i->score);
 	}
 	return r;
 }
@@ -69,12 +70,11 @@ size_t write_merged_query_list_intro(uint32_t query_id, TextBuffer& buf) {
 	return seek_pos;
 }
 
-void write_merged_query_list(const IntermediateRecord& r, const ReferenceDictionary& dict, TextBuffer& out, BitVector& ranking_db_filter, Statistics& stat) {
-	const uint32_t database_id = dict.database_id(r.subject_dict_id);
-	out.write(database_id);
+void write_merged_query_list(const IntermediateRecord& r, TextBuffer& out, BitVector& ranking_db_filter, Statistics& stat) {
+	/*out.write(r.subject_oid);
 	out.write(uint16_t(r.score));
-	ranking_db_filter.set(database_id);
-	stat.inc(Statistics::TARGET_HITS1);
+	ranking_db_filter.set(r.subject_oid);
+	stat.inc(Statistics::TARGET_HITS1);*/
 }
 
 void finish_merged_query_list(TextBuffer& buf, size_t seek_pos) {
