@@ -26,15 +26,24 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <list>
 #include <mutex>
 #include <float.h>
-#include "../search/trace_pt_buffer.h"
+#include <atomic>
 #include "../basic/diagonal_segment.h"
 #include "../basic/const.h"
 #include "../dp/hsp_traits.h"
 #include "../stats/hauser_correction.h"
 #include "extend.h"
 #include "../util/data_structures/flat_array.h"
-#include "../basic/parameters.h"
 #include "../stats/cbs.h"
+#include "../dp/flags.h"
+#include "../data/block.h"
+
+struct SequenceSet;
+
+namespace Search {
+
+struct Hit;
+
+}
 
 namespace Extension {
 
@@ -55,58 +64,27 @@ struct SeedHit {
 };
 
 struct WorkTarget {
-	WorkTarget(size_t block_id, const sequence& seq, int query_len, const Stats::Composition& query_comp, const int16_t** query_matrix) :
-		block_id(block_id),
-		seq(seq)
-	{
-		ungapped_score.fill(0);
-		if (config.comp_based_stats == Stats::CBS::HAUSER_AND_AVG_MATRIX_ADJUST) {
-			const int l = (int)seq.length();
-			const auto c = Stats::composition(seq);
-			auto r = Stats::s_TestToApplyREAdjustmentConditional(query_len, l, query_comp.data(), c.data(), score_matrix.background_freqs());
-			if (r == Stats::eCompoScaleOldMatrix)
-				return;
-			if (*query_matrix == nullptr) {
-				*query_matrix = Stats::make_16bit_matrix(Stats::CompositionMatrixAdjust(query_len, query_len, query_comp.data(), query_comp.data(), Stats::CBS::AVG_MATRIX_SCALE, score_matrix.ideal_lambda(), score_matrix.joint_probs(), score_matrix.background_freqs()));
-				++target_matrix_count;
-			}
-			if (target_matrices[block_id] == nullptr) {
-				int16_t* target_matrix = Stats::make_16bit_matrix(Stats::CompositionMatrixAdjust(l, l, c.data(), c.data(), Stats::CBS::AVG_MATRIX_SCALE, score_matrix.ideal_lambda(), score_matrix.joint_probs(), score_matrix.background_freqs()));
-				bool del = false;
-				{
-					std::lock_guard<std::mutex> lock(target_matrices_lock);
-					if (target_matrices[block_id] == nullptr)
-						target_matrices[block_id] = target_matrix;
-					else del = true;
-				}
-				if (del)
-					delete[] target_matrix;
-				++target_matrix_count;
-			}
-			matrix = Stats::TargetMatrix(*query_matrix, target_matrices[block_id]);
-		}
-		else
-			matrix = Stats::TargetMatrix(query_comp, query_len, seq);
-	}
+	WorkTarget(size_t block_id, const Sequence& seq, int query_len, const Stats::Composition& query_comp, const int16_t** query_matrix);
 	bool adjusted_matrix() const {
 		return !matrix.scores.empty();
 	}
 	size_t block_id;
-	sequence seq;
+	Sequence seq;
 	std::array<int, MAX_CONTEXT> ungapped_score;
 	std::array<std::list<Hsp_traits>, MAX_CONTEXT> hsp;
 	Stats::TargetMatrix matrix;
 };
 
-std::vector<WorkTarget> ungapped_stage(const sequence* query_seq, const Bias_correction* query_cb, const Stats::Composition& query_comp, FlatArray<SeedHit>& seed_hits, const std::vector<uint32_t>& target_block_ids, int flags, Statistics& stat);
+std::vector<WorkTarget> ungapped_stage(const Sequence* query_seq, const Bias_correction* query_cb, const Stats::Composition& query_comp, FlatArray<SeedHit>& seed_hits, const std::vector<uint32_t>& target_block_ids, DP::Flags flags, Statistics& stat, const Block& target_block, const Mode mode);
 
 struct Target {
 
-	Target(size_t block_id, const sequence &seq, int ungapped_score, const Stats::TargetMatrix& matrix):
+	Target(size_t block_id, const Sequence &seq, int ungapped_score, const Stats::TargetMatrix& matrix):
 		block_id(block_id),
 		seq(seq),
 		filter_score(0),
 		filter_evalue(DBL_MAX),
+		best_context(0),
 		ungapped_score(ungapped_score),
 		matrix(matrix)
 	{}
@@ -114,8 +92,11 @@ struct Target {
 	void add_hit(std::list<Hsp> &list, std::list<Hsp>::iterator it) {
 		std::list<Hsp> &l = hsp[it->frame];
 		l.splice(l.end(), list, it);
-		filter_evalue = std::min(filter_evalue, l.back().evalue);
-		filter_score = std::max(filter_score, l.back().score);
+		if (l.back().score > filter_score) {
+			filter_evalue = l.back().evalue;
+			filter_score = l.back().score;
+			best_context = l.back().frame;
+		}
 	}
 
 	static bool comp_evalue(const Target &t, const Target& u) {
@@ -130,14 +111,15 @@ struct Target {
 		return !matrix.scores.empty();
 	}
 
-	void apply_filters(int source_query_len, const char *query_title, const sequence& query_seq);
-	void inner_culling(int source_query_len);
+	void apply_filters(int source_query_len, const char *query_title, const Sequence& query_seq, const Block& targets);
+	void inner_culling();
 	void max_hsp_culling();
 
 	size_t block_id;
-	sequence seq;
+	Sequence seq;
 	int filter_score;
 	double filter_evalue;
+	int best_context;
 	int ungapped_score;
 	std::array<std::list<Hsp>, MAX_CONTEXT> hsp;
 	Stats::TargetMatrix matrix;
@@ -158,23 +140,22 @@ struct TargetScore {
 	}
 };
 
-void load_hits(hit* begin, hit* end, FlatArray<SeedHit> &hits, std::vector<uint32_t> &target_block_ids, std::vector<TargetScore> &target_scores, unsigned query_len);
-void culling(std::vector<Target>& targets, int source_query_len, const char* query_title, const sequence& query_seq, size_t min_keep);
-bool append_hits(std::vector<Target>& targets, std::vector<Target>::const_iterator begin, std::vector<Target>::const_iterator end, size_t chunk_size, int source_query_len, const char* query_title, const sequence& query_seq);
-std::vector<WorkTarget> gapped_filter(const sequence *query, const Bias_correction* query_cbs, std::vector<WorkTarget>& targets, Statistics &stat);
-void gapped_filter(const sequence* query, const Bias_correction* query_cbs, FlatArray<SeedHit> &seed_hits, std::vector<uint32_t> &target_block_ids, Statistics& stat, int flags, const Parameters &params);
-std::vector<Target> align(const std::vector<WorkTarget> &targets, const sequence *query_seq, const Bias_correction *query_cb, int source_query_len, int flags, Statistics &stat);
-std::vector<Match> align(std::vector<Target> &targets, const sequence *query_seq, const Bias_correction *query_cb, int source_query_len, int flags, Statistics &stat, bool first_round_traceback);
-std::vector<Target> full_db_align(const sequence *query_seq, const Bias_correction *query_cb, int flags, Statistics &stat);
+void culling(std::vector<Target>& targets, int source_query_len, const char* query_title, const Sequence& query_seq, size_t min_keep, const Block& target_block);
+bool append_hits(std::vector<Target>& targets, std::vector<Target>::const_iterator begin, std::vector<Target>::const_iterator end, size_t chunk_size, int source_query_len, const char* query_title, const Sequence& query_seq, const Block& target_block);
+std::vector<WorkTarget> gapped_filter(const Sequence *query, const Bias_correction* query_cbs, std::vector<WorkTarget>& targets, Statistics &stat);
+void gapped_filter(const Sequence* query, const Bias_correction* query_cbs, FlatArray<SeedHit> &seed_hits, std::vector<uint32_t> &target_block_ids, Statistics& stat, DP::Flags flags, const Search::Config &params);
+std::vector<Target> align(const std::vector<WorkTarget> &targets, const Sequence *query_seq, const Bias_correction *query_cb, int source_query_len, DP::Flags flags, const HspValues hsp_values, const Mode mode, Statistics &stat);
+std::vector<Match> align(std::vector<Target> &targets, const Sequence *query_seq, const Bias_correction *query_cb, int source_query_len, DP::Flags flags, const HspValues first_round, const Mode mode, Statistics &stat);
+std::vector<Target> full_db_align(const Sequence *query_seq, const Bias_correction *query_cb, DP::Flags flags, const HspValues hsp_values, Statistics &stat, const Block& target_block);
+void recompute_alt_hsps(vector<Match>& matches, const Sequence* query, const int query_source_len, const Bias_correction* query_cb, const HspValues v, Statistics& stats);
 
 std::vector<Match> extend(
 	size_t query_id,
-	const Parameters &params,
-	const Metadata &metadata,
+	const Search::Config& cfg,
 	Statistics &stat,
-	int flags,
-	const FlatArray<SeedHit>& seed_hits,
-	const std::vector<uint32_t>& target_block_ids,
+	DP::Flags flags,
+	FlatArray<SeedHit>& seed_hits,
+	std::vector<uint32_t>& target_block_ids,
 	const std::vector<TargetScore>& target_scores);
 
 struct Memory {
