@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
                         Eberhard Karls Universitaet Tuebingen
 
@@ -26,6 +26,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <exception>
 #include <iomanip>
 #include <numeric>
+#include <thread>
 #include "../util/command_line_parser.h"
 #include "config.h"
 #include "../util/util.h"
@@ -40,25 +41,56 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../cluster/cluster_registry.h"
 #include "../basic/translate.h"
 #include "../dp/dp.h"
-#include "masking.h"
+#include "../masking/masking.h"
 #include "../util/system/system.h"
 #include "../util/simd.h"
 #include "../util/parallel/multiprocessing.h"
 
-using namespace std;
+using std::thread;
+using std::stringstream;
+using std::endl;
+using std::cerr;
+using std::ostream;
+using std::cout;
+using std::unique_ptr;
+
+const EMap<Sensitivity> EnumTraits<Sensitivity>::to_string = {
+	{ Sensitivity::FAST, "fast" },
+	{ Sensitivity::DEFAULT, "default" },
+	{ Sensitivity::MID_SENSITIVE, "mid-sensitive" },
+	{ Sensitivity::SENSITIVE, "sensitive" },
+	{ Sensitivity::MORE_SENSITIVE, "more-sensitive" },
+	{ Sensitivity::VERY_SENSITIVE, "very-sensitive" },
+	{ Sensitivity::ULTRA_SENSITIVE, "ultra-sensitive" }
+};
+
+const SEMap<Sensitivity> EnumTraits<Sensitivity>::from_string = {
+	{ "fast", Sensitivity::FAST },
+	{ "default", Sensitivity::DEFAULT },
+	{ "mid-sensitive", Sensitivity::MID_SENSITIVE },
+	{ "sensitive", Sensitivity::SENSITIVE },
+	{ "more-sensitive", Sensitivity::MORE_SENSITIVE },
+	{ "very-sensitive", Sensitivity::VERY_SENSITIVE },
+	{ "ultra-sensitive", Sensitivity::ULTRA_SENSITIVE }
+};
+
+const EMap<Config::Algo> EnumTraits<Config::Algo>::to_string = { { Config::Algo::DOUBLE_INDEXED, "Double-indexed" }, { Config::Algo::QUERY_INDEXED, "Query-indexed"}, {Config::Algo::CTG_SEED, "Query-indexed with contiguous seed"} };
+const SEMap<Config::Algo> EnumTraits<Config::Algo>::from_string = { {"", Config::Algo::AUTO}, { "0", Config::Algo::DOUBLE_INDEXED}, {"1", Config::Algo::QUERY_INDEXED}, {"ctg", Config::Algo::CTG_SEED} };
 
 Config config;
 
 void print_warnings() {
 	if (config.sensitivity >= Sensitivity::VERY_SENSITIVE || config.verbosity == 0 || config.swipe_all)
 		return;
+	if (config.command != Config::blastp && config.command != Config::blastx)
+		return;
 	const double ram = total_ram();
 	unsigned b = 2, c = 4;
 	stringstream msg;
 	if (config.sensitivity >= Sensitivity::SENSITIVE) {
 		if (ram >= 63) {
 			c = 1;
-			if(c < config.lowmem)
+			if(c < config.lowmem_)
 				msg << "use this parameter for better performance: -c1";
 		}
 	}
@@ -81,7 +113,7 @@ void print_warnings() {
 		else if (ram >= 31) {
 			b = 4;
 		}
-		if ((b > 2 && b > config.chunk_size) || c < config.lowmem) {
+		if ((b > 2 && b > config.chunk_size) || c < config.lowmem_) {
 			msg << "increase the block size for better performance using these parameters : -b" << b;
 			if (c != 4)
 				msg << " -c" << c;
@@ -114,10 +146,25 @@ std::string Config::single_query_file() const {
 	return query_file.empty() ? string() : query_file.front();
 }
 
+Compressor Config::compressor() const
+{
+	if (compression.empty() || compression == "0")
+		return Compressor::NONE;
+	else if (compression == "1")
+		return Compressor::ZLIB;
+	else if (compression == "zstd")
+		return Compressor::ZSTD;
+	else
+		throw std::runtime_error("Invalid compression algorithm: " + compression);
+}
+
 Config::Config(int argc, const char **argv, bool check_io)
 {
 	Command_line_parser parser;
 	parser.add_command("makedb", "Build DIAMOND database from a FASTA file", makedb)
+#ifdef WITH_BLASTDB
+		.add_command("prepdb", "Prepare BLAST database for use with Diamond", prep_blast_db)
+#endif
 		.add_command("blastp", "Align amino acid query sequences against a protein reference database", blastp)
 		.add_command("blastx", "Align DNA query sequences against a protein reference database", blastx)
 		.add_command("view", "View DIAMOND alignment archive (DAA) formatted file", view)
@@ -126,6 +173,7 @@ Config::Config(int argc, const char **argv, bool check_io)
 		.add_command("getseq", "Retrieve sequences from a DIAMOND database file", getseq)
 		.add_command("dbinfo", "Print information about a DIAMOND database file", dbinfo)
 		.add_command("test", "Run regression tests", regression_test)
+		.add_command("makeidx", "Make database index", makeidx)
 		.add_command("roc", "", roc)
 		.add_command("benchmark", "", benchmark)
 #ifdef EXTRA
@@ -134,7 +182,6 @@ Config::Config(int argc, const char **argv, bool check_io)
 		.add_command("dbstat", "", db_stat)
 		.add_command("mask", "", mask)
 		.add_command("fastq2fasta", "", fastq2fasta)
-		.add_command("test-io", "", test_io)
 		.add_command("read-sim", "", read_sim)
 		.add_command("info", "", info)
 		.add_command("seed-stat", "", seed_stat)
@@ -146,17 +193,20 @@ Config::Config(int argc, const char **argv, bool check_io)
 		.add_command("simulate-seqs", "", simulate_seqs)
 		.add_command("split", "", split)
 		.add_command("upgma", "", upgma)
-		.add_command("upgmamc", "", upgma_mc)		
+		.add_command("upgmamc", "", upgma_mc)
 		.add_command("reverse", "", reverse_seqs)
 		.add_command("compute-medoids", "", compute_medoids)
 		.add_command("mutate", "", mutate)
 		.add_command("merge-tsv", "", merge_tsv)
 		.add_command("roc-id", "", rocid)
+		.add_command("find-shapes", "", find_shapes)
+		.add_command("composition", "", composition)
+		.add_command("join", "", JOIN)
+		.add_command("hashseqs", "", HASH_SEQS)
+		.add_command("listseeds", "", LIST_SEEDS)
 #endif
 		;
 
-	string traceback_mode_str;
-
 	Options_group general("General options");
 	general.add()
 		("threads", 'p', "number of CPU threads", threads_)
@@ -179,7 +229,9 @@ Config::Config(int argc, const char **argv, bool check_io)
 \tsstart means Start of alignment in subject\n\
 \tsend means End of alignment in subject\n\
 \tqseq means Aligned part of query sequence\n\
+\tqseq_translated means Aligned part of query sequence (translated)\n\
 \tfull_qseq means Query sequence\n\
+\tfull_qseq_mate means Query sequence of the mate\n\
 \tsseq means Aligned part of subject sequence\n\
 \tfull_sseq means Subject sequence\n\
 \tevalue means Expect value\n\
@@ -229,13 +281,15 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("max-size-set", 0, "Maximum size of a set", max_size_set, (size_t) 1000)
 		("external", 0, "save set of edges external", external, false)
 		("cluster-similarity", 0, "Clustering similarity measure", cluster_similarity)
+		("cluster-graph-file", 0, "Filename for dumping the graph or reading the graph if cluster-restart", cluster_graph_file)
+		("cluster-restart", 0, "Restart clustering from dumped graph", cluster_restart)
 		("mcl-expansion", 0, "MCL expansion coefficient (default=2)", cluster_mcl_expansion, (uint32_t) 2)
 		("mcl-inflation", 0, "MCL inflation coefficient (default=2.0)", cluster_mcl_inflation, 2.0)
 		("mcl-chunk-size", 0, "MCL chunk size per thread (default=100)", cluster_mcl_chunk_size, (uint32_t) 1)
 		("mcl-max-iterations", 0, "MCL maximum iterations (default=100)", cluster_mcl_max_iter, (uint32_t) 100)
 		("mcl-sparsity-switch", 0, "MCL switch to sparse matrix computation (default=0.8) ", cluster_mcl_sparsity_switch, 0.8)
-		("mcl-graph-file", 0, "Filename for dumping the graph or reading the graph if mcl-restart", cluster_mcl_graph_file)
-		("mcl-restart", 0, "Restart MCL from dumped graph", cluster_mcl_restart);
+		("mcl-nonsymmetric", 0, "Do not symmetrize the transistion matrix before clustering", cluster_mcl_nonsymmetric)
+		("mcl-stats", 0, "Some stats about the connected components in MCL", cluster_mcl_stats);
 
 	Options_group aligner("Aligner options");
 	aligner.add()
@@ -250,19 +304,22 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("top", 0, "report alignments within this percentage range of top alignment score (overrides --max-target-seqs)", toppercent, 100.0)
 		("max-hsps", 0, "maximum number of HSPs per target sequence to report for each query (default=1)", max_hsps, 1u)
 		("range-culling", 0, "restrict hit culling to overlapping query ranges", query_range_culling)
-		("compress", 0, "compression for output files (0=none, 1=gzip)", compression)
+		("compress", 0, "compression for output files (0=none, 1=gzip, zstd)", compression)
 		("evalue", 'e', "maximum e-value to report alignments (default=0.001)", max_evalue, 0.001)
 		("min-score", 0, "minimum bit score to report alignments (overrides e-value setting)", min_bit_score)
 		("id", 0, "minimum identity% to report an alignment", min_id)
 		("query-cover", 0, "minimum query cover% to report an alignment", query_cover)
 		("subject-cover", 0, "minimum subject cover% to report an alignment", subject_cover)
-		("mid-sensitive", 0, "enable mid-sensitive mode (default: fast)", mode_mid_sensitive)
-		("sensitive", 0, "enable sensitive mode (default: fast)", mode_sensitive)
-		("more-sensitive", 0, "enable more sensitive mode (default: fast)", mode_more_sensitive)
-		("very-sensitive", 0, "enable very sensitive mode (default: fast)", mode_very_sensitive)
-		("ultra-sensitive", 0, "enable ultra sensitive mode (default: fast)", mode_ultra_sensitive)
+		("fast", 0, "enable fast mode", mode_fast)
+		("mid-sensitive", 0, "enable mid-sensitive mode", mode_mid_sensitive)
+		("sensitive", 0, "enable sensitive mode)", mode_sensitive)
+		("more-sensitive", 0, "enable more sensitive mode", mode_more_sensitive)
+		("very-sensitive", 0, "enable very sensitive mode", mode_very_sensitive)
+		("ultra-sensitive", 0, "enable ultra sensitive mode", mode_ultra_sensitive)
+		("iterate", 0, "iterated search with increasing sensitivity", iterate, Option<vector<string>>(), 0)
+		("global-ranking", 'g', "number of targets for global ranking", global_ranking_targets)
 		("block-size", 'b', "sequence block size in billions of letters (default=2.0)", chunk_size)
-		("index-chunks", 'c', "number of chunks for index processing (default=4)", lowmem)
+		("index-chunks", 'c', "number of chunks for index processing (default=4)", lowmem_)
 		("tmpdir", 't', "directory for temporary files", tmpdir)
 		("parallel-tmpdir", 0, "directory for temporary files used by multiprocessing", parallel_tmpdir)
 		("gapopen", 0, "gap open penalty", gap_open, -1)
@@ -272,31 +329,40 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("matrix", 0, "score matrix for protein alignment (default=BLOSUM62)", matrix, string("blosum62"))
 		("custom-matrix", 0, "file containing custom scoring matrix", matrix_file)
 		("comp-based-stats", 0, "composition based statistics mode (0-4)", comp_based_stats, 1u)
-		("masking", 0, "enable tantan masking of repeat regions (0/1=default)", masking, -1)
+		("masking", 0, "masking algorithm (none, seg, tantan=default)", masking, string("tantan"))
 		("query-gencode", 0, "genetic code to use to translate query (see user manual)", query_gencode, 1u)
 		("salltitles", 0, "include full subject titles in DAA file", salltitles)
 		("sallseqid", 0, "include all subject ids in DAA file", sallseqid)
 		("no-self-hits", 0, "suppress reporting of identical self hits", no_self_hits)
 		("taxonlist", 0, "restrict search to list of taxon ids (comma-separated)", taxonlist)
-		("taxon-exclude", 0, "exclude list of taxon ids (comma-separated)", taxon_exclude);
+		("taxon-exclude", 0, "exclude list of taxon ids (comma-separated)", taxon_exclude)
+		("seqidlist", 0, "filter the database by list of accessions", seqidlist)
+		("skip-missing-seqids", 0, "ignore accessions missing in the database", skip_missing_seqids);
+
+	string algo_str;
 
 	Options_group advanced("Advanced options");
 	advanced.add()
-		("algo", 0, "Seed search algorithm (0=double-indexed/1=query-indexed)", algo, -1)
-		("bin", 0, "number of query bins for seed search", query_bins)
+		("algo", 0, "Seed search algorithm (0=double-indexed/1=query-indexed/ctg=contiguous-seed)", algo_str)
+		("bin", 0, "number of query bins for seed search", query_bins_)
 		("min-orf", 'l', "ignore translated sequences without an open reading frame of at least this length", run_len)
-		("freq-sd", 0, "number of standard deviations for ignoring frequent seeds", freq_sd, 0.0)
-		("id2", 0, "minimum number of identities for stage 1 hit", min_identities)
+		("seed-cut", 0, "cutoff for seed complexity", seed_cut_)
+		("freq-masking", 0, "mask seeds based on frequency", freq_masking)
+		("freq-sd", 0, "number of standard deviations for ignoring frequent seeds", freq_sd_, 0.0)
+		("motif-masking", 0, "softmask abundant motifs (0/1)", motif_masking)
+		("id2", 0, "minimum number of identities for stage 1 hit", min_identities_)
 		("xdrop", 'x', "xdrop for ungapped alignment", ungapped_xdrop, 12.3)
-		("gapped-filter-evalue", 0, "E-value threshold for gapped filter (auto)", gapped_filter_evalue, -1.0)
+		("gapped-filter-evalue", 0, "E-value threshold for gapped filter (auto)", gapped_filter_evalue_, -1.0)
 		("band", 0, "band for dynamic programming computation", padding)
 		("shapes", 's', "number of seed shapes (default=all available)", shapes)
 		("shape-mask", 0, "seed shapes", shape_mask)
 		("multiprocessing", 0, "enable distributed-memory parallel processing", multiprocessing)
 		("mp-init", 0, "initialize multiprocessing run", mp_init)
+		("mp-recover", 0, "enable continuation of interrupted multiprocessing run", mp_recover)
+		("mp-query-chunk", 0, "process only a single query chunk as specified", mp_query_chunk, -1)
 		("ext-chunk-size", 0, "chunk size for adaptive ranking (default=auto)", ext_chunk_size)
 		("no-ranking", 0, "disable ranking heuristic", no_ranking)
-		("ext", 0, "Extension mode (banded-fast/banded-slow/full)", ext)
+		("ext", 0, "Extension mode (banded-fast/banded-slow/full)", ext_)
 		("culling-overlap", 0, "minimum range overlap with higher scoring hit to delete a hit (default=50%)", inner_culling_overlap, 50.0)
 		("taxon-k", 0, "maximum number of targets to report per species", taxon_k, (uint64_t)0)
 		("range-cover", 0, "percentage of query range to be covered for range culling (default=50%)", query_range_cover, 50.0)
@@ -308,6 +374,8 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("file-buffer-size", 0, "file buffer size in bytes (default=67108864)", file_buffer_size, (size_t)67108864)
 		("memory-limit", 'M', "Memory limit for extension stage in GB", memory_limit)
 		("no-unlink", 0, "Do not unlink temporary files.", no_unlink)
+		("target-indexed", 0, "Enable target-indexed mode", target_indexed)
+		("ignore-warnings", 0, "Ignore warnings", ignore_warnings)
 		("cut-bar", 0, "", cut_bar)
 		("check-multi-target", 0, "", check_multi_target)
 		("roc-file", 0, "", roc_file)
@@ -324,7 +392,7 @@ Config::Config(int argc, const char **argv, bool check_io)
 
 	Options_group getseq_options("Getseq options");
 	getseq_options.add()
-		("seq", 0, "Sequence numbers to display.", seq_no);
+		("seq", 0, "Space-separated list of sequence numbers to display.", seq_no);
 
 	double rank_ratio2, lambda, K;
 	unsigned window, min_ungapped_score, hit_band, min_hit_score;
@@ -350,14 +418,8 @@ Config::Config(int argc, const char **argv, bool check_io)
 	Options_group hidden_options("", true);
 #endif
 	hidden_options.add()
-		("extend-all", 0, "extend all seed hits", extend_all)
-		("local-align", 0, "Local alignment algorithm", local_align_mode, 0u)
-		("slow-search", 0, "", slow_search)
-		("ht", 0, "", ht_mode)
-		("old-freq", 0, "", old_freq)
 		("match1", 0, "", match_file1)
 		("match2", 0, "", match_file2)
-		("max-hits", 'C', "maximum number of hits to consider for one seed", hit_cap)
 		("seed-freq", 0, "maximum seed frequency", max_seed_freq, -15.0)
 		("space-penalty", 0, "", space_penalty, 0.5)
 		("reverse", 0, "", reverse)
@@ -376,8 +438,6 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("d_new", 0, "", d_new, 1.0)
 		("score-estimate-factor", 0, "", score_estimate_factor, 0.0)
 		("diag-min-estimate", 0, "", diag_min_estimate, 17)
-		("qfilt", 0, "", qfilt)
-		("sfilt", 0, "", sfilt)
 		("path-cutoff", 0, "", path_cutoff, 0.92)
 		("sw", 0, "", use_smith_waterman)
 		("superblock", 0, "", superblock, 128)
@@ -389,8 +449,6 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("score-ratio", 0, "", score_ratio, 0.9)
 		("fetch-size", 0, "trace point fetch size", fetch_size, 4096u)
 		("target-fetch-size", 0, "number of target sequences to fetch for seed extension", target_fetch_size, 4u)
-		("small-query", 0, "", small_query)
-		("hashed-seeds", 0, "", hashed_seeds)
 		("rank-factor", 0, "", rank_factor, -1.0)
 		("transcript-len-estimate", 0, "", transcript_len_estimate, 1.0)
 		("family-counts", 0, "", family_counts_file)
@@ -418,25 +476,24 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("tantan-maxRepeatOffset", 0, "maximum tandem repeat period to consider (50)", tantan_maxRepeatOffset, 15)
 		("tantan-ungapped", 0, "use tantan masking in ungapped mode", tantan_ungapped)
 		("chaining-range-cover", 0, "", chaining_range_cover, (size_t)8)
-		("index-mode", 0, "index mode (0=4x12, 1=16x9)", index_mode)
 		("no-swipe-realign", 0, "", no_swipe_realign)
 		("chaining-maxnodes", 0, "", chaining_maxnodes)
 		("cutoff-score-8bit", 0, "", cutoff_score_8bit, 240)
-		("min-band-overlap", 0, "", min_band_overlap, 0.2)
+		("min-band-overlap", 0, "", min_band_overlap, 0.0)
 		("min-realign-overhang", 0, "", min_realign_overhang, 30)
 		("ungapped-window", 0, "", ungapped_window, 48)
 		("gapped-filter-diag-score", 0, "", gapped_filter_diag_bit_score, 12.0)
 		("gapped-filter-window", 0, "", gapped_filter_window, 200)
 		("output-hits", 0, "", output_hits)
-		("ungapped-evalue", 0, "", ungapped_evalue, -1.0)
-		("ungapped-evalue-short", 0, "", ungapped_evalue_short, -1.0)
+		("ungapped-evalue", 0, "", ungapped_evalue_, -1.0)
+		("ungapped-evalue-short", 0, "", ungapped_evalue_short_, -1.0)
 		("no-logfile", 0, "", no_logfile)
 		("no-heartbeat", 0, "", no_heartbeat)
 		("band-bin", 0, "", band_bin, 24)
 		("col-bin", 0, "", col_bin, 400)
 		("self", 0, "", self)
 		("trace-pt-fetch-size", 0, "", trace_pt_fetch_size, (size_t)10e9)
-		("tile-size", 0, "", tile_size, (uint32_t)1024)
+		("tile-size", 0, "", tile_size, (size_t)1024)
 		("short-query-ungapped-bitscore", 0, "", short_query_ungapped_bitscore, 25.0)
 		("short-query-max-len", 0, "", short_query_max_len, 60)
 		("gapped-filter-evalue1", 0, "", gapped_filter_evalue1, 2000.0)
@@ -449,8 +506,6 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("chaining-min-nodes", 0, "", chaining_min_nodes, (size_t)200)
 		("fast-tsv", 0, "", fast_tsv)
 		("target-parallel-verbosity", 0, "", target_parallel_verbosity, UINT_MAX)
-		("ext-targets", 0, "", global_ranking_targets)
-		("traceback-mode", 0, "", traceback_mode_str)
 		("query-memory", 0, "", query_memory)
 		("memory-intervals", 0, "", memory_intervals, (size_t)2)
 		("seed-hit-density", 0, "", seedhit_density)
@@ -466,37 +521,32 @@ Config::Config(int argc, const char **argv, bool check_io)
 		("cbs-matrix-scale", 0, "", cbs_matrix_scale, 1)
 		("query-count", 0, "", query_count, (size_t)1)
 		("cbs-angle", 0, "", cbs_angle, -1.0)
-		("target-seg", 0, "", target_seg, -1)
 		("cbs-err-tolerance", 0, "", cbs_err_tolerance, 0.00000001)
 		("cbs-it-limit", 0, "", cbs_it_limit, 2000)
 		("hash_join_swap", 0, "", hash_join_swap)
 		("deque_bucket_size", 0, "", deque_bucket_size, (size_t)524288)
 		("query-match-distance-threshold", 0, "", query_match_distance_threshold, -1.0)
 		("length-ratio-threshold", 0, "", length_ratio_threshold, -1.0)
-		("fast", 0, "", mode_fast)
-		("target-indexed", 0, "", target_indexed)
-		("mmap-target-index", 0, "", mmap_target_index)
-		("save-target-index", 0, "", save_target_index)
-		("max-swipe-dp", 0, "", max_swipe_dp, (size_t)4000000);
-	
+		("max-swipe-dp", 0, "", max_swipe_dp, (int64_t)1000000)
+		("short-seqids", 0, "", short_seqids)
+		("no-reextend", 0, "", no_reextend)
+		("no-reorder", 0, "", no_reorder)
+		("file1", 0, "", file1)
+		("file2", 0, "", file2)
+		("key2", 0, "", key2)
+		("motif-mask-file", 0, "", motif_mask_file)
+		("max-motif-len", 0, "", max_motif_len, 30)
+		("chaining-stacked-hsp-ratio", 0, "", chaining_stacked_hsp_ratio, 0.5);
+
 	parser.add(general).add(makedb).add(cluster).add(aligner).add(advanced).add(view_options).add(getseq_options).add(hidden_options).add(deprecated_options);
 	parser.store(argc, argv, command);
 
-	traceback_mode = set_string_option<TracebackMode>(traceback_mode_str, "--traceback-mode",
-		{ {"score", TracebackMode::SCORE_ONLY },
-		{"stat", TracebackMode::STAT},
-		{"vector", TracebackMode::VECTOR},
-		{"buffer", TracebackMode::SCORE_BUFFER} });
-
 	if (toppercent != 100.0 && max_alignments != 25)
 		throw std::runtime_error("--top and --max-target-seqs are mutually exclusive.");
 
 	if (command == blastx && no_self_hits)
 		throw std::runtime_error("--no-self-hits option is not supported in blastx mode.");
 
-	if (command == blastx && swipe_all)
-		throw std::runtime_error("Full db alignment is not supported in blastx mode.");
-
 	if (long_reads) {
 		query_range_culling = true;
 		if (toppercent == 100.0)
@@ -505,51 +555,21 @@ Config::Config(int argc, const char **argv, bool check_io)
 			frame_shift = 15;
 	}
 
-	if (global_ranking_targets > 0 && (query_range_culling || taxon_k || multiprocessing || mp_init || (command == blastx) || comp_based_stats >= 2))
+	if (global_ranking_targets > 0 && (query_range_culling || taxon_k || multiprocessing || mp_init || mp_recover || comp_based_stats >= 2 || frame_shift > 0))
 		throw std::runtime_error("Global ranking is not supported in this mode.");
 
-	if (global_ranking_targets > 0) {
-		if (ext != "" && ext != "full")
-			throw std::runtime_error("Global ranking only supports full matrix extension.");
-		ext = "full";
-	}
-
 #ifdef EXTRA
 	if (comp_based_stats >= Stats::CBS::COUNT)
 #else
 	if (comp_based_stats >= 5)
-#endif	
+#endif
 		throw std::runtime_error("Invalid value for --comp-based-stats. Permitted values: 0, 1, 2, 3, 4.");
 
-	if (masking == -1)
-		masking = Stats::CBS::tantan(comp_based_stats);
-
-	if (target_seg == -1)
-		target_seg = Stats::CBS::target_seg(comp_based_stats);
-
 	Stats::comp_based_stats = Stats::CBS(comp_based_stats, query_match_distance_threshold, length_ratio_threshold, cbs_angle);
 
 	if (command == blastx && !Stats::CBS::support_translated(comp_based_stats))
 		throw std::runtime_error("This mode of composition based stats is not supported for translated searches.");
 
-	if (swipe_all)
-		ext = "full";
-
-	if (max_hsps > 1 && ext == "full")
-		throw std::runtime_error("--max-hsps > 1 is not supported for full matrix extension.");
-
-	if (ext == "full" && comp_based_stats >= 2)
-		throw std::runtime_error("This mode of composition based stats is not supported for full matrix extension.");
-
-	if (target_seg < 0 || target_seg > 1)
-		throw std::runtime_error("Permitted values for --target-seg: 0, 1");
-
-	if (masking < 0 || masking > 1)
-		throw std::runtime_error("Permitted values for --masking: 0, 1");
-
-	if (target_indexed)
-		hashed_seeds = true;
-
 	if (check_io) {
 		switch (command) {
 		case Config::makedb:
@@ -570,15 +590,12 @@ Config::Config(int argc, const char **argv, bool check_io)
 				output_file = daa_file;
 			}
 			if (daa_file.length() > 0 || (output_format.size() > 0 && (output_format[0] == "daa" || output_format[0] == "100"))) {
-				if (compression != 0)
+				if (!compression.empty())
 					throw std::runtime_error("Compression is not supported for DAA format.");
 				if (!no_auto_append)
 					auto_append_extension(output_file, ".daa");
 			}
 			break;
-		case Config::view:
-			if (daa_file == "")
-				throw std::runtime_error("Missing parameter: DAA file (--daa/-a)");
 		default:
 			;
 		}
@@ -590,9 +607,6 @@ Config::Config(int argc, const char **argv, bool check_io)
 		}
 	}
 
-	if (hit_cap != 0)
-		throw std::runtime_error("Deprecated parameter: --max-hits/-C.");
-
 	if (debug_log)
 		verbosity = 3;
 	else if (quiet)
@@ -626,18 +640,19 @@ Config::Config(int argc, const char **argv, bool check_io)
 	if (!no_auto_append) {
 		if (command == Config::makedb)
 			auto_append_extension(database, ".dmnd");
-		else
-			auto_append_extension_if_exists(database, ".dmnd");
 		if (command == Config::view)
 			auto_append_extension(daa_file, ".daa");
-		if (compression == 1)
+		if (compression == "1")
 			auto_append_extension(output_file, ".gz");
+		if (compression == "zstd")
+			auto_append_extension(output_file, ".zst");
 	}
 
 	if (verbosity >= 1 || command == regression_test) {
 		ostream &header_out = command == Config::help ? cout : cerr;
 		header_out << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << " (C) Max Planck Society for the Advancement of Science" << endl;
-		header_out << "Documentation, support and updates available at http://www.diamondsearch.org" << endl << endl;
+		header_out << "Documentation, support and updates available at http://www.diamondsearch.org" << endl;
+		header_out << "Please cite: http://dx.doi.org/10.1038/s41592-021-01101-x Nature Methods (2021)" << endl << endl;
 	}
 	log_stream << Const::program_name << " v" << Const::version_string << "." << (unsigned)Const::build_version << endl;
 #ifndef NDEBUG
@@ -669,12 +684,13 @@ Config::Config(int argc, const char **argv, bool check_io)
 	case Config::cluster:
 	case Config::regression_test:
 	case Config::compute_medoids:
+	case Config::LIST_SEEDS:
 		if (frame_shift != 0 && command == Config::blastp)
 			throw std::runtime_error("Frameshift alignments are only supported for translated searches.");
 		if (query_range_culling && frame_shift == 0)
 			throw std::runtime_error("Query range culling is only supported in frameshift alignment mode (option -F).");
 		if (matrix_file == "") {
-			score_matrix = Score_matrix(to_upper_case(matrix), gap_open, gap_extend, frame_shift, stop_match_score, 0, cbs_matrix_scale);
+			score_matrix = ScoreMatrix(to_upper_case(matrix), gap_open, gap_extend, frame_shift, stop_match_score, 0, cbs_matrix_scale);
 		}
 		else {
 			if (gap_open == -1 || gap_extend == -1)
@@ -683,7 +699,7 @@ Config::Config(int argc, const char **argv, bool check_io)
 				throw std::runtime_error("Custom scoring matrices are not supported for the DAA format.");
 			if (comp_based_stats > 1)
 				throw std::runtime_error("This value for --comp-based-stats is not supported when using a custom scoring matrix.");
-			score_matrix = Score_matrix(matrix_file, gap_open, gap_extend, stop_match_score, Score_matrix::Custom());
+			score_matrix = ScoreMatrix(matrix_file, gap_open, gap_extend, stop_match_score, ScoreMatrix::Custom());
 		}
 		if(command == Config::cluster){
 			if(!Workflow::Cluster::ClusterRegistry::has(cluster_algo)){
@@ -697,8 +713,7 @@ Config::Config(int argc, const char **argv, bool check_io)
 			}
 		}
 		message_stream << "Scoring parameters: " << score_matrix << endl;
-		//if (masking == 1 || target_seg)
-			Masking::instance = unique_ptr<Masking>(new Masking(score_matrix));
+		Masking::instance = unique_ptr<Masking>(new Masking(score_matrix));
 	}
 
 	if (command == Config::blastp || command == Config::blastx || command == Config::benchmark || command == Config::model_sim || command == Config::opt
@@ -706,7 +721,6 @@ Config::Config(int argc, const char **argv, bool check_io)
 		if (tmpdir == "")
 			tmpdir = extract_dir(output_file);
 
-		init_cbs();
 		raw_ungapped_xdrop = score_matrix.rawscore(ungapped_xdrop);
 		verbose_stream << "CPU features detected: " << SIMD::features() << endl;
 #if defined(RUNTIME_DISPATCH) && defined(__SSE__)
@@ -714,6 +728,7 @@ Config::Config(int argc, const char **argv, bool check_io)
 #else
 		verbose_stream << "Runtime dispatch: disabled" << endl;
 #endif
+		log_stream << "L3 cache size: " << l3_cache_size() << endl;
 	}
 
 	sensitivity = Sensitivity::DEFAULT;
@@ -724,12 +739,7 @@ Config::Config(int argc, const char **argv, bool check_io)
 	if (mode_very_sensitive) set_sens(Sensitivity::VERY_SENSITIVE);
 	if (mode_ultra_sensitive) set_sens(Sensitivity::ULTRA_SENSITIVE);
 
-	if (algo == Config::query_indexed && (sensitivity == Sensitivity::MID_SENSITIVE || sensitivity >= Sensitivity::VERY_SENSITIVE))
-		throw std::runtime_error("Query-indexed mode is not supported for this sensitivity setting.");
-
-	const set<string> ext_modes = { "", "banded-fast", "banded-slow", "full" };
-	if (ext_modes.find(ext) == ext_modes.end())
-		throw std::runtime_error("Possible values for --ext are: banded-fast, banded-slow, full");
+	algo = from_string<Algo>(algo_str);
 
 	Translator::init(query_gencode);
 
@@ -754,24 +764,16 @@ Config::Config(int argc, const char **argv, bool check_io)
 	else if (query_file.size() > 1)
 		throw std::runtime_error("--query/-q has more than one argument.");
 
-	if ((mmap_target_index || save_target_index) && !target_indexed)
-		throw std::runtime_error("Option requires --target-indexed.");
-
-	if (mmap_target_index && save_target_index)
-		throw std::runtime_error("Options are exclusive.");
-
-	if (target_indexed && lowmem != 1)
+	if (target_indexed && lowmem_ != 1)
 		throw std::runtime_error("--target-indexed requires -c1.");
 
 	/*log_stream << "sizeof(hit)=" << sizeof(hit) << " sizeof(packed_uint40_t)=" << sizeof(packed_uint40_t)
 		<< " sizeof(sorted_list::entry)=" << sizeof(sorted_list::entry) << endl;*/
 
 	if (swipe_all) {
-		algo = double_indexed;
+		algo = Algo::DOUBLE_INDEXED;
 	}
 
-	use_lazy_dict = false;
-
 	if (query_range_culling && taxon_k != 0)
 		throw std::runtime_error("--taxon-k is not supported for --range-culling mode.");
 
