@@ -25,9 +25,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <vector>
 #include <stdint.h>
 #include <map>
+#include "../util/enum.h"
+#include "../util/options/option.h"
+#include "value.h"
 
 enum class Sensitivity { FAST = 0, DEFAULT = 1, MID_SENSITIVE = 2, SENSITIVE = 3, MORE_SENSITIVE = 4, VERY_SENSITIVE = 5, ULTRA_SENSITIVE = 6 };
-enum class TracebackMode { NONE = 0, SCORE_ONLY = 1, STAT = 2, VECTOR = 3, SCORE_BUFFER = 4 };
+enum class Compressor;
+
+template<> struct EnumTraits<Sensitivity> {
+	static const EMap<Sensitivity> to_string;
+	static const SEMap<Sensitivity> from_string;
+};
 
 struct Config
 {
@@ -40,18 +48,16 @@ struct Config
 	string	database;
 	string_vector query_file;
 	unsigned	merge_seq_treshold;
-	unsigned	hit_cap;
 	unsigned shapes;
-	unsigned	index_mode;
 	size_t	max_alignments;
 	string	match_file1;
 	string	match_file2;
 	int		padding;
 	unsigned	output_threads;
-	unsigned compression;
-	unsigned		lowmem;
+	string compression;
+	unsigned		lowmem_;
 	double	chunk_size;
-	unsigned min_identities;
+	unsigned min_identities_;
 	unsigned min_identities2;
 	double ungapped_xdrop;
 	int		raw_ungapped_xdrop;
@@ -85,19 +91,13 @@ struct Config
 	unsigned fetch_size;
 	uint64_t	db_size;
 	double	query_cover;
-
 	bool		mode_sensitive;
 	unsigned	verbosity;
 	bool no_auto_append;
-	unsigned local_align_mode;
-	bool extend_all;
-	bool slow_search;
 	string_vector seq_no;
 	double rank_factor;
 	double rank_ratio;
-	bool ht_mode;
-	bool old_freq;
-	double freq_sd;
+	double freq_sd_;
 	unsigned target_fetch_size;
 	bool mode_more_sensitive;
 	string matrix_file;
@@ -117,26 +117,23 @@ struct Config
 	bool no_self_hits;
 	unsigned id_left, id_right, id_n;
 	int bmatch, bmismatch, bcutoff;
-	unsigned query_bins;
+	unsigned query_bins_;
 	uint64_t n_ants;
 	double rho;
 	double p_best;
 	double d_exp, d_new;
 	double score_estimate_factor;
 	int diag_min_estimate;
-	string qfilt, sfilt;
 	double path_cutoff;
 	bool use_smith_waterman;
 	string prot_accession2taxid;
 	int superblock;
 	unsigned max_cells;
-	int masking;
+	string masking;
 	bool log_query;
 	bool log_subject;
 	unsigned threads_align;
 	double score_ratio;
-	bool small_query;
-	bool hashed_seeds;
 	string nodesdmp;
 	bool sallseqid;
 	string query_strands;
@@ -155,7 +152,6 @@ struct Config
 	bool sort_join;
 	bool simple_freq;
 	double freq_treshold;
-	bool use_lazy_dict;
 	string aligned_file;
 	bool use_dataset_field;
 	bool store_query_quality;
@@ -199,10 +195,10 @@ struct Config
 	int ungapped_window;
 	int gapped_filter_diag_score;
 	double gapped_filter_diag_bit_score;
-	double gapped_filter_evalue;
+	double gapped_filter_evalue_;
 	int gapped_filter_window;
 	bool output_hits;
-	double ungapped_evalue;
+	double ungapped_evalue_;
 	bool no_logfile;
 	bool no_heartbeat;
 	int band_bin;
@@ -210,13 +206,13 @@ struct Config
 	size_t file_buffer_size;
 	bool self;
 	size_t trace_pt_fetch_size;
-	uint32_t tile_size;
+	size_t tile_size;
 	double short_query_ungapped_bitscore;
 	int short_query_max_len;
 	double gapped_filter_evalue1;
 	size_t ext_chunk_size;
 	double ext_min_yield;
-	string ext;
+	string ext_;
 	int full_sw_len;
 	double relaxed_evalue_factor;
 	string type;
@@ -246,7 +242,6 @@ struct Config
 	int family_cap;
 	int cbs_matrix_scale;
 	size_t query_count;
-	int target_seg;
 	double cbs_err_tolerance;
 	int cbs_it_limit;
 	double query_match_distance_threshold;
@@ -254,32 +249,49 @@ struct Config
 	bool hash_join_swap;
 	bool target_indexed;
 	size_t deque_bucket_size;
-	bool mmap_target_index;
-	bool save_target_index;
 	bool mode_fast;
 	double log_evalue_scale;
-	double ungapped_evalue_short;
-	size_t max_swipe_dp;
+	double ungapped_evalue_short_;
+	int64_t max_swipe_dp;
+	std::string seqidlist;
+	bool skip_missing_seqids;
+	Option<string_vector> iterate;
+	bool ignore_warnings;
+	bool short_seqids;
+	bool no_reextend;
+	double seed_cut_;
+	bool no_reorder;
+	string file1;
+	string file2;
+	size_t key2;
+	string motif_mask_file;
+	string motif_masking;
+	bool freq_masking;
+	Loc max_motif_len;
+	double chaining_stacked_hsp_ratio;
 
 	Sensitivity sensitivity;
-	TracebackMode traceback_mode;
 
 	bool multiprocessing;
 	bool mp_init;
+	bool mp_recover;
+	int mp_query_chunk;
 
 	enum {
 		makedb = 0, blastp = 1, blastx = 2, view = 3, help = 4, version = 5, getseq = 6, benchmark = 7, random_seqs = 8, compare = 9, sort = 10, roc = 11, db_stat = 12, model_sim = 13,
 		match_file_stat = 14, model_seqs = 15, opt = 16, mask = 17, fastq2fasta = 18, dbinfo = 19, test_extra = 20, test_io = 21, db_annot_stats = 22, read_sim = 23, info = 24, seed_stat = 25,
 		smith_waterman = 26, cluster = 27, translate = 28, filter_blasttab = 29, show_cbs = 30, simulate_seqs = 31, split = 32, upgma = 33, upgma_mc = 34, regression_test = 35,
-		reverse_seqs = 36, compute_medoids = 37, mutate = 38, merge_tsv = 39, rocid = 40
+		reverse_seqs = 36, compute_medoids = 37, mutate = 38, merge_tsv = 39, rocid = 40, makeidx = 41, find_shapes, prep_blast_db, composition, JOIN, HASH_SEQS, LIST_SEEDS
 	};
 	unsigned	command;
 
-	enum { double_indexed = 0, query_indexed = 1, subject_indexed = 2 };
-	int algo;
+	enum class Algo { AUTO = -1, DOUBLE_INDEXED = 0, QUERY_INDEXED = 1, CTG_SEED };
+	Algo algo;
 
 	string cluster_algo;
-	string	cluster_similarity;
+	string cluster_similarity;
+	string cluster_graph_file;
+	bool cluster_restart;
 
 	size_t max_size_set;
 	bool external;
@@ -289,19 +301,8 @@ struct Config
 	double cluster_mcl_sparsity_switch;
 	uint32_t cluster_mcl_chunk_size;
 	uint32_t cluster_mcl_max_iter;
-	string	cluster_mcl_graph_file;
-	bool	cluster_mcl_restart;
-
-	
-	std::map<std::string, Sensitivity> sens_map{
-		{"fast", Sensitivity::FAST },
-		{"default", Sensitivity::DEFAULT},
-		{"sensitive", Sensitivity::SENSITIVE},
-		{"mid-sensitive", Sensitivity::MID_SENSITIVE},
-		{"more-sensitive", Sensitivity::MORE_SENSITIVE},
-		{"very-sensitive", Sensitivity::VERY_SENSITIVE},
-		{"ultra-sensitive", Sensitivity::ULTRA_SENSITIVE}
-	};
+	bool cluster_mcl_stats;
+	bool cluster_mcl_nonsymmetric;
 
 	enum { query_parallel = 0, target_parallel = 1 };
 	unsigned load_balancing;
@@ -312,7 +313,7 @@ struct Config
 	inline unsigned get_run_len(unsigned length)
 	{
 		if (run_len == 0) {
-			if (length < 30)
+			if (length < 30 || frame_shift != 0)
 				return 1;
 			else if (length < 100)
 				return 20;
@@ -335,9 +336,12 @@ struct Config
 	std::string single_query_file() const;
 
 	bool mem_buffered() const { return tmpdir == "/dev/shm"; }
+	Compressor compressor() const;
 
   	template<typename _t>
 	static void set_option(_t& option, _t value, _t def = 0) { if (option == def) option = value; }
+	template<typename _t>
+	static void set_option(_t& option, _t value, _t def, _t alt) { if (value != def) option = value; else option = alt; }
 };
 
 void print_warnings();
@@ -347,3 +351,8 @@ template<typename _t>
 _t top_cutoff_score(_t top_score) {
 	return _t((1.0 - config.toppercent / 100.0)*top_score);
 }
+
+template<> struct EnumTraits<Config::Algo> {
+	static const EMap<Config::Algo> to_string;
+	static const SEMap<Config::Algo> from_string;
+};
