@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
                         Eberhard Karls Universitaet Tuebingen
 						
@@ -25,13 +25,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <limits>
 #include <list>
 #include "sequence.h"
-#include "../util/util.h"
-#include "../util/async_buffer.h"
 #include "packed_loc.h"
-#include "../util/system.h"
 #include "value.h"
 #include "packed_transcript.h"
-#include "../stats/score_matrix.h"
 #include "translated_position.h"
 #include "diagonal_segment.h"
 
@@ -44,10 +40,15 @@ inline interval normalized_range(unsigned pos, int len, Strand strand)
 
 struct IntermediateRecord;
 
+namespace Stats {
+struct TargetMatrix;
+}
+
 struct Hsp
 {
 
-	Hsp() :
+	Hsp(const bool backtraced = false) :
+		backtraced(backtraced),
 		score(0),
 		frame(0),
 		length(0),
@@ -58,10 +59,12 @@ struct Hsp
 		gaps(0),
 		swipe_target(0),
 		d_begin(0),
-		d_end(0)
+		d_end(0),
+		matrix(nullptr)
 	{}
 
-	Hsp(int score, int swipe_target = 0) :
+	Hsp(const bool backtraced, int score, int swipe_target = 0) :
+		backtraced(backtraced),
 		score(score),
 		frame(0),
 		length(0),
@@ -72,7 +75,8 @@ struct Hsp
 		gaps(0),
 		swipe_target(swipe_target),
 		d_begin(0),
-		d_end(0)
+		d_end(0),
+		matrix(nullptr)
 	{}
 
 	Hsp(const IntermediateRecord &r, unsigned query_source_len);
@@ -176,6 +180,10 @@ struct Hsp
 		return score > rhs.score || (score == rhs.score && query_source_range.begin_ < rhs.query_source_range.begin_);
 	}
 
+	static bool cmp_evalue(const Hsp& a, const Hsp& b) {
+		return a.evalue < b.evalue || (a.evalue == b.evalue && a.score > b.score);
+	}
+
 	double id_percent() const
 	{
 		return (double)identities * 100.0 / (double)length;
@@ -210,10 +218,10 @@ struct Hsp
 	bool is_enveloped_by(const Hsp &hsp, double p) const;
 	bool is_enveloped_by(std::list<Hsp>::const_iterator begin, std::list<Hsp>::const_iterator end, double p) const;
 	bool is_weakly_enveloped_by(std::list<Hsp>::const_iterator begin, std::list<Hsp>::const_iterator end, int cutoff) const;
-	void push_back(const DiagonalSegment &d, const TranslatedSequence &query, const sequence &subject, bool reversed);
+	void push_back(const DiagonalSegment &d, const TranslatedSequence &query, const Sequence& subject, bool reversed);
 	void push_match(Letter q, Letter s, bool positive);
 	void push_gap(Edit_operation op, int length, const Letter *subject);
-	void splice(const DiagonalSegment &d0, const DiagonalSegment &d1, const TranslatedSequence &query, const sequence &subject, bool reversed);
+	void splice(const DiagonalSegment &d0, const DiagonalSegment &d1, const TranslatedSequence &query, const Sequence& subject, bool reversed);
 	void set_begin(const DiagonalSegment &d, int dna_len);
 	void set_end(const DiagonalSegment &d, int dna_len);
 	void set_begin(int i, int j, Frame frame, int dna_len);
@@ -222,44 +230,48 @@ struct Hsp
 
 	bool is_weakly_enveloped(const Hsp &j) const;
 	std::pair<int, int> diagonal_bounds() const;
+	bool backtraced;
 	int score, frame, length, identities, mismatches, positives, gap_openings, gaps, swipe_target, d_begin, d_end;
-	interval query_source_range, query_range, subject_range, seed_hit_range;
-	double evalue;
+	interval query_source_range, query_range, subject_range;
+	double evalue, bit_score;
+	Sequence target_seq;
+	const Stats::TargetMatrix* matrix;
 	Packed_transcript transcript;
 };
 
-struct Hsp_context
+struct HspContext
 {
-	Hsp_context(
+	HspContext(
 		Hsp& hsp,
 		unsigned query_id,
 		const TranslatedSequence &query,
-		const char *query_name,
-		unsigned subject_id,
-		unsigned orig_subject_id,
-		const char *subject_name,
+		const char *query_title,
+		unsigned subject_oid,
 		unsigned subject_len,
+		const char* subject_title,
 		unsigned hit_num,
 		unsigned hsp_num,
-		const sequence &subject_seq,
-		int ungapped_score = 0
-	) :
+		const Sequence &subject_seq,
+		int ungapped_score = 0,
+		const double query_self_aln_score = 0.0,
+		const double target_self_aln_score = 0.0) :
 		query(query),
-		query_name(query_name),
-		subject_name(subject_name),
+		query_title(query_title),
+		target_title(subject_title),
 		query_id(query_id),
-		subject_id(subject_id),
-		orig_subject_id(orig_subject_id),
-		subject_len(subject_len),
+		subject_oid(subject_oid),
+		subject_len(subject_len),		
 		hit_num(hit_num),
 		hsp_num(hsp_num),
 		ungapped_score(ungapped_score),
+		query_self_aln_score(query_self_aln_score),
+		target_self_aln_score(target_self_aln_score),
 		subject_seq(subject_seq),
 		hsp_(hsp)		
 	{}
 	struct Iterator : public Hsp::Iterator
 	{
-		Iterator(const Hsp_context &parent) :
+		Iterator(const HspContext &parent) :
 			Hsp::Iterator(parent.hsp_),
 			parent_(parent)
 		{ }
@@ -301,23 +313,19 @@ struct Hsp_context
 				return value_traits.alphabet[(long)subject()];
 			}
 		}
-		char midline_char() const
+		char midline_char(int score) const
 		{
 			switch (op()) {
 			case op_match:
 				return value_traits.alphabet[(long)query()];
 			case op_substitution:
-				return score() > 0 ? '+' : ' ';
+				return score > 0 ? '+' : ' ';
 			default:
 				return ' ';
 			}
 		}
-		int score() const
-		{
-			return score_matrix(query(), subject());
-		}
 	private:
-		const Hsp_context &parent_;
+		const HspContext &parent_;
 	};
 	Iterator begin() const
 	{
@@ -333,9 +341,8 @@ struct Hsp_context
 	{
 		return hsp_.evalue;
 	}
-	double bit_score() const
-	{
-		return score_matrix.bitscore(score());
+	double bit_score() const {
+		return hsp_.bit_score;
 	}
 	unsigned frame() const
 	{ return hsp_.frame; }
@@ -363,13 +370,16 @@ struct Hsp_context
 	{ return hsp_.blast_query_frame(); }
 	Packed_transcript transcript() const
 	{ return hsp_.transcript; }
-	Hsp_context& parse();
+	HspContext& parse();
 
 	const TranslatedSequence query;
-	const char *query_name, *subject_name;
-	const unsigned query_id, subject_id, orig_subject_id, subject_len, hit_num, hsp_num;
+	const char* query_title, *target_title;
+	const unsigned query_id, subject_oid;
+	const Loc subject_len;
+	const unsigned hit_num, hsp_num;
 	int ungapped_score;
-	const sequence subject_seq;
+	const double query_self_aln_score, target_self_aln_score;
+	const Sequence subject_seq;
 private:	
 	Hsp &hsp_;
 };
