@@ -60,8 +60,7 @@ struct packed_uint40_t
 	{ return (uint64_t)(x) - (uint64_t)(y); }
 } PACKED_ATTRIBUTE ;
 
-typedef packed_uint40_t Packed_loc;
-typedef size_t Loc;
+typedef packed_uint40_t PackedLoc;
 
 #pragma pack()
 
