@@ -22,41 +22,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #pragma once
 #include <vector>
+#include <array>
 #include <string>
 #include <string.h>
 #include "value.h"
-#include "../util/util.h"
 #include "sequence.h"
 #include "translate.h"
 
 struct Reduction
 {
 
-	Reduction(const char *definition_string)
-	{
-		memset(map_, 0, sizeof(map_));
-		memset(map8_, 0, sizeof(map8_));
-		memset(map8b_, 0, sizeof(map8b_));
-		map_[(long)MASK_LETTER] = MASK_LETTER;
-		map_[(long)STOP_LETTER] = MASK_LETTER;
-		const vector<string> tokens(tokenize(definition_string, " "));
-		size_ = (unsigned)tokens.size();
-		bit_size_exact_ = log(size_) / log(2);
-		bit_size_ = (uint64_t)ceil(bit_size_exact_);
-		for (unsigned i = 0; i<size_; ++i)
-			for (unsigned j = 0; j<tokens[i].length(); ++j) {
-				const char ch = tokens[i][j];
-				map_[(long)value_traits.from_char(ch)] = i;
-				map8_[(long)value_traits.from_char(ch)] = i;
-				map8b_[(long)value_traits.from_char(ch)] = i;
-			}
-		map8_[(long)MASK_LETTER] = (Letter)size_;
-		map8_[(long)STOP_LETTER] = (Letter)size_;
-		map8_[(long)DELIMITER_LETTER] = (Letter)size_;
-		map8b_[(long)MASK_LETTER] = (Letter)(size_ + 1);
-		map8b_[(long)STOP_LETTER] = (Letter)(size_ + 1);
-		map8b_[(long)DELIMITER_LETTER] = (Letter)(size_ + 1);
-	}
+	Reduction(const char* definition_string);
 
 	unsigned size() const
 	{
@@ -104,14 +80,20 @@ struct Reduction
 		return os;
 	}
 
-	static void reduce_seq(const sequence &seq, vector<Letter> &dst)
+	static void reduce_seq(const Sequence &seq, vector<Letter> &dst)
 	{
 		dst.clear();
 		dst.resize(seq.length());
-		for (unsigned i = 0; i < seq.length(); ++i)
+		for (Loc i = 0; i < seq.length(); ++i)
 			dst[i] = reduction(seq[i]);
 	}
 
+	double freq(unsigned bucket) const {
+		return freq_[bucket];
+	}
+
+	std::string decode_seed(const uint64_t seed, const size_t len) const;
+
 	static Reduction reduction;
 
 private:
@@ -121,5 +103,6 @@ private:
 	unsigned size_;
 	uint64_t bit_size_;
 	double bit_size_exact_;
+	std::array<double, 20> freq_;
 
 };
