@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
 						
 Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
@@ -23,10 +23,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "shape.h"
 #include "sequence.h"
 #include "../util/hash_function.h"
+#include "../util/algo/MurmurHash3.h"
 
-struct Seed_iterator
+struct SeedIterator
 {
-	Seed_iterator(vector<Letter> &seq, const Shape &sh):
+	SeedIterator(vector<Letter> &seq, const Shape &sh):
 		ptr_ (seq.data()),
 		end_ (ptr_ + seq.size() - sh.length_ + 1)
 	{}
@@ -38,24 +39,25 @@ struct Seed_iterator
 	{
 		return sh.set_seed_reduced(seed, ptr_++);
 	}
+	SeedIterator& operator++() {
+		ptr_++;
+		return *this;
+	}
 private:
 	const Letter *ptr_, *end_;
 };
 
-template<uint64_t _b>
-struct Hashed_seed_iterator
+template<uint64_t B>
+struct HashedSeedIterator
 {
-	Hashed_seed_iterator(const sequence &seq, const Shape &sh):
+	HashedSeedIterator(const Sequence &seq, const Shape &sh):
 		ptr_(seq.data()),
 		end_(ptr_ + seq.length()),
 		last_(0)
 	{
-		for (uint64_t i = 0; i < sh.length_ - 1; ++i)
-#ifdef SEQ_MASK
-			last_ = (last_ << _b) | Reduction::reduction(letter_mask(*(ptr_++)));
-#else
-			last_ = (last_ << _b) | Reduction::reduction(*(ptr_++));
-#endif
+		for (int i = 0; (i < sh.length_ - 1) && ptr_ < end_; ++i) {
+			last_ = (last_ << B) | Reduction::reduction(letter_mask(*(ptr_++)));
+		}
 	}
 	bool good() const
 	{
@@ -63,12 +65,8 @@ struct Hashed_seed_iterator
 	}
 	bool get(uint64_t &seed, uint64_t mask)
 	{
-		last_ <<= _b;
-#ifdef SEQ_MASK
-		const Letter l = *(ptr_++) & LETTER_MASK;
-#else
-		const Letter l = *(ptr_++);
-#endif
+		last_ <<= B;
+		const Letter l = letter_mask(*(ptr_++));
 		if (!is_amino_acid(l))
 			return false;
 		last_ |= Reduction::reduction(l);
@@ -80,20 +78,18 @@ private:
 	uint64_t last_;
 };
 
-template<uint64_t _l, uint64_t _b>
-struct Contiguous_seed_iterator
+struct FilterMaskedSeeds { };
+
+template<int L, uint64_t B, typename Filter>
+struct ContiguousSeedIterator
 {
-	Contiguous_seed_iterator(const sequence &seq) :
+	ContiguousSeedIterator(const Sequence &seq) :
 		ptr_(seq.data()),
 		end_(ptr_ + seq.length()),
 		last_(0)
 	{
-		for (uint64_t i = 0; i < _l - 1; ++i)
-#ifdef SEQ_MASK
-			last_ = (last_ << _b) | Reduction::reduction(letter_mask(*(ptr_++)));
-#else
-			last_ = (last_ << _b) | Reduction::reduction(*(ptr_++));
-#endif
+		for (int i = 0; i < L - 1; ++i)
+			last_ = (last_ << B) | Reduction::reduction(letter_mask(*(ptr_++)));
 	}
 	bool good() const
 	{
@@ -101,24 +97,68 @@ struct Contiguous_seed_iterator
 	}
 	bool get(uint64_t &seed)
 	{
-		last_ <<= _b;
-		last_ &= (1 << (_b*_l)) - 1;
-#ifdef SEQ_MASK
-		const Letter l = *(ptr_++) & LETTER_MASK;
-#else
-		const Letter l = *(ptr_++);
-#endif
-		if (l == value_traits.mask_char)
-			return false;
-		last_ |= Reduction::reduction(l);
-		seed = last_;
-		return true;
+		for (;;) {
+			last_ <<= B;
+			last_ &= (uint64_t(1) << (B*L)) - 1;
+			const Letter l = letter_mask(*(ptr_++));
+			last_ |= Reduction::reduction(l);
+			seed = last_;
+			return true;
+		}
 	}
-	static uint64_t length()
+	static int length()
 	{
-		return _l;
+		return L;
 	}
 private:
 	const Letter *ptr_, *end_;
 	uint64_t last_;
+	unsigned mask_;
 };
+
+template<int L, uint64_t B>
+struct ContiguousSeedIterator<L, B, FilterMaskedSeeds>
+{
+	ContiguousSeedIterator(const Sequence &seq) :
+		ptr_(seq.data()),
+		end_(ptr_ + seq.length()),
+		last_(0),
+		mask_(0)
+	{
+		for (int i = 0; i < L - 1; ++i) {
+			const Letter l = letter_mask(*(ptr_++));
+			last_ = (last_ << B) | Reduction::reduction(l);
+			if (!is_amino_acid(l))
+				mask_ |= 1;
+			mask_ <<= 1;
+		}
+	}
+	bool good() const
+	{
+		return ptr_ < end_;
+	}
+	bool get(uint64_t &seed)
+	{
+		for (;;) {
+			last_ <<= B;
+			last_ &= (uint64_t(1) << (B*L)) - 1;
+			mask_ <<= 1;
+			mask_ &= (1 << L) - 1;
+			const Letter l = letter_mask(*(ptr_++));
+			const unsigned r = Reduction::reduction(l);
+			last_ |= r;
+			seed = last_;
+			if (!is_amino_acid(l))
+				mask_ |= 1;
+			return mask_ == 0;
+		}
+	}
+	static int length()
+	{
+		return L;
+	}
+private:
+	const Letter *ptr_, *end_;
+	uint64_t last_;
+	unsigned mask_;
+};
\ No newline at end of file
