@@ -27,7 +27,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "seed.h"
 #include "../stats/score_matrix.h"
 #include "reduction.h"
-#include "../util/util.h"
 #include "config.h"
 #include "translate.h"
 
@@ -39,16 +38,15 @@ struct Shape
 		weight_ (0),
 		d_ (0),
 		mask_ (0),
-		rev_mask_ (0),
-		id_ (0)
+		rev_mask_ (0)
 	{ memset(positions_, 0, sizeof(uint32_t)*Const::max_seed_weight); }
 
-	Shape(const char *code, unsigned id):
-		weight_ (0),
-		mask_ (0),
-		rev_mask_ (0),
-		id_ (id),
+	Shape(const char* code, unsigned id) :
+		weight_(0),
+		mask_(0),
+		rev_mask_(0),
 		long_mask_(0)
+		//long_mask_sse_(_mm_setzero_si128())
 	{
 		assert(strlen(code) <= 32);
 		memset(positions_, 0, sizeof(uint32_t)*Const::max_seed_weight);
@@ -57,6 +55,7 @@ struct Shape
 		for(;i<strlen(code);++i) {
 			rev_mask_ <<= 1;
 			long_mask_ <<= b;
+			//long_mask_sse_ = _mm_slli_si128(long_mask_sse_, 1);
 			if(code[i] == '1') {
 				assert(weight_ < Const::max_seed_weight);
 				positions_[weight_] = i;
@@ -64,6 +63,7 @@ struct Shape
 				mask_ |= 1 << i;
 				rev_mask_ |= 1;
 				long_mask_ |= (1 << b) - 1;
+				//long_mask_sse_ = _mm_insert_epi8(long_mask_sse_, 0xff, 0);
 			}
 		}
 		length_ = i;
@@ -76,7 +76,7 @@ struct Shape
 #ifdef FREQUENCY_MASKING
 		double f = 0;
 #endif
-		for (unsigned i = 0; i < weight_; ++i) {
+		for (int i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
 #ifdef SEQ_MASK
 			l &= LETTER_MASK;
@@ -100,12 +100,12 @@ struct Shape
 	{
 		s = 0;
 		const uint64_t b = Reduction::reduction.bit_size();
-		for (unsigned i = 0; i < weight_; ++i) {
+		for (int i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
 #ifdef SEQ_MASK
 			l &= LETTER_MASK;
 #endif
-			if (l == value_traits.mask_char || l == sequence::DELIMITER || l == STOP_LETTER)
+			if (l == value_traits.mask_char || l == Sequence::DELIMITER || l == STOP_LETTER)
 				return false;
 			unsigned r = Reduction::reduction(l);
 			s <<= b;
@@ -117,12 +117,12 @@ struct Shape
 	inline bool set_seed_reduced(Packed_seed &s, const Letter *seq) const
 	{
 		s = 0;
-		for (unsigned i = 0; i < weight_; ++i) {
+		for (int i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
 #ifdef SEQ_MASK
 			l &= LETTER_MASK;
 #endif
-			if (l == value_traits.mask_char)
+			if (l == MASK_LETTER)
 				return false;
 			s *= Reduction::reduction.size();
 			s += uint64_t(l);
@@ -132,7 +132,7 @@ struct Shape
 
 	inline bool set_seed(Seed &s, const Letter *seq) const
 	{
-		for (unsigned i = 0; i < weight_; ++i) {
+		for (int i = 0; i < weight_; ++i) {
 			Letter l = seq[positions_[i]];
 #ifdef SEQ_MASK
 			l &= LETTER_MASK;
@@ -146,7 +146,7 @@ struct Shape
 	
 	friend std::ostream& operator<<(std::ostream&s, const Shape &sh)
 	{
-		for (unsigned i = 0; i < sh.length_; ++i)
+		for (int i = 0; i < sh.length_; ++i)
 			s << ((sh.mask_ & (1 << i)) ? '1' : '0');
 		return s;
 	}
@@ -161,7 +161,9 @@ struct Shape
 		return long_mask_;
 	}
 
-	uint32_t length_, weight_, positions_[Const::max_seed_weight], d_, mask_, rev_mask_, id_;
+	int32_t length_, weight_, positions_[Const::max_seed_weight];
+	uint32_t d_, mask_, rev_mask_;
 	uint64_t long_mask_;
+	//__m128i long_mask_sse_;
 
 };
