@@ -16,15 +16,14 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef CHAINING_H_
-#define CHAINING_H_
-
+#pragma once
 #include <utility>
 #include <list>
+#include <stddef.h>
 #include "../dp/hsp_traits.h"
 #include "../stats/hauser_correction.h"
 
-std::pair<int, std::list<Hsp_traits>> greedy_align(sequence query, sequence subject, std::vector<Diagonal_segment>::const_iterator begin, std::vector<Diagonal_segment>::const_iterator end, bool log, unsigned frame);
+std::pair<int, std::list<Hsp_traits>> greedy_align(Sequence query, Sequence subject, std::vector<Diagonal_segment>::const_iterator begin, std::vector<Diagonal_segment>::const_iterator end, bool log, unsigned frame);
 
 struct Diagonal_node : public Diagonal_segment
 {
@@ -157,13 +156,19 @@ struct Diag_graph
 		if (edges.empty())
 			return edges.end();
 		int max_score = d.score;
-		vector<Edge>::const_iterator max_edge = edges.end();
-		for (vector<Edge>::const_iterator i = edges.begin() + d.link_idx - 1; i >= edges.begin() && i->node_in == node; --i)
+		ptrdiff_t max_i = -1;
+		for (ptrdiff_t i = d.link_idx - 1; i >= 0 && edges[i].node_in == node; --i)
+			if (edges[i].j < j && edges[i].prefix_score > max_score) {
+				max_i = i;
+				max_score = edges[i].prefix_score;
+			}
+
+		/*for (vector<Edge>::const_iterator i = edges.begin() + d.link_idx - 1; i >= edges.begin() && i->node_in == node; --i)
 			if (i->j < j && i->prefix_score > max_score) {
 				max_edge = i;
 				max_score = i->prefix_score;
-			}
-		return max_edge;
+			}*/
+		return max_i >= 0 ? edges.begin() + max_i : edges.end();
 	}
 
 	int prefix_score(size_t node, int j, int &path_max, int &path_min) const
@@ -184,13 +189,11 @@ struct Diag_graph
 		return nodes[k];
 	}
 
-	void print(sequence query, sequence subject) const;
+	void print(Sequence query, Sequence subject) const;
 	size_t top_node() const;
 
 	vector<Diagonal_node> nodes;
 	vector<Edge> edges;
 };
 
-void smith_waterman(sequence q, sequence s, const Diag_graph &diags);
-
-#endif
\ No newline at end of file
+void smith_waterman(Sequence q, Sequence s, const Diag_graph &diags);
\ No newline at end of file
