@@ -31,30 +31,47 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../output/output_format.h"
 #include "../dp/hsp_traits.h"
 #include "chaining.h"
+#include "../util/util.h"
 
 using std::endl;
 using std::cout;
 using std::map;
 using std::list;
 using std::set;
+using std::max;
+using std::min;
 
 bool disjoint(list<Hsp_traits>::const_iterator begin, list<Hsp_traits>::const_iterator end, const Hsp_traits &t, int cutoff)
 {
-	for (; begin != end; ++begin)
-		if (begin->partial_score(t) < cutoff || !begin->collinear(t))
+	for (; begin != end; ++begin) {
+		const double ot = t.subject_range.overlap_factor(begin->subject_range),
+			oq = t.query_range.overlap_factor(begin->query_range);
+		if ((1.0 - min(ot, oq)) * t.score / begin->score >= config.chaining_stacked_hsp_ratio)
+			continue;
+		if ((1.0 - max(ot, oq)) * t.score < cutoff)
+			return false;
+		//if (begin->partial_score(t) < cutoff || !begin->collinear(t))
 		//if (!begin->disjoint(t) || !begin->collinear(t))
 		//if (!begin->rel_disjoint(t))
-			return false;
+		//	return false;
+	}
 	return true;
 }
 
 bool disjoint(list<Hsp_traits>::const_iterator begin, list<Hsp_traits>::const_iterator end, const Diagonal_segment &d, int cutoff)
 {
-	for (; begin != end; ++begin)
-		if (begin->partial_score(d) < cutoff || !begin->collinear(d))
+	for (; begin != end; ++begin) {
+		const double ot = d.subject_range().overlap_factor(begin->subject_range),
+			oq = d.query_range().overlap_factor(begin->query_range);
+		if ((1.0 - min(ot, oq)) * d.score / begin->score >= config.chaining_stacked_hsp_ratio)
+			continue;
+		if ((1.0 - max(ot, oq)) * d.score < cutoff)
+			return false;
+		//if (begin->partial_score(d) < cutoff || !begin->collinear(d))
 		//if (!begin->disjoint(d) || !begin->collinear(d))
 		//if (!begin->rel_disjoint(d))
-			return false;
+		//return false;
+	}
 	return true;
 }
 
@@ -82,13 +99,13 @@ void Diag_graph::load(vector<Diagonal_segment>::const_iterator begin, vector<Dia
 	}
 }
 
-void Diag_graph::print(sequence query, sequence subject) const
+void Diag_graph::print(Sequence query, Sequence subject) const
 {
 	for (int k = 0; k < (int)nodes.size(); ++k) {
 		const Diagonal_segment &d = nodes[k];
 		cout << "Diag n=" << k << " i=" << d.i << " j=" << d.j << " d=" << d.diag() << " score=" << d.score << " len=" << d.len << endl;
-		cout << sequence(query, d.i, d.query_last()) << endl;
-		cout << sequence(subject, d.j, d.subject_last()) << endl;
+		cout << Sequence(query, d.i, d.query_last()) << endl;
+		cout << Sequence(subject, d.j, d.subject_last()) << endl;
 	}
 }
 
@@ -159,7 +176,7 @@ struct Link
 	}
 };
 
-int score_range(sequence query, sequence subject, int i, int j, int j_end)
+int score_range(Sequence query, Sequence subject, int i, int j, int j_end)
 {
 	int score = 0;
 	while (j < j_end) {
@@ -170,7 +187,7 @@ int score_range(sequence query, sequence subject, int i, int j, int j_end)
 	return score;
 }
 
-int get_hgap_link(const Diagonal_segment &d1, const Diagonal_segment &d2, sequence query, sequence subject, Link &l, int padding)
+int get_hgap_link(const Diagonal_segment &d1, const Diagonal_segment &d2, Sequence query, Sequence subject, Link &l, int padding)
 {
 	const int d = d1.diag() - d2.diag(),
 		j2_end = std::min(std::max((int)d2.j, d1.subject_last() + d + 1 + padding), d2.subject_last());
@@ -221,14 +238,14 @@ int get_hgap_link(const Diagonal_segment &d1, const Diagonal_segment &d2, sequen
 	return max_score;
 }
 
-int get_vgap_link(const Diagonal_segment &d1, const Diagonal_segment &d2, sequence query, sequence subject, Link &l, int padding)
+int get_vgap_link(const Diagonal_segment &d1, const Diagonal_segment &d2, Sequence query, Sequence subject, Link &l, int padding)
 {
 	int s = get_hgap_link(d1.transpose(), d2.transpose(), subject, query, l, padding);
 	l.transpose();
 	return s;
 }
 
-int get_link(const Diagonal_segment &d1, const Diagonal_segment &d2, sequence query, sequence subject, Link &l, int padding)
+int get_link(const Diagonal_segment &d1, const Diagonal_segment &d2, Sequence query, Sequence subject, Link &l, int padding)
 {
 	if (d1.diag() < d2.diag())
 		return get_vgap_link(d1, d2, query, subject, l, padding);
@@ -495,7 +512,7 @@ struct Greedy_aligner2
 		unsigned next;
 		int max_score = 0, max_j = (int)subject.length();
 		do {
-			Hsp *hsp = log ? new Hsp : 0;
+			Hsp *hsp = log ? new Hsp(true) : 0;
 			Hsp_traits t(frame);
 			next = std::numeric_limits<unsigned>::max();
 			backtrace(top_node, hsp, t, max_shift, next, max_j);
@@ -596,7 +613,7 @@ struct Greedy_aligner2
 		return run(hsps, ts, 0.1, 19, band);
 	}
 
-	Greedy_aligner2(const sequence &query, const sequence &subject, bool log, unsigned frame) :
+	Greedy_aligner2(const Sequence &query, const Sequence &subject, bool log, unsigned frame) :
 		query(query),
 		subject(subject),
 		//query_bc(query_bc),
@@ -605,7 +622,7 @@ struct Greedy_aligner2
 	{
 	}
 
-	const sequence query, subject;
+	const Sequence query, subject;
 	//const Bias_correction &query_bc;
 	const bool log;
 	const unsigned frame;
@@ -617,7 +634,7 @@ struct Greedy_aligner2
 thread_local Diag_graph Greedy_aligner2::diags;
 thread_local map<int, unsigned> Greedy_aligner2::window;
 
-std::pair<int, list<Hsp_traits>> greedy_align(sequence query, sequence subject, vector<Diagonal_segment>::const_iterator begin, vector<Diagonal_segment>::const_iterator end, bool log, unsigned frame)
+std::pair<int, list<Hsp_traits>> greedy_align(Sequence query, Sequence subject, vector<Diagonal_segment>::const_iterator begin, vector<Diagonal_segment>::const_iterator end, bool log, unsigned frame)
 {
 	const int band = config.chaining_maxgap;
 	if (end - begin == 1)
