@@ -22,6 +22,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 class ClusteringAlgorithm {
 public:
 	virtual void run() = 0;
-	virtual std::string get_key() = 0;
 	virtual std::string get_description() = 0;
+	virtual ~ClusteringAlgorithm(){};
 };
