@@ -17,18 +17,39 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
+#include <algorithm>
+#include <memory>
 #include <numeric>
 #include <iomanip>
+#include <thread>
 #include "mcl.h"
 #include "sparse_matrix_stream.h"
+#include "../util/util.h"
+#include "../util/sequence/sequence.h"
 
 #define MASK_INVERSE        0xC000000000000000
 #define MASK_NORMAL_NODE    0x4000000000000000
 #define MASK_ATTRACTOR_NODE 0x8000000000000000
 #define MASK_SINGLE_NODE    0xC000000000000000
 
-
-using namespace std;
+using std::thread;
+using std::numeric_limits;
+using std::chrono::high_resolution_clock;
+using std::chrono::duration_cast;
+using std::chrono::milliseconds;
+using std::function;
+using std::setprecision;
+using std::endl;
+using std::setw;
+using std::atomic_uint;
+using std::right;
+using std::memory_order_relaxed;
+using std::shared_ptr;
+using std::fill;
+using std::cout;
+using std::ostream;
+using std::move;
+using std::max;
 
 namespace Workflow { namespace Cluster{
 
@@ -36,23 +57,14 @@ string MCL::get_description() {
 	return "Markov clustering according to doi:10.1137/040608635";
 }
 
-inline uint64_t get_idx(uint32_t i_idx, uint32_t n_cols, uint32_t j_idx){
-	return i_idx * n_cols + j_idx;
-}
-
-inline void get_indices (uint64_t combined, uint32_t* i, uint32_t n_cols, uint32_t* j){
-	*j = combined % n_cols;
-	*i = ( combined - *j) / n_cols;
-}
-
-vector<Eigen::Triplet<float>> multiply(Eigen::SparseMatrix<float>* a, Eigen::SparseMatrix<float>* b , uint32_t iThr, uint32_t nThr){
+vector<Eigen::Triplet<float>> MCL::sparse_matrix_multiply(Eigen::SparseMatrix<float>* a, Eigen::SparseMatrix<float>* b , uint32_t iThr, uint32_t nThr){
 	uint32_t n_cols = b->cols();
 	uint32_t n_rows = a->rows();
 	std::vector<float> result_col(n_rows, 0.0);
 	vector<Eigen::Triplet<float>> data;
 	for (uint32_t j=0; j<n_cols; ++j) {
 		if(j%nThr == iThr){
-			fill(result_col.begin(), result_col.end(), 0.0);
+			fill(result_col.begin(), result_col.end(), 0.0f);
 			for (Eigen::SparseMatrix<float>::InnerIterator  rhsIt(*b, j); rhsIt; ++rhsIt) {
 				const float y = rhsIt.value();
 				const uint32_t k = rhsIt.row();
@@ -63,7 +75,7 @@ vector<Eigen::Triplet<float>> multiply(Eigen::SparseMatrix<float>* a, Eigen::Spa
 				}
 			}
 			for (uint32_t i=0; i<n_rows; ++i) {
-				if(result_col[i] > numeric_limits<float>::epsilon()) {
+				if(abs(result_col[i]) > numeric_limits<float>::epsilon()) {
 					data.emplace_back(i, j, result_col[i]);
 				}
 			}
@@ -73,7 +85,7 @@ vector<Eigen::Triplet<float>> multiply(Eigen::SparseMatrix<float>* a, Eigen::Spa
 }
 
 void MCL::get_exp(Eigen::SparseMatrix<float>* in, Eigen::SparseMatrix<float>* out, float r, uint32_t nThr){
-	chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+	high_resolution_clock::time_point t = high_resolution_clock::now();
 	if( r - (int) r == 0 ){
 		// TODO: at some r it may be more beneficial to diagnoalize in and only take the exponents of the eigenvalues
 		*out = *in;
@@ -82,7 +94,7 @@ void MCL::get_exp(Eigen::SparseMatrix<float>* in, Eigen::SparseMatrix<float>* ou
 			vector<Eigen::Triplet<float>> data;
 			std::mutex m;
 			auto mult = [&](const uint32_t iThr){
-				vector<Eigen::Triplet<float>> t_data = multiply(in, out, iThr, nThr);
+				vector<Eigen::Triplet<float>> t_data = sparse_matrix_multiply(in, out, iThr, nThr);
 				m.lock();
 				data.insert(data.end(), t_data.begin(), t_data.end());
 				m.unlock();
@@ -106,19 +118,19 @@ void MCL::get_exp(Eigen::SparseMatrix<float>* in, Eigen::SparseMatrix<float>* ou
 		throw runtime_error(" Eigen does not provide an eigenvalue solver for sparse matrices");
 	}
 	*out = out->pruned();
-	sparse_exp_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+	sparse_exp_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 }
 
 void MCL::get_exp(Eigen::MatrixXf* in, Eigen::MatrixXf* out, float r){
 	// TODO: at some r it may be more beneficial to diagnoalize in and only take the exponents of the eigenvalues
-	chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+	high_resolution_clock::time_point t = high_resolution_clock::now();
 	if(r - (int) r == 0){
 		*out = *in;
 		for(uint32_t i=1; i<r; i++){ 
 			(*out) *= (*in);
 		}
 		//*out = in->pow(r)
-		dense_int_exp_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+		dense_int_exp_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 	}
 	else{ 
 		// TODO: check whether the matrix is self-adjoint and use SelfAdjointEigenSolver instead
@@ -133,12 +145,12 @@ void MCL::get_exp(Eigen::MatrixXf* in, Eigen::MatrixXf* out, float r){
 		if( thr > numeric_limits<float>::epsilon() ){
 			out->noalias() = (V * d.real() * V.inverse()).real();
 		}
-		dense_gen_exp_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+		dense_gen_exp_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 	}
 }
 
 
-vector<Eigen::Triplet<float>> gammaIze(Eigen::SparseMatrix<float>* in, float r, uint32_t iThr, uint32_t nThr){
+vector<Eigen::Triplet<float>> MCL::sparse_matrix_get_gamma(Eigen::SparseMatrix<float>* in, float r, uint32_t iThr, uint32_t nThr){
 	vector<Eigen::Triplet<float>> data;
 	for (uint32_t k=0; k<in->outerSize(); ++k){
 		if( k%nThr == iThr){
@@ -148,7 +160,7 @@ vector<Eigen::Triplet<float>> gammaIze(Eigen::SparseMatrix<float>* in, float r,
 			}
 			for (Eigen::SparseMatrix<float>::InnerIterator it(*in, k); it; ++it){
 				float val = pow(it.value(), r) / colSum;
-				if(val > numeric_limits<float>::epsilon()) {
+				if(abs(val) > numeric_limits<float>::epsilon()) {
 					data.emplace_back(it.row(), it.col(), val);
 				}
 			}
@@ -158,11 +170,11 @@ vector<Eigen::Triplet<float>> gammaIze(Eigen::SparseMatrix<float>* in, float r,
 }
 
 void MCL::get_gamma(Eigen::SparseMatrix<float>* in, Eigen::SparseMatrix<float>* out, float r, uint32_t nThr){
-	chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+	high_resolution_clock::time_point t = high_resolution_clock::now();
 	vector<Eigen::Triplet<float>> data;
 	std::mutex m;
 	auto mult = [&](const uint32_t iThr){
-		vector<Eigen::Triplet<float>> t_data = gammaIze(in, r, iThr, nThr);
+		vector<Eigen::Triplet<float>> t_data = sparse_matrix_get_gamma(in, r, iThr, nThr);
 		m.lock();
 		data.insert(data.end(), t_data.begin(), t_data.end());
 		m.unlock();
@@ -179,10 +191,10 @@ void MCL::get_gamma(Eigen::SparseMatrix<float>* in, Eigen::SparseMatrix<float>*
 	out->setZero();
 	out->setFromTriplets(data.begin(), data.end(), [] (const float&, const float &b) { return b; });
 	*out = out->pruned(1.0, numeric_limits<float>::epsilon());
-	sparse_gamma_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+	sparse_gamma_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 }
 
-float get_norm(Eigen::SparseMatrix<float>* in, uint32_t nThr){
+float MCL::sparse_matrix_get_norm(Eigen::SparseMatrix<float>* in, uint32_t nThr){
 	vector<float> data(nThr);
 	fill(data.begin(), data.end(), 0.0);
 	auto norm = [&](const uint32_t iThr){
@@ -207,7 +219,7 @@ float get_norm(Eigen::SparseMatrix<float>* in, uint32_t nThr){
 }
 
 void MCL::get_gamma(Eigen::MatrixXf* in, Eigen::MatrixXf* out, float r){
-	chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+	high_resolution_clock::time_point t = high_resolution_clock::now();
 	// Note that Eigen matrices are column-major, so this is the most efficient way
 	for (uint32_t icol=0; icol<in->cols(); ++icol){
 		float colSum = 0.0f;
@@ -217,24 +229,21 @@ void MCL::get_gamma(Eigen::MatrixXf* in, Eigen::MatrixXf* out, float r){
 		for (uint32_t irow=0; irow<in->rows(); ++irow){
 			out->coeffRef(irow, icol) =  pow(in->coeffRef(irow, icol) , r) / colSum;
 		}
-	}
-	dense_gamma_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+		}
+		dense_gamma_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 }
 
 void MCL::markov_process(Eigen::SparseMatrix<float>* m, float inflation, float expansion, uint32_t max_iter, function<uint32_t()> getThreads){
-	for (uint32_t idiag=0; idiag<m->cols(); ++idiag){
-		assert(abs(m->coeffRef(idiag, idiag)) > numeric_limits<float>::epsilon());
-	}
 	uint32_t iteration = 0;
 	float diff_norm = numeric_limits<float>::max();
 	Eigen::SparseMatrix<float> msquared(m->rows(), m->cols());
 	Eigen::SparseMatrix<float> m_update(m->rows(), m->cols());
 	get_gamma(m, m, 1, getThreads()); // This is to get a matrix of random walks on the graph -> TODO: find out if something else is more suitable
-	while( iteration < max_iter && diff_norm > 1e-6*m->rows() ){
+	while( iteration < max_iter && diff_norm > numeric_limits<float>::epsilon()){
 		get_exp(m, &msquared, expansion, getThreads());
 		get_gamma(&msquared, &m_update, inflation, getThreads());
 		*m -= m_update;
-		diff_norm = get_norm(m, getThreads());
+		diff_norm = sparse_matrix_get_norm(m, getThreads());
 		*m = m_update;
 		iteration++;
 	}
@@ -244,15 +253,12 @@ void MCL::markov_process(Eigen::SparseMatrix<float>* m, float inflation, float e
 }
 
 void MCL::markov_process(Eigen::MatrixXf* m, float inflation, float expansion, uint32_t max_iter){
-	for (uint32_t idiag=0; idiag<m->cols(); ++idiag){
-		assert(abs(m->coeffRef(idiag, idiag)) > numeric_limits<float>::epsilon());
-	}
 	uint32_t iteration = 0;
 	float diff_norm = numeric_limits<float>::max();
 	Eigen::MatrixXf msquared(m->rows(), m->cols());
 	Eigen::MatrixXf m_update(m->rows(), m->cols());
 	get_gamma(m, m, 1); // This is to get a matrix of random walks on the graph -> TODO: find out if something else is more suitable
-	while( iteration < max_iter && diff_norm > numeric_limits<float>::epsilon()*m->rows() ){
+	while( iteration < max_iter && diff_norm > numeric_limits<float>::epsilon()){
 		get_exp(m, &msquared, expansion);
 		get_gamma(&msquared, &m_update, inflation);
 		*m -= m_update;
@@ -265,42 +271,65 @@ void MCL::markov_process(Eigen::MatrixXf* m, float inflation, float expansion, u
 	}
 }
 
-void MCL::print_stats(uint64_t nElements, uint32_t nComponents, uint32_t nComponentsLt1, vector<uint32_t>& sort_order, vector<vector<uint32_t>>& indices, vector<vector<Eigen::Triplet<float>>>& components){
+void MCL::print_stats(uint64_t nElements, uint32_t nComponents, uint32_t nComponentsLt1, vector<uint32_t>& sort_order, vector<vector<uint32_t>>& indices, SparseMatrixStream<float>* ms){
+	task_timer timer;
+	timer.go("Collecting stats");
 	const uint32_t chunk_size = config.cluster_mcl_chunk_size;
 	const uint32_t nThreads = min(config.threads_, nComponents / chunk_size);
 	const float inflation = (float) config.cluster_mcl_inflation;
 	const float expansion = (float) config.cluster_mcl_expansion;
-	vector<float> sparsities(nComponentsLt1);
-	uint64_t icomp = 0;
-	generate(sparsities.begin(), sparsities.end(), [&icomp, &sort_order, &indices, &components](){ 
-				const uint32_t iComponent = sort_order[icomp++];
-				const uint32_t ssize = components[iComponent].size();
-				const uint64_t dsize = indices[iComponent].size() * indices[iComponent].size();
-				return 1.0-(1.0 * ssize) / dsize;
-			});
-	vector<float> neighbors(nComponentsLt1);
-	icomp = 0;
-	generate(neighbors.begin(), neighbors.end(), [&icomp, &sort_order, &indices, &components](){ 
-				const uint32_t iComponent = sort_order[icomp++];
-				uint32_t neighbors = 0;
-				for(auto t: components[iComponent]){
-					if(t.row() != t.col()) neighbors++;
-				}
-				return neighbors / ((float) indices.size());
-			});
 
-	// rough memory calc before sorting sparsities
+	vector<float> sparsities(nComponentsLt1);
+	vector<float> neighbors(nComponentsLt1);
 	vector<float> memories(nComponentsLt1);
-	icomp = 0;
-	generate(memories.begin(), memories.end(), [&](){ 
-				const uint32_t iComponent = sort_order[icomp++];
-				if(sparsities[icomp - 1] >= config.cluster_mcl_sparsity_switch){ 
-					return indices[iComponent].size() * (1+pow(neighbors[icomp], expansion)) * (2 * sizeof(uint32_t) + sizeof(float));
-				}
-				else{
-					return (float) sizeof(float) * indices[iComponent].size() * indices[iComponent].size();
-				}
-			});
+
+	uint32_t max_job_size = 0;
+	for(uint32_t i=0; i<chunk_size; i++) max_job_size+=indices[sort_order[i]].size();
+	uint32_t my_counter = 0;
+	uint32_t component_counter = 1;
+
+	uint32_t my_chunk_size = chunk_size;
+	// Memory conservative collection of info from file
+	ms->allocate_read_buffer(1);
+	while(my_counter < nComponentsLt1){
+		unordered_set<uint32_t> all;
+		uint32_t upper_limit =  min(my_counter+my_chunk_size, nComponentsLt1);
+		vector<vector<uint32_t>*> loc_i;
+		for(uint32_t chunk_counter = my_counter; chunk_counter<upper_limit; chunk_counter++){
+			loc_i.push_back(&indices[sort_order[chunk_counter]]);
+		}
+		vector<vector<Eigen::Triplet<float>>> loc_c = ms->collect_components(&loc_i, 0);
+		uint32_t ichunk = 0;
+		generate(sparsities.begin()+my_counter, sparsities.begin()+upper_limit, [my_counter, ichunk, &sort_order, &indices, &loc_c]() mutable { 
+					const uint32_t iComponent = sort_order[my_counter++];
+					const uint32_t ssize = loc_c[ichunk++].size();
+					const uint64_t dsize = indices[iComponent].size() * indices[iComponent].size();
+					return 1.0-(1.0 * ssize) / dsize;
+				});
+		generate(neighbors.begin()+my_counter, neighbors.begin()+upper_limit, [my_counter, ichunk, &sort_order, &indices, &loc_c]() mutable { 
+					uint32_t neighbors = 0;
+					const uint32_t iComponent = sort_order[my_counter++];
+					for(auto t: loc_c[ichunk++]){
+						if(t.row() != t.col()) neighbors++;
+					}
+					return neighbors / ((float) indices[iComponent].size());
+				});
+		generate(memories.begin()+my_counter, memories.begin()+upper_limit, [my_counter, &sort_order, &indices, &neighbors, &sparsities, &expansion]() mutable -> float { 
+					const uint32_t iComponent = sort_order[my_counter++];
+					if(sparsities[my_counter- 1] >= config.cluster_mcl_sparsity_switch){ 
+						return (float)indices[iComponent].size() * (1+pow(neighbors[my_counter-1], expansion)) * (2 * sizeof(uint32_t) + sizeof(float));
+					}
+					else{
+						return (float) sizeof(float) * indices[iComponent].size() * indices[iComponent].size();
+					}
+				});
+		my_chunk_size = 0;
+		for(auto const i : loc_i) my_chunk_size += i->size();
+		my_chunk_size = loc_i.size() * (max_job_size / my_chunk_size);
+		my_counter = component_counter;
+		component_counter += my_chunk_size;
+	}
+
 	sort(memories.rbegin(), memories.rend());
 	float mem_req = nElements * (2 * sizeof(uint32_t) + sizeof(float));
 	for(uint32_t iThr = 0; iThr < nThreads; iThr++){
@@ -334,6 +363,7 @@ void MCL::print_stats(uint64_t nElements, uint32_t nComponents, uint32_t nCompon
 	float median2 = nComponentsLt1 > 0 && nComponentsLt1 % 2 == 0 ? 
 		(indices[sort_order[nComponentsLt1/2]].size() + indices[sort_order[nComponentsLt1/2+1]].size()) / 2.0 : 
 		indices[sort_order[nComponentsLt1/2+1]].size();
+	timer.finish();
 
 	message_stream << "Number of DIAMOND hits:          " << nElements << endl;
 	message_stream << "Number of independet components: " << nComponentsLt1 <<" ("<<nComponents<< " incl. singletons)" << endl;
@@ -353,13 +383,15 @@ void MCL::print_stats(uint64_t nElements, uint32_t nComponents, uint32_t nCompon
 	string uints[] = {"", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"};
 	uint32_t idx = ((uint32_t)log(mem_req)/log(1024));
 	message_stream << setprecision(2) << mem_req/pow(1024,idx) << uints[idx] << endl;
+	ms->release_read_buffer();
 }
 
-SparseMatrixStream<float>* get_graph_handle(DatabaseFile& db){
-	if(config.cluster_mcl_restart){
+shared_ptr<SparseMatrixStream<float>> get_graph_handle(shared_ptr<SequenceFile>& db){
+	const bool symmetric = !config.cluster_mcl_nonsymmetric;
+	if(config.cluster_restart){
 		task_timer timer;
 		timer.go("Reading cluster checkpoint file");
-		SparseMatrixStream<float>* ms = SparseMatrixStream<float>::fromFile(config.cluster_mcl_graph_file, config.chunk_size);
+		shared_ptr<SparseMatrixStream<float>> ms(SparseMatrixStream<float>::fromFile(symmetric, config.cluster_graph_file, config.chunk_size));
 		timer.finish();
 		ms->done();
 		return ms;
@@ -368,21 +400,41 @@ SparseMatrixStream<float>* get_graph_handle(DatabaseFile& db){
 	config.no_self_hits = false;
 	string format = config.cluster_similarity;
 	if(format.empty()){
-		format = "qcovhsp*scovhsp*pident";
+		format = "qcovhsp/100*scovhsp/100*pident/100";
 	}
 	config.output_format = {"clus", format};
-	Workflow::Search::Options opt;
-	opt.db = &db;
-	opt.self = true;
-	SparseMatrixStream<float>* ms = new SparseMatrixStream<float>(db.ref_header.sequences, config.cluster_mcl_graph_file);
+	shared_ptr<SparseMatrixStream<float>> ms(new SparseMatrixStream<float>(symmetric, db->sequence_count(), config.cluster_graph_file));
 	if(config.chunk_size > 0){
 		ms->set_max_mem(config.chunk_size);
 	}
-	opt.consumer = ms;
-	Workflow::Search::run(opt);
+	Search::run(db, nullptr, ms);
 	ms->done();
 	return ms;
 }
+Eigen::SparseMatrix<float> MCL::get_sparse_matrix_and_clear(vector<uint32_t>* order, vector<Eigen::Triplet<float>>* m, bool symmetric){
+	Eigen::SparseMatrix<float> m_sparse(order->size(), order->size());
+	if(symmetric){
+		uint64_t oSize = m->size();
+		for(uint64_t i = 0; i<oSize; i++){
+			Eigen::Triplet<float>& t = m->at(i);
+			if(t.col() != t.row()){ 
+				m->emplace_back(t.col(), t.row(), t.value());
+			}
+		}
+	}
+	m_sparse.setFromTriplets(m->begin(), m->end());
+	m->clear();
+	return m_sparse;
+}
+Eigen::MatrixXf MCL::get_dense_matrix_and_clear(vector<uint32_t>* order, vector<Eigen::Triplet<float>>* m, bool symmetric){
+	Eigen::MatrixXf m_dense = Eigen::MatrixXf::Zero(order->size(), order->size());
+	for(Eigen::Triplet<float> const & t : *m){
+		m_dense(t.row(), t.col()) = t.value();
+		if(symmetric && t.col() != t.row()) m_dense(t.col(), t.row()) = t.value();
+	}
+	m->clear();
+	return m_dense;
+}
 
 void MCL::run(){
 	// testing
@@ -509,10 +561,10 @@ void MCL::run(){
 	// }
 	// return;
 
-	if (config.database == "") throw runtime_error("Missing parameter: database file (--db/-d)");
-	unique_ptr<DatabaseFile> db(DatabaseFile::auto_create_from_fasta());
+	if (config.database.empty()) throw runtime_error("Missing parameter: database file (--db/-d)");
+	shared_ptr<SequenceFile> db(SequenceFile::auto_create(config.database));
 	statistics.reset();
-	SparseMatrixStream<float>* ms = get_graph_handle(*db);
+	shared_ptr<SparseMatrixStream<float>> ms(get_graph_handle(db));
 	task_timer timer;
 	timer.go("Computing independent components");
 	vector<vector<uint32_t>> indices = ms->get_indices();
@@ -526,17 +578,20 @@ void MCL::run(){
 	iota(sort_order.begin(), sort_order.end(), 0);
 	sort(sort_order.begin(), sort_order.end(), [&](uint32_t i, uint32_t j){return indices[i].size() > indices[j].size();});
 	timer.finish();
-	//print_stats(nElements, nComponents, nComponentsLt1, sort_order, indices, components);
+	if(config.cluster_mcl_stats){
+		print_stats(nElements, nComponents, nComponentsLt1, sort_order, indices, ms.get());
+	}
 
 	timer.go("Clustering components");
 	// Note, we will access the clustering_result from several threads below and a vector does not guarantee thread-safety in these situations.
 	// Note also, that the use of the disjoint_set structure guarantees that each thread will access a different part of the clustering_result
-	uint64_t* clustering_result = new uint64_t[db->ref_header.sequences];
-	fill(clustering_result, clustering_result+db->ref_header.sequences, 0UL);
+	uint64_t* clustering_result = new uint64_t[db->sequence_count()];
+	fill(clustering_result, clustering_result+db->sequence_count(), 0UL);
 
 	// note that the disconnected components are sorted by size
 	const uint32_t chunk_size = config.cluster_mcl_chunk_size;
 	const uint32_t nThreads = min(config.threads_, nComponents / chunk_size);
+	const uint32_t exessThreads = config.threads_ - nThreads;
 	ms->allocate_read_buffer(nThreads);
 	const float inflation = (float) config.cluster_mcl_inflation;
 	const float expansion = (float) config.cluster_mcl_expansion;
@@ -544,6 +599,7 @@ void MCL::run(){
 	const uint32_t max_iter = config.cluster_mcl_max_iter;
 	uint32_t max_job_size = 0;
 	for(uint32_t i=0; i<chunk_size; i++) max_job_size+=indices[sort_order[i]].size();
+	const bool symmetric = !config.cluster_mcl_nonsymmetric;
 	
 	// Collect some stats on the way
 	uint32_t* jobs_per_thread = new uint32_t[nThreads];
@@ -556,7 +612,7 @@ void MCL::run(){
 	atomic_uint threads_done(0);
 
 	auto mcl_clustering = [&](const uint32_t iThr){
-		chrono::high_resolution_clock::time_point thread_start = chrono::high_resolution_clock::now();
+		high_resolution_clock::time_point thread_start = high_resolution_clock::now();
 		uint32_t n_dense = 0;
 		uint32_t n_sparse = 0;
 		uint32_t n_singletons = 0;
@@ -568,16 +624,15 @@ void MCL::run(){
 			unordered_set<uint32_t> all;
 			uint32_t upper_limit =  min(my_counter+my_chunk_size, max_counter);
 			vector<vector<uint32_t>*> loc_i;
-			my_chunk_size = 0;
 			for(uint32_t chunk_counter = my_counter; chunk_counter<upper_limit; chunk_counter++){
 				loc_i.push_back(&indices[sort_order[chunk_counter]]);
-				my_chunk_size += indices[sort_order[chunk_counter]].size();
 			}
 			vector<vector<Eigen::Triplet<float>>> loc_c = ms->collect_components(&loc_i, iThr);
 			uint32_t ichunk = 0;
 			for(uint32_t chunk_counter = my_counter; chunk_counter<upper_limit; chunk_counter++){
 				n_jobs_done++;
 				vector<uint32_t>* order = loc_i[ichunk];
+
 				if(order->size() > 1){
 					vector<Eigen::Triplet<float>>* m = &loc_c[ichunk];
 					assert(m->size() <= order->size() * order->size());
@@ -586,20 +641,19 @@ void MCL::run(){
 					unordered_set<uint32_t> attractors;
 
 					const float sparsity = 1.0-(1.0 * m->size()) / (order->size()*order->size());
-					chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+					high_resolution_clock::time_point t = high_resolution_clock::now();
 					//TODO: a size limit for the dense matrix should control this as well
 					if(sparsity >= config.cluster_mcl_sparsity_switch && expansion - (int) expansion == 0){ 
 						n_sparse++;
-						Eigen::SparseMatrix<float> m_sparse(order->size(), order->size());
-						m_sparse.setFromTriplets(m->begin(), m->end());
-						sparse_create_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
-						auto getThreads = [&threads_done, &nThreads, &iThr](){
+						Eigen::SparseMatrix<float> m_sparse = get_sparse_matrix_and_clear(order, m, symmetric);
+						sparse_create_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
+						auto getThreads = [&threads_done, &nThreads, &exessThreads, &iThr](){
 							uint32_t td=threads_done.load();
-							uint32_t rt=nThreads-td;
-							return 1 + td / rt + (iThr < (td % rt) ? 1 : 0);
+							uint32_t rt= nThreads - td;
+							return 1 + (td+exessThreads) / rt + (iThr < ((td+exessThreads) % rt) ? 1 : 0);
 						};
 						markov_process(&m_sparse, inflation, expansion, max_iter, getThreads);
-						chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+						high_resolution_clock::time_point t = high_resolution_clock::now();
 						LazyDisjointIntegralSet<uint32_t> disjointSet(m_sparse.cols());
 						for (uint32_t k=0; k<m_sparse.outerSize(); ++k){
 							for (Eigen::SparseMatrix<float>::InnerIterator it(m_sparse, k); it; ++it){
@@ -611,21 +665,18 @@ void MCL::run(){
 							}
 						}
 						list_of_sets = disjointSet.getListOfSets();
-						sparse_list_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+						sparse_list_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 					}
 					else{
 						n_dense++;
-						Eigen::MatrixXf m_dense = Eigen::MatrixXf::Zero(order->size(), order->size());
-						for(Eigen::Triplet<float> const & t : *m){
-							m_dense(t.row(), t.col()) = t.value();
-						}
-						dense_create_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+						Eigen::MatrixXf m_dense = get_dense_matrix_and_clear(order, m, symmetric);
+						dense_create_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 						markov_process(&m_dense, inflation, expansion, max_iter);
-						chrono::high_resolution_clock::time_point t = chrono::high_resolution_clock::now();
+						high_resolution_clock::time_point t = high_resolution_clock::now();
 						LazyDisjointIntegralSet<uint32_t> disjointSet(m_dense.cols());
 						for (uint32_t icol=0; icol<m_dense.cols(); ++icol){
 							for (uint32_t irow=0; irow<m_dense.rows(); ++irow){
-								if( abs(m_dense(irow, icol)) > numeric_limits<float>::epsilon()){
+								if(abs(m_dense(irow, icol)) > numeric_limits<float>::epsilon()){
 									disjointSet.merge(irow, icol);
 									if(irow == icol){
 										attractors.emplace(irow);
@@ -634,7 +685,7 @@ void MCL::run(){
 							}
 						}
 						list_of_sets = disjointSet.getListOfSets();
-						dense_list_time += chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - t).count();
+						dense_list_time += duration_cast<milliseconds>(high_resolution_clock::now() - t).count();
 					}
 					for(unordered_set<uint32_t> subset : list_of_sets){
 						assert(cluster_id < 0x3fffffffffffffff);
@@ -665,7 +716,7 @@ void MCL::run(){
 		nClustersEq1.fetch_add(n_singletons, memory_order_relaxed);
 		jobs_per_thread[iThr] = n_jobs_done;
 		threads_done.fetch_add(1, memory_order_relaxed);
-		time_per_thread[iThr] = (chrono::duration_cast<chrono::milliseconds>(chrono::high_resolution_clock::now() - thread_start).count()) / 1000.0;
+		time_per_thread[iThr] = (duration_cast<milliseconds>(high_resolution_clock::now() - thread_start).count()) / 1000.0;
 	};
 
 	vector<thread> threads;
@@ -677,7 +728,6 @@ void MCL::run(){
 		threads[iThread].join();
 	}
 	ms->release_read_buffer();
-	delete ms;
 	timer.finish();
 
 	// Output stats
@@ -729,15 +779,15 @@ void MCL::run(){
 	ostream *out = config.output_file.empty() ? &cout : new ofstream(config.output_file.c_str());
 	vector<Letter> seq;
 	string id;
-	db->seek_direct();
-	Hsp hsp;
+	db->init_seq_access();
+	//Hsp hsp;
 	size_t n;
 	out->precision(3);
-	for (int i = 0; i < (int)db->ref_header.sequences; ++i) {
-		db->read_seq(id, seq);
+	for (int i = 0; i < (int)db->sequence_count(); ++i) {
+		db->read_seq(seq, id);
 		const uint64_t cid = (~MASK_INVERSE) & clustering_result[i];
 		const uint64_t lab = MASK_INVERSE & clustering_result[i];
-		(*out) << blast_id(id) << '\t' ;
+		(*out) << Util::Seq::seqid(id.c_str(), false) << '\t' ;
 		switch(lab){
 			case MASK_SINGLE_NODE:
 				(*out) << cid << '\t'<< 's';
