@@ -16,8 +16,10 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#include "multi_step_cluster.h"
 #include <unordered_map>
+#include "multi_step_cluster.h"
+#include "../util/util.h"
+#include "../util/sequence/sequence.h"
 
 using namespace std;
 
@@ -35,7 +37,7 @@ BitVector MultiStep::rep_bitset(const vector<int> &centroid, const BitVector *su
 	return r;
 }
 
-vector<int> MultiStep::cluster(DatabaseFile& db, const BitVector* filter) {
+vector<int> MultiStep::cluster(shared_ptr<SequenceFile>& db, const shared_ptr<BitVector>& filter) {
 	statistics.reset();
 	config.command = Config::blastp;
 	//config.no_self_hits = true;
@@ -44,20 +46,13 @@ vector<int> MultiStep::cluster(DatabaseFile& db, const BitVector* filter) {
 	config.output_format = { "bin" };
 	config.query_cover = 80;
 	config.subject_cover = 80;
-	config.algo = 0;
-	config.index_mode = 0;
-	config.freq_sd = 0;
+	config.algo = Config::Algo::DOUBLE_INDEXED;
+	//config.freq_sd = 0;
 	config.max_alignments = numeric_limits<size_t>::max();
 
-	Workflow::Search::Options opt;
-	opt.db = &db;
-	opt.self = true;
-	Neighbors nb(db.ref_header.sequences);
+	shared_ptr<Neighbors> nb(new Neighbors(db->sequence_count()));
 
-	opt.consumer = &nb;
-	opt.db_filter = filter;
-
-	Workflow::Search::run(opt);
+	Search::run(db, nullptr, nb, filter);
 	
 	/*
 	auto lo = nb.dSet.getListOfSets();
@@ -70,11 +65,11 @@ vector<int> MultiStep::cluster(DatabaseFile& db, const BitVector* filter) {
 	}
 	*/
 
-	vector<unordered_set<uint32_t>> connected = nb.dSet.getListOfSets();
-	vector<uint32_t> EdgSet(nb.number_edges.size());
-	unordered_map <uint32_t, NodEdgSet> components = find_connected_components(connected, EdgSet, nb.number_edges);
+	vector<unordered_set<int64_t>> connected = nb->dSet.getListOfSets();
+	vector<uint32_t> EdgSet(nb->number_edges.size());
+	unordered_map <int64_t, NodEdgSet> components = find_connected_components(connected, EdgSet, nb->number_edges);
 	message_stream << "Number of connected components: " << components.size() << endl;
-	message_stream << "Average number of nodes per connected component: " << (double)nb.number_edges.size() / components.size() << endl;
+	message_stream << "Average number of nodes per connected component: " << (double)nb->number_edges.size() / components.size() << endl;
 
 	uint32_t large = max_element(components.begin(), components.end(), [](const pair<uint32_t, NodEdgSet>& left, const pair<uint32_t, NodEdgSet>& right) {return left.second.nodes < right.second.nodes; })-> second.nodes;
 	message_stream << "Largest connected component has " << large << " nodes." << endl;
@@ -86,17 +81,17 @@ vector<int> MultiStep::cluster(DatabaseFile& db, const BitVector* filter) {
 
 
 	if (config.external) {
-		save_edges_external(nb.tempfiles, tmp_sets, components, EdgSet);
-		return cluster_sets(db.ref_header.sequences, tmp_sets);
+		save_edges_external(nb->tempfiles, tmp_sets, components, EdgSet);
+		return cluster_sets(db->sequence_count(), tmp_sets);
 	}
 
 	else {
-		return Util::Algo::greedy_vortex_cover(nb);
+		return Util::Algo::greedy_vertex_cover(*nb);
 	}
 
 }
 
-void MultiStep::save_edges_external(vector<TempFile*>& all_edges, vector<TempFile*>& sorted_edges, const unordered_map<uint32_t, NodEdgSet>& comp, const vector<uint32_t>& s_index){
+void MultiStep::save_edges_external(vector<TempFile*>& all_edges, vector<TempFile*>& sorted_edges, const unordered_map<int64_t, NodEdgSet>& comp, const vector<uint32_t>& s_index){
 
 	size_t count = 0;
 	uint32_t query;
@@ -144,7 +139,7 @@ vector<int> MultiStep::cluster_sets(const size_t nb_size, vector<TempFile*> &sor
 				break;
 			}
 		}
-		curr = Util::Algo::greedy_vortex_cover(tmp_neighbors);
+		curr = Util::Algo::greedy_vertex_cover(tmp_neighbors);
 
 		for (int i = 0; i < (int)curr.size(); i++) {
 				if (curr[i] != i) {
@@ -159,9 +154,9 @@ vector<int> MultiStep::cluster_sets(const size_t nb_size, vector<TempFile*> &sor
 	return cluster;
 }
 
-unordered_map<uint32_t, NodEdgSet> MultiStep::find_connected_components(const vector<unordered_set<uint32_t>> &connected, vector<uint32_t> &EdgSet, const vector <size_t>& nedges){
+unordered_map<int64_t, NodEdgSet> MultiStep::find_connected_components(const vector<unordered_set<int64_t>> &connected, vector<uint32_t> &EdgSet, const vector <size_t>& nedges){
 	
-	unordered_map<uint32_t, NodEdgSet> ne;
+	unordered_map<int64_t, NodEdgSet> ne;
 	for (size_t i = 0; i < connected.size(); i++) {
 		for(uint32_t j : connected[i]){
 			EdgSet[j] = i;
@@ -173,8 +168,8 @@ unordered_map<uint32_t, NodEdgSet> MultiStep::find_connected_components(const ve
 	return ne;
 }
 
-vector<TempFile*> MultiStep::mapping_comp_set(unordered_map<uint32_t, NodEdgSet>& comp) {
-	vector <vector<uint32_t>> set;
+vector<TempFile*> MultiStep::mapping_comp_set(unordered_map<int64_t, NodEdgSet>& comp) {
+	vector <vector<int64_t>> set;
 	vector <size_t> size;
 	vector<TempFile*> temp_set;
 	
@@ -235,47 +230,40 @@ void MultiStep::run() {
 	if (config.database == "")
 		throw runtime_error("Missing parameter: database file (--db/-d)");
 	config.command = Config::makedb;
-	unique_ptr<DatabaseFile> db(DatabaseFile::auto_create_from_fasta());
-	const size_t seq_count = db->ref_header.sequences;
+	shared_ptr<SequenceFile> db(SequenceFile::auto_create(config.database));
+	const size_t seq_count = db->sequence_count();
 
-	BitVector current_reps;
-	BitVector previous_reps;
+	shared_ptr<BitVector> current_reps, previous_reps;
 
 	vector<int> current_centroids;
 	vector<int> previous_centroids;
 	
 	for (size_t i = 0; i < config.cluster_steps.size(); i++) {
 		
-		if (config.sens_map.find(config.cluster_steps[i]) == config.sens_map.end()) {
-			throw std::runtime_error("Invalid value for parameter --cluster-steps");
-		}
-			
-		config.sensitivity = config.sens_map[config.cluster_steps[i]];
-		current_centroids = cluster(*db, i==0 ? nullptr: &previous_reps);
-		steps(current_reps, previous_reps, current_centroids, previous_centroids, i);
+		config.sensitivity = from_string<Sensitivity>(config.cluster_steps[i]);
+		current_centroids = cluster(db, i == 0 ? nullptr : previous_reps);
+		steps(*current_reps, *previous_reps, current_centroids, previous_centroids, i);
 	}
 		
 	task_timer timer("Generating output");
-	Sequence_set* rep_seqs;
-	String_set<char, 0>* rep_ids;
-	vector<unsigned> rep_database_id, rep_block_id(seq_count);
-	db->rewind();
-	db->load_seqs(&rep_database_id, (size_t)1e11, &rep_seqs, &rep_ids, true, &previous_reps);
-	for (size_t i = 0; i < rep_database_id.size(); ++i)
-		rep_block_id[rep_database_id[i]] = (unsigned)i;
+	vector<unsigned> rep_block_id(seq_count);
+	db->set_seqinfo_ptr(0);
+	Block* block = db->load_seqs((size_t)1e11, true, previous_reps.get());
+	for (size_t i = 0; i < block->seqs().size(); ++i)
+		rep_block_id[block->block_id2oid(i)] = (unsigned)i;
 
 	ostream* out = config.output_file.empty() ? &cout : new ofstream(config.output_file.c_str());
 	vector<Letter> seq;
 	string id;
-	db->seek_direct();
-	Hsp hsp;
+	db->init_seq_access();
+	//Hsp hsp;
 	out->precision(3);
 
-	for (int i = 0; i < (int)db->ref_header.sequences; ++i) {
-		db->read_seq(id, seq);
+	for (int i = 0; i < (int)db->sequence_count(); ++i) {
+		db->read_seq(seq, id);
 		const unsigned r = rep_block_id[previous_centroids[i]];
-		(*out) << blast_id(id) << '\t'
-			<< blast_id((*rep_ids)[r]) << '\n';
+		(*out) << Util::Seq::seqid(id.c_str(), false) << '\t'
+			<< Util::Seq::seqid(block->ids()[r], false) << '\n';
 		/*if ((int)i == centroid2[i])
 			(*out) << "100\t100\t100\t0" << endl;
 		else {
@@ -288,8 +276,7 @@ void MultiStep::run() {
 		} */
 	}
 	if (out != &cout) delete out;
-	delete rep_seqs;
-	delete rep_ids;
+	delete block;
 	db->close();
 
 }
