@@ -1,6 +1,10 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+                        Eberhard Karls Universitaet Tuebingen
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -21,14 +25,29 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "seed_set.h"
 #include "enum_seeds.h"
 #include "../util/data_structures/deque.h"
+#include "../search/seed_complexity.h"
 
 using std::array;
 
-typedef vector<Array<SeedArray::Entry*, Const::seedp>> PtrSet;
+typedef vector<array<SeedArray::Entry*, Const::seedp>> PtrSet;
 
-char* SeedArray::alloc_buffer(const Partitioned_histogram &hst)
+char* SeedArray::alloc_buffer(const SeedHistogram &hst, size_t index_chunks)
 {
-	return new char[sizeof(Entry) * hst.max_chunk_size()];
+	return new char[sizeof(Entry) * hst.max_chunk_size(index_chunks)];
+}
+
+static size_t seed_bits(const SeedEncoding code) {
+	switch (code) {
+	case SeedEncoding::HASHED:
+		return sizeof(SeedArray::Entry::Key) * 8;
+	case SeedEncoding::SPACED_FACTOR:
+		return ceil(shapes[0].weight_ * Reduction::reduction.bit_size_exact()) - Const::seedp_bits;
+	case SeedEncoding::CONTIGUOUS:
+		return shapes[0].length_ * Reduction::reduction.bit_size() - Const::seedp_bits;
+	default:
+		break;
+	}
+	throw std::runtime_error("Unknown seed encoding.");
 }
 
 struct BufferedWriter
@@ -39,12 +58,12 @@ struct BufferedWriter
 		memset(n, 0, sizeof(n));
 		memcpy(this->ptr, ptr, sizeof(this->ptr));
 	}
-	void push(Packed_seed key, Loc value, const SeedPartitionRange &range)
+	void push(Packed_seed key, int64_t value, uint32_t block_id, const SeedPartitionRange &range)
 	{
 		const unsigned p = seed_partition(key);
 		if (range.contains(p)) {
 			assert(n[p] < BUFFER_SIZE);
-			buf[p][n[p]++] = SeedArray::Entry(seed_partition_offset(key), value);
+			buf[p][n[p]++] = SeedArray::Entry(seed_partition_offset(key), value, block_id);
 			if (n[p] == BUFFER_SIZE)
 				flush(p);
 		}
@@ -65,7 +84,7 @@ struct BufferedWriter
 	uint8_t n[Const::seedp];
 };
 
-PtrSet build_iterators(SeedArray &sa, const shape_histogram &hst)
+PtrSet build_iterators(SeedArray &sa, const ShapeHistogram &hst)
 {
 	PtrSet iterators(hst.size());
 	for (unsigned i = 0; i < Const::seedp; ++i)
@@ -83,9 +102,9 @@ struct BuildCallback
 		range(range),
 		it(new BufferedWriter(ptr))
 	{ }
-	bool operator()(uint64_t seed, uint64_t pos, size_t shape)
+	bool operator()(uint64_t seed, uint64_t pos, uint32_t block_id, size_t shape)
 	{
-		it->push(seed, pos, range);
+		it->push(seed, pos, block_id, range);
 		return true;
 	}
 	void finish()
@@ -101,32 +120,36 @@ struct BuildCallback
 };
 
 template<typename _filter>
-SeedArray::SeedArray(const Sequence_set &seqs, size_t shape, const shape_histogram &hst, const SeedPartitionRange &range, const vector<size_t> &seq_partition, char *buffer, const _filter *filter) :
+SeedArray::SeedArray(Block &seqs, const ShapeHistogram &hst, const SeedPartitionRange &range, char *buffer, const _filter *filter, const EnumCfg& enum_cfg) :
+	key_bits(seed_bits(enum_cfg.code)),
 	data_((Entry*)buffer)
 {
+	if (enum_cfg.shape_end - enum_cfg.shape_begin > 1)
+		throw std::runtime_error("SeedArray construction for >1 shape.");
 	begin_[range.begin()] = 0;
 	for (size_t i = range.begin(); i < range.end(); ++i)
 		begin_[i + 1] = begin_[i] + partition_size(hst, i);
 
 	PtrSet iterators(build_iterators(*this, hst));
 	PtrVector<BuildCallback> cb;
-	for (size_t i = 0; i < seq_partition.size() - 1; ++i)
-		cb.push_back(new BuildCallback(range, iterators[i].begin()));
-	enum_seeds(&seqs, cb, seq_partition, shape, shape + 1, filter);
+	for (size_t i = 0; i < enum_cfg.partition->size() - 1; ++i)
+		cb.push_back(new BuildCallback(range, iterators[i].data()));
+	stats_ = enum_seeds(seqs, cb, filter, enum_cfg);
 }
 
-template SeedArray::SeedArray(const Sequence_set &, size_t, const shape_histogram &, const SeedPartitionRange &, const vector<size_t>&, char *buffer, const No_filter *);
-template SeedArray::SeedArray(const Sequence_set &, size_t, const shape_histogram &, const SeedPartitionRange &, const vector<size_t>&, char *buffer, const Seed_set *);
-template SeedArray::SeedArray(const Sequence_set &, size_t, const shape_histogram &, const SeedPartitionRange &, const vector<size_t>&, char *buffer, const Hashed_seed_set *);
+template SeedArray::SeedArray(Block&, const ShapeHistogram&, const SeedPartitionRange &, char *buffer, const NoFilter *, const EnumCfg&);
+template SeedArray::SeedArray(Block&, const ShapeHistogram&, const SeedPartitionRange &, char *buffer, const SeedSet *, const EnumCfg&);
+template SeedArray::SeedArray(Block&, const ShapeHistogram&, const SeedPartitionRange &, char *buffer, const HashedSeedSet *, const EnumCfg&);
 
 struct BufferedWriter2
 {
 	static const unsigned BUFFER_SIZE = 16;
-	BufferedWriter2()
+	BufferedWriter2():
+		out(Const::seedp)
 	{
 		memset(n, 0, sizeof(n));
 	}
-	void push(Packed_seed key, Loc value, const SeedPartitionRange& range)
+	void push(Packed_seed key, int64_t value, const SeedPartitionRange& range)
 	{
 		const unsigned p = seed_partition(key);
 		if (range.contains(p)) {
@@ -147,7 +170,7 @@ struct BufferedWriter2
 			if (n[p] > 0)
 				flush(p);
 	}
-	Deque<SeedArray::Entry> out[Const::seedp];
+	vector<Deque<SeedArray::Entry, 15>> out;
 	SeedArray::Entry buf[Const::seedp][BUFFER_SIZE];
 	uint8_t n[Const::seedp];
 };
@@ -158,7 +181,7 @@ struct BuildCallback2
 		range(range),
 		it(new BufferedWriter2())
 	{ }
-	bool operator()(uint64_t seed, uint64_t pos, size_t shape)
+	bool operator()(uint64_t seed, uint64_t pos, uint32_t block_id, size_t shape)
 	{
 		it->push(seed, pos, range);
 		return true;
@@ -176,14 +199,18 @@ struct BuildCallback2
 };
 
 template<typename _filter>
-SeedArray::SeedArray(const Sequence_set& seqs, size_t shape, const SeedPartitionRange& range, const _filter* filter) :
+SeedArray::SeedArray(Block& seqs, const SeedPartitionRange& range, const _filter* filter, EnumCfg& enum_cfg) :
+	key_bits(seed_bits(enum_cfg.code)),
 	data_(nullptr)
 {
-	const auto seq_partition = seqs.partition(config.threads_);
+	if (enum_cfg.shape_end - enum_cfg.shape_begin > 1)
+		throw std::runtime_error("SeedArray construction for >1 shape.");
+	const auto seq_partition = seqs.seqs().partition(config.threads_);
 	PtrVector<BuildCallback2> cb;
 	for (size_t i = 0; i < seq_partition.size() - 1; ++i)
 		cb.push_back(new BuildCallback2(range));
-	enum_seeds(&seqs, cb, seq_partition, shape, shape + 1, filter);
+	enum_cfg.partition = &seq_partition;
+	stats_ = enum_seeds(seqs, cb, filter, enum_cfg);
 
 	array<size_t, Const::seedp> counts;
 	counts.fill(0);
@@ -198,4 +225,4 @@ SeedArray::SeedArray(const Sequence_set& seqs, size_t shape, const SeedPartition
 	}
 }
 
-template SeedArray::SeedArray(const Sequence_set&, size_t, const SeedPartitionRange&, const Hashed_seed_set*);
\ No newline at end of file
+template SeedArray::SeedArray(Block&, const SeedPartitionRange&, const HashedSeedSet*, EnumCfg&);
\ No newline at end of file
