@@ -20,14 +20,18 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <array>
 #include <vector>
 #include "seed_histogram.h"
-#include "../basic/packed_loc.h"
+#include "../search/seed_complexity.h"
+#include "flags.h"
 
 #pragma pack(1)
+// #define KEEP_TARGET_ID
+
+struct Block;
 
 struct SeedArray
 {
 
-	typedef Packed_loc _pos;
+	using Loc = PackedLoc;
 
 	struct Entry
 	{
@@ -35,26 +39,36 @@ struct SeedArray
 			key(),
 			value()
 		{ }
-		Entry(unsigned key, _pos value) :
+		Entry(unsigned key, Loc value) :
 			key(key),
 			value(value)
 		{ }
+		Entry(unsigned key, Loc pos, uint32_t block_id):
+			key(key),
+#ifdef KEEP_TARGET_ID
+			value(pos, block_id)
+#else
+			value(pos)
+#endif
+		{}
 		uint32_t key;
-		_pos value;
+		
 		struct GetKey {
 			uint32_t operator()(const Entry& e) const {
 				return e.key;
 			}
 		};
-		typedef _pos Value;
-		typedef uint32_t Key;
+
+		using Key = uint32_t;
+		using Value = SeedLoc;
+		SeedLoc value;
 	} PACKED_ATTRIBUTE;
 
 	template<typename _filter>
-	SeedArray(const Sequence_set &seqs, size_t shape, const shape_histogram &hst, const SeedPartitionRange &range, const vector<size_t> &seq_partition, char *buffer, const _filter *filter);
+	SeedArray(Block &seqs, const ShapeHistogram &hst, const SeedPartitionRange &range, char *buffer, const _filter *filter, const EnumCfg& enum_cfg);
 
 	template<typename _filter>
-	SeedArray(const Sequence_set& seqs, size_t shape, const SeedPartitionRange& range, const _filter* filter);
+	SeedArray(Block& seqs, const SeedPartitionRange& range, const _filter* filter, EnumCfg& cfg);
 
 	Entry* begin(unsigned i)
 	{
@@ -92,13 +106,20 @@ struct SeedArray
 		}
 	}
 
-	static char *alloc_buffer(const Partitioned_histogram &hst);
+	const Search::SeedStats& stats() const {
+		return stats_;
+	}
+
+	static char *alloc_buffer(const SeedHistogram &hst, size_t index_chunks);
+
+	const size_t key_bits;
 
 private:
 
 	Entry *data_;
 	size_t begin_[Const::seedp + 1];
 	std::array<std::vector<Entry>, Const::seedp> entries_;
+	Search::SeedStats stats_;
 
 };
 
