@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,13 +21,22 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #pragma once
 #include <vector>
-#include "sequence_set.h"
 #include "../util/hash_table.h"
 #include "../util/ptr_vector.h"
+#include "../util/data_structures/hash_set.h"
+#include "../masking/masking.h"
+#define NOMINMAX
+#include "../lib/mio/mmap.hpp"
 
-struct Seed_set
+const uint64_t SEED_INDEX_MAGIC_NUMBER = 0x2d6ba306ecbf6aba;
+const uint32_t SEED_INDEX_VERSION = 0;
+const size_t SEED_INDEX_HEADER_SIZE = 16;
+
+struct Block;
+
+struct SeedSet
 {
-	Seed_set(const Sequence_set &seqs, double max_coverage);
+	SeedSet(Block &seqs, double max_coverage, const std::vector<bool>* skip, const double seed_cut, const MaskingAlgo soft_masking);
 	bool contains(uint64_t key, uint64_t shape) const
 	{
 		return data_[key];
@@ -38,15 +50,21 @@ private:
 	double coverage_;
 };
 
-struct Hashed_seed_set
+struct HashedSeedSet
 {
-	Hashed_seed_set(const Sequence_set &seqs);
-	~Hashed_seed_set();
+	typedef HashSet<Modulo2, Identity> Table;
+	HashedSeedSet(Block &seqs, const std::vector<bool>* skip, const double seed_cut, const MaskingAlgo soft_masking);
+	HashedSeedSet(const std::string& index_file);
+	~HashedSeedSet();
 	bool contains(uint64_t key, uint64_t shape) const
 	{
 		return data_[shape].contains(key);
 	}
+	const Table& table(size_t i) const {
+		return data_[i];
+	}
+	size_t max_table_size() const;
 private:
-	PtrVector<PHash_set<Modulo2, No_hash>> data_;
-	std::vector<int> fd_;
+	PtrVector<Table> data_;
+	std::unique_ptr<mio::mmap_source> mmap_;
 };
