@@ -30,6 +30,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 using std::list;
 using std::thread;
 using std::atomic;
+using std::pair;
 
 namespace DISPATCH_ARCH {
 
@@ -327,15 +328,16 @@ struct Banded3FrameSwipeMatrixRef<_sv, DP::ScoreOnly>
 };
 
 template<typename _sv>
-Hsp traceback(sequence *query, Strand strand, int dna_len, const Banded3FrameSwipeTracebackMatrix<_sv> &dp, const DpTarget &target, int d_begin, typename ScoreTraits<_sv>::Score max_score, double evalue, int max_col, int channel, int i0, int i1)
+Hsp traceback(Sequence *query, Strand strand, int dna_len, const Banded3FrameSwipeTracebackMatrix<_sv> &dp, const DpTarget &target, int d_begin, typename ScoreTraits<_sv>::Score max_score, double evalue, int max_col, int channel, int i0, int i1)
 {
 	typedef typename ScoreTraits<_sv>::Score Score;
 	const int j0 = i1 - (target.d_end - 1), d1 = target.d_end;
 	typename Banded3FrameSwipeTracebackMatrix<_sv>::TracebackIterator it(dp.traceback(max_col + 1, i0 + max_col, j0 + max_col, dna_len, channel, max_score));
 	
-	Hsp out;
+	Hsp out(true);
 	out.swipe_target = target.target_idx;
 	out.score = ScoreTraits<_sv>::int_score(max_score) * config.cbs_matrix_scale;
+	out.bit_score = score_matrix.bitscore(out.score);
 	out.evalue = evalue;
 	out.transcript.reserve(size_t(out.score * config.transcript_len_estimate));
 
@@ -371,12 +373,13 @@ Hsp traceback(sequence *query, Strand strand, int dna_len, const Banded3FrameSwi
 }
 
 template<typename _sv>
-Hsp traceback(sequence *query, Strand strand, int dna_len, const Banded3FrameSwipeMatrix<_sv> &dp, const DpTarget &target, int d_begin, typename ScoreTraits<_sv>::Score max_score, double evalue, int max_col, int channel, int i0, int i1)
+Hsp traceback(Sequence *query, Strand strand, int dna_len, const Banded3FrameSwipeMatrix<_sv> &dp, const DpTarget &target, int d_begin, typename ScoreTraits<_sv>::Score max_score, double evalue, int max_col, int channel, int i0, int i1)
 {
-	Hsp out;
+	Hsp out(false);
 	const int j0 = i1 - (target.d_end - 1);
 	out.swipe_target = target.target_idx;
 	out.score = ScoreTraits<_sv>::int_score(max_score) * config.cbs_matrix_scale;
+	out.bit_score = score_matrix.bitscore(out.score);
 	out.evalue = evalue;
 	out.query_range.end_ = std::min(i0 + max_col + (int)dp.band() / 3 / 2, (int)query[0].length());
 	out.query_range.begin_ = std::max(out.query_range.end_ - (j0 + max_col), 0);
@@ -398,7 +401,7 @@ list<Hsp> banded_3frame_swipe(
 	typedef typename ScoreTraits<_sv>::Score Score;
 
 	assert(subject_end - subject_begin <= ScoreTraits<_sv>::CHANNELS);
-	sequence q[3];
+	Sequence q[3];
 	query.get_strand(strand, q);
 	const int qlen = (int)q[0].length(), qlen2 = (int)q[1].length(), qlen3 = (int)q[2].length();
 
@@ -422,6 +425,7 @@ list<Hsp> banded_3frame_swipe(
 		frameshift_penalty(score_matrix.frame_shift());
 	
 	SwipeProfile<_sv> profile;
+	std::array<const int8_t*, 32> target_scores;
 	Score best[ScoreTraits<_sv>::CHANNELS];
 	int max_col[ScoreTraits<_sv>::CHANNELS];
 	for (int i = 0; i < ScoreTraits<_sv>::CHANNELS; ++i) {
@@ -440,7 +444,11 @@ list<Hsp> banded_3frame_swipe(
 		_sv vgap0, vgap1, vgap2, hgap, col_best;
 		vgap0 = vgap1 = vgap2 = col_best = ScoreTraits<_sv>::zero();
 
+#ifdef __SSSE3__
 		profile.set(targets.get());
+#else
+		profile.set(targets.get(target_scores.data()));
+#endif
 		for (int i = i0_; i <= i1_; ++i) {
 			hgap = it.hgap();
 			_sv next = cell_update<_sv>(it.sm3, it.sm4, it.sm2, profile.get(q[0][i]), extend_penalty, open_penalty, frameshift_penalty, hgap, vgap0, col_best);
@@ -513,11 +521,11 @@ list<Hsp> banded_3frame_swipe_targets(vector<DpTarget>::const_iterator begin,
 	vector<DpTarget> &overflow)
 {
 	list<Hsp> out;
-	for (vector<DpTarget>::const_iterator i = begin; i < end; i += ScoreTraits<_sv>::CHANNELS) {
-		if (score_only || config.traceback_mode == TracebackMode::SCORE_ONLY)
-			out.splice(out.end(), banded_3frame_swipe<_sv, DP::ScoreOnly>(query, strand, i, i + std::min(vector<DpTarget>::const_iterator::difference_type(ScoreTraits<_sv>::CHANNELS), end - i), stat, parallel, overflow));
+	for (vector<DpTarget>::const_iterator i = begin; i < end; i += std::min((ptrdiff_t)ScoreTraits<_sv>::CHANNELS, end - i)) {
+		if (score_only)
+			out.splice(out.end(), banded_3frame_swipe<_sv, DP::ScoreOnly>(query, strand, i, i + std::min(ptrdiff_t(ScoreTraits<_sv>::CHANNELS), end - i), stat, parallel, overflow));
 		else
-			out.splice(out.end(), banded_3frame_swipe<_sv, DP::Traceback>(query, strand, i, i + std::min(vector<DpTarget>::const_iterator::difference_type(ScoreTraits<_sv>::CHANNELS), end - i), stat, parallel, overflow));
+			out.splice(out.end(), banded_3frame_swipe<_sv, DP::Traceback>(query, strand, i, i + std::min(ptrdiff_t(ScoreTraits<_sv>::CHANNELS), end - i), stat, parallel, overflow));
 	}
 	return out;
 }
@@ -537,7 +545,7 @@ void banded_3frame_swipe_worker(vector<DpTarget>::const_iterator begin,
 	while (begin + (pos = next->fetch_add(config.swipe_chunk_size)) < end)
 #ifdef __SSE2__
 		if(score_only)
-			out->splice(out->end(), banded_3frame_swipe_targets<score_vector<int16_t>>(begin + pos, min(begin + pos + config.swipe_chunk_size, end), score_only, *query, strand, stat, true, of));
+			out->splice(out->end(), banded_3frame_swipe_targets<ScoreVector<int16_t, SHRT_MIN>>(begin + pos, min(begin + pos + config.swipe_chunk_size, end), score_only, *query, strand, stat, true, of));
 		else
 			out->splice(out->end(), banded_3frame_swipe_targets<int32_t>(begin + pos, min(begin + pos + config.swipe_chunk_size, end), score_only, *query, strand, stat, true, of));
 #else
@@ -584,7 +592,7 @@ list<Hsp> banded_3frame_swipe(const TranslatedSequence &query, Strand strand, ve
 	}
 	else {
 		if(score_only)
-			out = banded_3frame_swipe_targets<score_vector<int16_t>>(target_begin, target_end, score_only, query, strand, stat, false, overflow16);
+			out = banded_3frame_swipe_targets<ScoreVector<int16_t, SHRT_MIN>>(target_begin, target_end, score_only, query, strand, stat, false, overflow16);
 		else
 			out = banded_3frame_swipe_targets<int32_t>(target_begin, target_end, score_only, query, strand, stat, false, overflow16);
 	}
