@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
 						
 Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
@@ -28,6 +28,37 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../../util/dynamic_iterator.h"
 #include "../../stats/hauser_correction.h"
 
+template<typename T, int N>
+struct SmallVector
+{
+	SmallVector() :
+		n(0)
+	{}
+	T& operator[](int i)
+	{
+		return data[i];
+	}
+	const T& operator[](int i) const
+	{
+		return data[i];
+	}
+	int size() const
+	{
+		return n;
+	}
+	void push_back(const T& x)
+	{
+		data[n++] = x;
+	}
+	void erase(int i)
+	{
+		memmove(&data[i], &data[i + 1], (--n - i) * sizeof(T));
+	}
+private:
+	T data[N];
+	int n;
+};
+
 namespace DISPATCH_ARCH {
 
 template<typename _t>
@@ -35,7 +66,6 @@ struct TargetIterator
 {
 
 	typedef ::DISPATCH_ARCH::SIMD::Vector<_t> SeqVector;
-	typedef ::DISPATCH_ARCH::score_vector<_t> ScoreVector;
 	enum {
 		CHANNELS = SeqVector::CHANNELS
 	};
@@ -80,7 +110,6 @@ struct TargetIterator
 	}
 
 
-#ifdef __SSSE3__
 	SeqVector get() const
 	{
 		alignas(32) _t s[CHANNELS];
@@ -91,17 +120,6 @@ struct TargetIterator
 		}
 		return SeqVector(s);
 	}
-#else
-	uint64_t get() const
-	{
-		uint64_t dst = 0;
-		for (int i = 0; i < active.size(); ++i) {
-			const int channel = active[i];
-			dst |= uint64_t((*this)[channel]) << (8 * channel);
-		}
-		return dst;
-	}
-#endif
 
 	const int8_t** get(const int8_t** target_scores) const {
 		static const int8_t blank[32] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
@@ -157,121 +175,39 @@ struct TargetIterator
 
 	int pos[CHANNELS], target[CHANNELS], next, n_targets, cols;
 	bool custom_matrix_16bit;
-	Static_vector<int, CHANNELS> active;
+	SmallVector<int, CHANNELS> active;
 	const vector<DpTarget>::const_iterator subject_begin;
 };
 
-template<typename _t>
-struct TargetBuffer
-{
-
-	typedef ::DISPATCH_ARCH::SIMD::Vector<_t> SeqVector;
-	enum { CHANNELS = SeqVector::CHANNELS };
-
-	TargetBuffer(typename std::vector<DpTarget>::const_iterator subject_begin, typename std::vector<DpTarget>::const_iterator subject_end):
-		next(0),
-		n_targets(int(subject_end - subject_begin)),
-		subject_begin(subject_begin)
-	{
-		for (; next < std::min((int)CHANNELS, n_targets); ++next) {
-			pos[next] = 0;
-			target[next] = next;
-			active.push_back(next);
-		}
-	}
-
-	int max_len() const {
-		int l = 0;
-		for (int i = 0; i < n_targets; ++i)
-			l = std::max(l, (int)subject_begin[i].seq.length());
-		return l;
-	}
-
-	char operator[](int channel) const
-	{
-		if (pos[channel] >= 0) {
-			return dp_target(channel).seq[pos[channel]];
-		}
-		else
-			return SUPER_HARD_MASK;;
-	}
-
-#ifdef __SSSE3__
-	SeqVector seq_vector() const
-	{
-		alignas(32) _t s[CHANNELS];
-		std::fill(s, s + CHANNELS, SUPER_HARD_MASK);
-		for (int i = 0; i < active.size(); ++i) {
-			const int channel = active[i];
-			s[channel] = (*this)[channel];
-		}
-		return SeqVector(s);
-	}
-#else
-	uint64_t seq_vector()
-	{
-		uint64_t dst = 0;
-		for (int i = 0; i < active.size(); ++i) {
-			const int channel = active[i];
-			dst |= uint64_t((*this)[channel]) << (8 * channel);
-		}
-		return dst;
-	}
-#endif
-
-	bool init_target(int i, int channel)
-	{
-		if (next < n_targets) {
-			pos[channel] = 0;
-			target[channel] = next++;
-			return true;
-		}
-		active.erase(i);
-		return false;
-	}
-
-	bool inc(int channel)
-	{
-		++pos[channel];
-		if (pos[channel] >= (int)dp_target(channel).seq.length())
-			return false;
-		return true;
-	}
-
-	const DpTarget& dp_target(int channel) const {
-		return subject_begin[target[channel]];
-	}
-
-	int pos[CHANNELS], target[CHANNELS], next, n_targets, cols;
-	Static_vector<int, CHANNELS> active;
-	const vector<DpTarget>::const_iterator subject_begin;
-
-};
-
-template<typename _t>
+template<typename T, typename It>
 struct AsyncTargetBuffer
 {
 
-	typedef ::DISPATCH_ARCH::SIMD::Vector<_t> SeqVector;
+	typedef ::DISPATCH_ARCH::SIMD::Vector<T> SeqVector;
 	enum { CHANNELS = SeqVector::CHANNELS };
 
-	AsyncTargetBuffer(DynamicIterator<DpTarget>& target_it):
-		target_it(target_it)
+	AsyncTargetBuffer(const It begin, const It end, std::atomic_size_t* const next):
+		begin(begin),
+		target_count(end - begin),
+		next(next),
+		custom_matrix_16bit(false)
 	{
-		for (int i = 0; i < CHANNELS; ++i) {
-			DpTarget t = target_it++;
+		size_t n, i = 0;
+		while (i < CHANNELS && (n = (*next)++) < target_count) {
+			DpTarget t = begin[n];
 			if (t.blank())
-				return;
+				t.target_idx = n;
 			pos[i] = 0;
 			dp_targets[i] = t;
 			active.push_back(i);
+			++i;
 		}
 	}
 
 	int max_len() const {
 		int l = 0;
-		for (size_t i = 0; i < target_it.count; ++i)
-			l = std::max(l, (int)target_it[i].seq.length());
+		for (size_t i = 0; i < target_count; ++i)
+			l = std::max(l, (int)DpTarget(begin[i]).seq.length());
 		return l;
 	}
 
@@ -284,10 +220,9 @@ struct AsyncTargetBuffer
 			return SUPER_HARD_MASK;
 	}
 
-#ifdef __SSSE3__
 	SeqVector seq_vector() const
 	{
-		alignas(32) _t s[CHANNELS];
+		alignas(32) T s[CHANNELS];
 		std::fill(s, s + CHANNELS, SUPER_HARD_MASK);
 		for (int i = 0; i < active.size(); ++i) {
 			const int channel = active[i];
@@ -295,28 +230,45 @@ struct AsyncTargetBuffer
 		}
 		return SeqVector(s);
 	}
-#else
-	uint64_t seq_vector()
-	{
-		uint64_t dst = 0;
+
+	const int8_t** get(const int8_t** target_scores) const {
+		static const int8_t blank[32] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+		std::fill(target_scores, target_scores + 32, blank);
 		for (int i = 0; i < active.size(); ++i) {
 			const int channel = active[i];
-			dst |= uint64_t((*this)[channel]) << (8 * channel);
+			const int l = (int)(*this)[channel];
+			const DpTarget& dp_target = dp_targets[channel];
+			target_scores[channel] = dp_target.adjusted_matrix() ? &dp_target.matrix->scores[32 * l] : &score_matrix.matrix8()[32 * l];
 		}
-		return dst;
+		return target_scores;
+	}
+
+	std::vector<const int32_t*> get32() const {
+		static const int32_t blank[32] = { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+		std::vector<const int32_t*> target_scores(CHANNELS);
+		std::fill(target_scores.begin(), target_scores.end(), blank);
+		for (int i = 0; i < active.size(); ++i) {
+			const int channel = active[i];
+			const int l = (int)(*this)[channel];
+			const DpTarget& dp_target = dp_targets[channel];
+			target_scores[channel] = dp_target.adjusted_matrix() ? &dp_target.matrix->scores32[32 * l] : &score_matrix.matrix32()[32 * l];
+		}
+		return target_scores;
 	}
-#endif
 
 	bool init_target(int i, int channel)
 	{
-		DpTarget t = target_it++;
-		if (!t.blank()) {
-			pos[channel] = 0;
-			dp_targets[channel] = t;
-			return true;
+		const size_t n = (*next)++;
+		if (n >= target_count) {
+			active.erase(i);
+			return false;
 		}
-		active.erase(i);
-		return false;
+		DpTarget t = begin[n];
+		if (t.blank())
+			t.target_idx = n;
+		pos[channel] = 0;
+		dp_targets[channel] = t;
+		return true;
 	}
 
 	bool inc(int channel)
@@ -327,10 +279,27 @@ struct AsyncTargetBuffer
 		return true;
 	}
 
+	uint32_t cbs_mask() {
+		uint32_t r = 0;
+		custom_matrix_16bit = false;
+		for (int i = 0; i < active.size(); ++i) {
+			const int channel = active[i];
+			if (dp_targets[channel].adjusted_matrix()) {
+				r |= 1 << channel;
+				if (dp_targets[channel].matrix->score_max > SCHAR_MAX || dp_targets[channel].matrix->score_min < SCHAR_MIN)
+					custom_matrix_16bit = true;
+			}
+		}
+		return r;
+	}
+
 	int pos[CHANNELS];
-	Static_vector<int, CHANNELS> active;
-	DynamicIterator<DpTarget>& target_it;
+	SmallVector<int, CHANNELS> active;
+	const It begin;
+	const size_t target_count;
+	std::atomic_size_t* const next;
 	DpTarget dp_targets[CHANNELS];
+	bool custom_matrix_16bit;
 
 };
 
