@@ -18,17 +18,17 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #include "output_format.h"
-#include "recursive_parser.h"
 #include "../data/queries.h"
 
-void Clustering_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const 
+void Clustering_format::print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const
 {
 }
 
-void Clustering_format::print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) 
+void Clustering_format::print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) 
 {
-	out.write((uint32_t) query_block_to_database_id[r.query_id]);
-	out.write((uint32_t) r.orig_subject_id);
-	RecursiveParser rp(&r, format->c_str(), false);
-	out.write(rp.evaluate());
+	out.write((uint32_t) metadata.query->block_id2oid(r.query_id));
+	out.write((uint32_t) r.subject_oid);
+	RecursiveParser rp(&r, format.c_str(), false);
+	const double res = rp.evaluate();
+	out.write(res);
 }
