@@ -21,35 +21,52 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #pragma once
+#include <algorithm>
 #include <exception>
 #include <memory>
 #include "../basic/match.h"
-#include "../output/daa_file.h"
-#include "../output/daa_record.h"
+#include "../output/daa/daa_file.h"
+#include "../output/daa/daa_record.h"
 #include "../util/io/output_file.h"
 #include "../stats/score_matrix.h"
 #include "../util/escape_sequences.h"
-#include "../basic/parameters.h"
-#include "../data/metadata.h"
 #include "../util/io/consumer.h"
+#include "../output/recursive_parser.h"
+#include "../util/enum.h"
+#include "../run/config.h"
+#include "../dp/flags.h"
+
+namespace Output {
+
+enum class Flags : int {
+	NONE            = 0,
+	FULL_TITLES     = 1,
+	ALL_SEQIDS      = 1 << 1,
+	TARGET_SEQS     = 1 << 2,
+	SELF_ALN_SCORES = 1 << 3
+};
+
+DEF_ENUM_FLAG_OPERATORS(Flags)
+
+}
 
 struct Output_format
 {
-	Output_format(unsigned code):
+	Output_format(unsigned code, HspValues hsp_values = HspValues::TRANSCRIPT, Output::Flags flags = Output::Flags::NONE):
 		code(code),
 		needs_taxon_id_lists(false),
 		needs_taxon_nodes(false),
 		needs_taxon_scientific_names(false),
 		needs_taxon_ranks(false),
-		needs_transcript(true),
-		needs_stats(false),
-		needs_paired_end_info(false)
+		needs_paired_end_info(false),
+		hsp_values(hsp_values),
+		flags(flags)
 	{}
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const
 	{}
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Search::Config &parameters) const
 	{}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out)
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out)
 	{}
 	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const
 	{ }
@@ -64,7 +81,9 @@ struct Output_format
 		return code;
 	}
 	unsigned code;
-	bool needs_taxon_id_lists, needs_taxon_nodes, needs_taxon_scientific_names, needs_taxon_ranks, needs_transcript, needs_stats, needs_paired_end_info;
+	bool needs_taxon_id_lists, needs_taxon_nodes, needs_taxon_scientific_names, needs_taxon_ranks, needs_paired_end_info;
+	HspValues hsp_values;
+	Output::Flags flags;
 	enum { daa, blast_tab, blast_xml, sam, blast_pairwise, null, taxon, paf, bin1 };
 };
 
@@ -83,9 +102,7 @@ struct Null_format : public Output_format
 
 struct DAA_format : public Output_format
 {
-	DAA_format():
-		Output_format(daa)
-	{}
+	DAA_format();
 	virtual Output_format* clone() const
 	{
 		return new DAA_format(*this);
@@ -94,7 +111,8 @@ struct DAA_format : public Output_format
 
 struct OutputField {
 	const std::string key, description;
-	const bool need_transcript, need_stats;
+	const HspValues hsp_values;
+	const Output::Flags flags;
 };
 
 struct Blast_tab_format : public Output_format
@@ -102,24 +120,24 @@ struct Blast_tab_format : public Output_format
 	static const std::vector<OutputField> field_def;
 	Blast_tab_format();
 	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const override;
+	virtual void print_match(const HspContext& r, const Search::Config& metadata, TextBuffer &out) override;
 	virtual ~Blast_tab_format()
 	{ }
 	virtual Output_format* clone() const override
 	{
 		return new Blast_tab_format(*this);
 	}
-	vector<unsigned> fields;
+	vector<int64_t> fields;
 };
 
 struct PAF_format : public Output_format
 {
 	PAF_format():
-		Output_format(paf)
+		Output_format(paf, HspValues::TRANSCRIPT, Output::Flags::NONE)
 	{}
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const;
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out);
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const;
+	virtual void print_match(const HspContext& r, const Search::Config& metadata, TextBuffer &out);
 	virtual ~PAF_format()
 	{ }
 	virtual Output_format* clone() const
@@ -131,11 +149,11 @@ struct PAF_format : public Output_format
 struct Sam_format : public Output_format
 {
 	Sam_format():
-		Output_format(sam)
+		Output_format(sam, HspValues::TRANSCRIPT, Output::Flags::NONE)
 	{ }
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) override;
 	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const override;
 	virtual ~Sam_format()
 	{ }
 	virtual Output_format* clone() const override
@@ -147,14 +165,14 @@ struct Sam_format : public Output_format
 struct XML_format : public Output_format
 {
 	XML_format():
-		Output_format(blast_xml)
+		Output_format(blast_xml, HspValues::TRANSCRIPT, Output::Flags::FULL_TITLES)
 	{
 		config.salltitles = true;
-	}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	} 
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) override;
 	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const override;
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Search::Config &parameters) const override;
 	virtual void print_footer(Consumer &f) const override;
 	virtual ~XML_format()
 	{ }
@@ -167,14 +185,14 @@ struct XML_format : public Output_format
 struct Pairwise_format : public Output_format
 {
 	Pairwise_format() :
-		Output_format(blast_pairwise)
+		Output_format(blast_pairwise, HspValues::TRANSCRIPT, Output::Flags::FULL_TITLES)
 	{
 		config.salltitles = true;
 	}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) override;
 	virtual void print_header(Consumer &f, int mode, const char *matrix, int gap_open, int gap_extend, double evalue, const char *first_query_name, unsigned first_query_len) const override;
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const override;
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Search::Config &parameters) const override;
 	virtual void print_footer(Consumer &f) const override;
 	virtual ~Pairwise_format()
 	{ }
@@ -187,15 +205,15 @@ struct Pairwise_format : public Output_format
 struct Taxon_format : public Output_format
 {
 	Taxon_format() :
-		Output_format(taxon),
+		Output_format(taxon, HspValues::NONE, Output::Flags::NONE),
 		taxid(0),
 		evalue(std::numeric_limits<double>::max())
 	{
 		needs_taxon_id_lists = true;
 		needs_taxon_nodes = true;
 	}
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
-	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Parameters &parameters) const override;
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) override;
+	virtual void print_query_epilog(TextBuffer &out, const char *query_title, bool unaligned, const Search::Config &parameters) const override;
 	virtual ~Taxon_format()
 	{ }
 	virtual Output_format* clone() const override
@@ -211,8 +229,8 @@ struct Bin1_format : public Output_format
 	Bin1_format():
 		Output_format(bin1)
 	{}
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const override;
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) override;
 	virtual ~Bin1_format()
 	{ }
 	virtual Output_format* clone() const override
@@ -223,11 +241,12 @@ struct Bin1_format : public Output_format
 
 struct Clustering_format : public Output_format
 {
-	const string* const format;
-	Clustering_format(const string* const format): Output_format(bin1), format(format) {
+	string format;
+	Clustering_format(const string* const format): Output_format(bin1) {
+		this->format = RecursiveParser::clean_expression(format);
 	}
-	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned) const override;
-	virtual void print_match(const Hsp_context& r, const Metadata &metadata, TextBuffer &out) override;
+	virtual void print_query_intro(size_t query_num, const char *query_name, unsigned query_len, TextBuffer &out, bool unaligned, const Search::Config& cfg) const override;
+	virtual void print_match(const HspContext& r, const Search::Config &metadata, TextBuffer &out) override;
 	virtual ~Clustering_format()
 	{ }
 	virtual Output_format* clone() const override
@@ -239,9 +258,9 @@ struct Clustering_format : public Output_format
 struct Binary_format : public Output_format
 {
 	Binary_format() :
-		Output_format(bin1)
+		Output_format(bin1, HspValues::NONE)
 	{}
-	virtual void print_match(const Hsp_context& r, const Metadata& metadata, TextBuffer& out) override;
+	virtual void print_match(const HspContext& r, const Search::Config & metadata, TextBuffer& out) override;
 	virtual ~Binary_format()
 	{ }
 	virtual Output_format* clone() const override
@@ -251,7 +270,6 @@ struct Binary_format : public Output_format
 };
 
 Output_format* get_output_format();
-void init_output(bool have_taxon_id_lists, bool have_taxon_nodes, bool have_taxon_scientific_names);
+void init_output();
 void print_hsp(Hsp &hsp, const TranslatedSequence &query);
-void print_cigar(const Hsp_context &r, TextBuffer &buf);
-
+void print_cigar(const HspContext &r, TextBuffer &buf);
