@@ -24,6 +24,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../cluster/cluster_registry.h"
 #include "../output/recursive_parser.h"
 #include "../util/simd.h"
+#include "../data/dmnd/dmnd.h"
 
 using std::cout;
 using std::cerr;
@@ -33,11 +34,11 @@ void model_seqs();
 void opt();
 void run_masker();
 void fastq2fasta();
-void view();
+void view_daa();
+void view_tsv();
 void db_info();
 void test_main();
 void benchmark_sw();
-void test_io();
 void db_annot_stats();
 void read_sim();
 void info();
@@ -51,6 +52,15 @@ void get_medoids_from_tree();
 void roc();
 void merge_tsv();
 void roc_id();
+void makeindex();
+void find_shapes();
+void composition();
+void join();
+void hash_seqs();
+void list_seeds();
+#ifdef WITH_BLASTDB
+void prep_blast_db();
+#endif
 
 void split();
 namespace Benchmark { DECL_DISPATCH(void, benchmark, ()) }
@@ -73,14 +83,19 @@ int main(int ac, const char* av[])
 			cout << Const::program_name << " version " << Const::version_string << endl;
 			break;
 		case Config::makedb:
-			make_db();
+			DatabaseFile::make_db();
 			break;
 		case Config::blastp:
 		case Config::blastx:
-			Workflow::Search::run(Workflow::Search::Options());
+			Search::run();
 			break;
 		case Config::view:
-			view();
+			if (!config.daa_file.empty())
+				view_daa();
+			else if (!config.input_ref_file.empty())
+				view_tsv();
+			else
+				throw std::runtime_error("The view command requires either a text (option --in) or DAA (option -a) input file.");
 			break;
 		case Config::getseq:
 			get_seq();
@@ -103,9 +118,6 @@ int main(int ac, const char* av[])
 		case Config::dbinfo:
 			db_info();
 			break;
-		case Config::test_io:
-			test_io();
-			break;
 		case Config::read_sim:
 			read_sim();
 			break;
@@ -118,7 +130,8 @@ int main(int ac, const char* av[])
 		case Config::cluster:
 			// Why is cluster_similarity not set at the end of the Config constructor?
 			if(!config.cluster_similarity.empty()){
-				RecursiveParser rp(nullptr, config.cluster_similarity.c_str(), true);
+				string expression = RecursiveParser::clean_expression(&config.cluster_similarity);
+				RecursiveParser rp(nullptr, expression.c_str(), true);
 				try{
 					rp.evaluate();
 				}
@@ -174,6 +187,29 @@ int main(int ac, const char* av[])
 		case Config::rocid:
 			roc_id();
 			break;
+		case Config::makeidx:
+			makeindex();
+			break;
+		case Config::find_shapes:
+			find_shapes();
+			break;
+		case Config::HASH_SEQS:
+			hash_seqs();
+			break;
+#ifdef WITH_BLASTDB
+		case Config::prep_blast_db:
+			prep_blast_db();
+			break;
+#endif
+		case Config::composition:
+			composition();
+			break;
+		case Config::JOIN:
+			join();
+			break;
+		case Config::LIST_SEEDS:
+			list_seeds();
+			break;
 		default:
 			return 1;
 		}
