@@ -24,36 +24,48 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include <sstream>
 #include <memory>
 #include <chrono>
+#include <thread>
 #include "tools.h"
 #include "../basic/config.h"
 #include "../data/sequence_set.h"
 #include "../util/seq_file_format.h"
 #include "../data/queries.h"
 #include "../data/load_seqs.h"
-#include "../data/reference.h"
-#include "../basic/masking.h"
+#include "../data/sequence_file.h"
+#include "../masking/masking.h"
 #include "../dp/dp.h"
 #include "../basic/packed_transcript.h"
+#include "../data/reference.h"
+#include "../data/dmnd/dmnd.h"
+#include "../util/util.h"
+#include "../util/sequence/sequence.h"
+#include "../basic/translate.h"
 
-using namespace std;
+using std::thread;
 using std::chrono::high_resolution_clock;
 using std::chrono::duration_cast;
+using std::cout;
+using std::endl;
+using std::cerr;
+using std::unique_ptr;
 
 void get_seq()
 {
-	DatabaseFile db_file(config.database);
-	db_file.get_seq();
+	SequenceFile* db_file = SequenceFile::auto_create(config.database);
+	db_file->get_seq();
+	delete db_file;
 }
 
 void random_seqs()
 {
 	DatabaseFile db_file(config.database);
-	db_file.load_seqs(nullptr, std::numeric_limits<size_t>::max(), &ref_seqs::data_, &ref_ids::data_);
-	cout << "Sequences = " << ref_seqs::get().get_length() << endl;
+	Block* ref_seqs = db_file.load_seqs(std::numeric_limits<size_t>::max());
+	const auto& r = ref_seqs->seqs();
+	cout << "Sequences = " << r.size() << endl;
 	std::set<unsigned> n;
 	const size_t count = atoi(config.seq_no[0].c_str());
 	while (n.size() < count)
-		n.insert((rand()*RAND_MAX + rand()) % ref_seqs::get().get_length());
+		n.insert((rand()*RAND_MAX + rand()) % r.size());
 	OutputFile out(config.output_file);
 	unsigned j = 0;
 	
@@ -64,12 +76,13 @@ void random_seqs()
 		if (config.reverse)
 			; // ref_seqs::get()[*i].print(ss, value_traits, sequence::Reversed());
 		else
-			ss << ref_seqs::get()[*i];
+			ss << r[*i];
 		ss << endl;
 		s = ss.str();
 		out.write(s.data(), s.length());
 	}
 	out.close();
+	delete ref_seqs;
 }
 
 void sort_file()
@@ -90,14 +103,15 @@ void sort_file()
 void db_stat()
 {
 	DatabaseFile db_file(config.database);
-	db_file.load_seqs(nullptr, std::numeric_limits<size_t>::max(), &ref_seqs::data_, &ref_ids::data_);
-	cout << "Sequences = " << ref_seqs::get().get_length() << endl;
+	Block* ref_seqs = db_file.load_seqs(std::numeric_limits<size_t>::max());
+	const auto& r = ref_seqs->seqs();
+	cout << "Sequences = " << r.size() << endl;
 
 	size_t letters = 0;
 	vector<size_t> letter_freq(20);
-	for (size_t i = 0; i < ref_seqs::get().get_length(); ++i) {
-		const sequence seq = ref_seqs::get()[i];
-		for (size_t j = 0; j < seq.length(); ++j) {
+	for (size_t i = 0; i < r.size(); ++i) {
+		const Sequence seq = r[i];
+		for (Loc j = 0; j < seq.length(); ++j) {
 			if (seq[j] < 20) {
 				++letters;
 				++letter_freq[(int)seq[j]];
@@ -124,7 +138,7 @@ void run_masker()
 	while (format.get_seq(id, seq, f, value_traits)) {
 		cout << '>' << id << endl;
 		seq2 = seq;
-		Masking::get()(seq2.data(), seq2.size());
+		Masking::get()(seq2.data(), seq2.size(), MaskingAlgo::TANTAN, 0);
 		/*for (size_t i = 0; i < seq.size(); ++i) {
 			char c = value_traits.alphabet[(long)seq[i]];
 			if (seq2[i] == value_traits.mask_char)
@@ -132,7 +146,7 @@ void run_masker()
 			cout << c;
 		}
 		cout << endl;*/
-		cout << sequence(seq2.data(), seq2.size()) << endl;
+		cout << Sequence(seq2.data(), seq2.size()) << endl;
 		size_t n = 0;
 		for (size_t i = 0; i < seq2.size(); ++i)
 			if (seq2[i] == value_traits.mask_char) {
@@ -156,30 +170,11 @@ void fastq2fasta()
 	size_t n = 0, max = atoi(config.seq_no[0].c_str());
 	while (n < max && format.get_seq(id, seq, *f, value_traits)) {
 		cout << '>' << id << endl;
-		cout << sequence(seq.data(), seq.size()) << endl;
+		cout << Sequence(seq.data(), seq.size()) << endl;
 		++n;
 	}
 }
 
-void test_io()
-{
-	const size_t buf_size = 1;
-	//Timer t;
-	//t.start();
-	/*InputFile f(config.query_file, InputFile::BUFFERED);
-	char buf[buf_size];	
-	size_t total = 0;
-	while (f.read(buf, buf_size) == buf_size)
-		total += buf_size;*/
-
-	DatabaseFile db(config.database);
-	db.load_seqs(nullptr, std::numeric_limits<size_t>::max(), &ref_seqs::data_, &ref_ids::data_);
-
-	size_t total = ref_seqs::get().raw_len() + ref_ids::get().raw_len();
-	//cout << "MBytes/sec = " << total / 1e6 / t.getElapsedTime() << endl;
-	//cout << "Time = " << t.getElapsedTime() << "s" << endl;
-}
-
 void read_sim()
 {
 	const double ID = 0.35;
@@ -200,7 +195,7 @@ void read_sim()
 				buf << nucleotide_traits.alphabet[rand() % 4];
 		}
 		buf << '\n';
-		out.write(buf.get_begin(), buf.size());
+		out.write(buf.data(), buf.size());
 		buf.clear();
 	}
 	out.close();
@@ -244,11 +239,11 @@ void pairwise_worker(TextInputFile *in, std::mutex *input_lock, std::mutex *outp
 			return;
 		}
 		input_lock->unlock();
-		const string ir = blast_id(id_r), iq = blast_id(id_q);
-		Hsp hsp;
-		smith_waterman(sequence(query), sequence(ref), hsp);
-		Hsp_context context(hsp, 0, TranslatedSequence(query), "", 0, 0, "", 0, 0, 0, sequence());
-		Hsp_context::Iterator it = context.begin();
+		const string ir = Util::Seq::seqid(id_r.c_str(), false), iq = Util::Seq::seqid(id_q.c_str(), false);
+		Hsp hsp(true);
+		//smith_waterman(Sequence(query), Sequence(ref), hsp);
+		HspContext context(hsp, 0, TranslatedSequence(query), "", 0, 0, nullptr, 0, 0, Sequence());
+		HspContext::Iterator it = context.begin();
 		std::stringstream ss;
 		while (it.good()) {
 			if (it.op() == op_substitution)
@@ -267,7 +262,7 @@ void pairwise()
 {
 	input_value_traits = nucleotide_traits;
 	value_traits = nucleotide_traits;
-	score_matrix = Score_matrix("DNA", 5, 2, 0, 1);
+	score_matrix = ScoreMatrix("DNA", 5, 2, 0, 1);
 
 	TextInputFile in(config.single_query_file());
 	std::mutex input_lock, output_lock;
@@ -280,7 +275,7 @@ void pairwise()
 
 void fasta_skip_to(string &id, vector<Letter> &seq, string &blast_id, TextInputFile &f)
 {
-	while (::blast_id(id) != blast_id) {
+	while (Util::Seq::seqid(id.c_str(), false) != blast_id) {
 		if (!FASTA_format().get_seq(id, seq, f, value_traits))
 			throw runtime_error("Sequence not found in FASTA file.");
 	}
@@ -295,7 +290,7 @@ void translate() {
 	while (FASTA_format().get_seq(id, seq, in, input_value_traits)) {
 		Translator::translate(seq, proteins);
 		cout << ">" << id << endl;
-		cout << sequence(proteins[0]) << endl;
+		cout << Sequence(proteins[0]) << endl;
 	}
 	in.close();
 }
@@ -311,23 +306,22 @@ void reverse() {
 		buf << '\\';
 		buf.write_raw(id.data(), id.size());
 		buf << '\n';
-		sequence(seq).print(buf, amino_acid_traits, sequence::Reversed());
+		Sequence(seq).print(buf, amino_acid_traits, Sequence::Reversed());
 		buf << '\n';
 		buf << '\0';
-		cout << buf.get_begin();
+		cout << buf.data();
 		buf.clear();
 	}
 	in.close();
 }
 
 void show_cbs() {
-	score_matrix = Score_matrix("BLOSUM62", config.gap_open, config.gap_extend, config.frame_shift, 1);
-	init_cbs();
+	score_matrix = ScoreMatrix("BLOSUM62", config.gap_open, config.gap_extend, config.frame_shift, 1);
 	TextInputFile in(config.single_query_file());
 	string id;
 	vector<Letter> seq;
 	while (FASTA_format().get_seq(id, seq, in, value_traits)) {
-		Bias_correction bc{ sequence(seq) };
+		Bias_correction bc{ Sequence(seq) };
 		for (size_t i = 0; i < seq.size(); ++i)
 			cout << value_traits.alphabet[(long)seq[i]] << '\t' << bc[i] << endl;
 	}
