@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2016-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
 						
 Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
@@ -20,33 +20,13 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
 #pragma once
-#include <list>
-#include "../basic/config.h"
-#include "../util/data_structures/bit_vector.h"
+#include <memory>
+#include "../data/sequence_file.h"
+#include "../util/io/text_input_file.h"
+#include "../util/io/consumer.h"
 
-struct DatabaseFile;
-struct Consumer;
-struct TextInputFile;
-
-namespace Workflow { 
 namespace Search {
 
-struct Options {
-	Options():
-		self(config.self),
-		db(nullptr),
-		consumer(nullptr),
-		query_file(nullptr),
-		db_filter(nullptr)
-	{}
-	bool self;
-	DatabaseFile *db;
-	Consumer *consumer;
-	std::list<TextInputFile> *query_file;
-	const BitVector* db_filter;
-};
-
-void run(const Options &options);
+void run(const std::shared_ptr<SequenceFile>& db = nullptr, const std::shared_ptr<std::list<TextInputFile>>& query = nullptr, const std::shared_ptr<Consumer>& out = nullptr, const std::shared_ptr<BitVector>& db_filter = nullptr);
 
 }
-}
