@@ -21,7 +21,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "cbs.h"
 #include "../basic/config.h"
 #include "score_matrix.h"
-#include "../basic/masking.h"
+#include "../masking/masking.h"
 
 namespace Stats {
 
@@ -48,11 +48,11 @@ CBS::CBS(unsigned code, double query_match_distance_threshold, double length_rat
         this->length_ratio_threshold = length_ratio_threshold;
 }
 
-Composition composition(const sequence& s) {
+Composition composition(const Sequence& s) {
     Composition r;
     r.fill(0.0);
     int n = 0;
-    for (size_t i = 0; i < s.length(); ++i) {
+    for (Loc i = 0; i < s.length(); ++i) {
         int l = s[i];
         if (l < 20) {
             ++r[l];
@@ -66,34 +66,38 @@ Composition composition(const sequence& s) {
     return r;
 }
 
-int count_true_aa(const sequence& s) {
+int count_true_aa(const Sequence& s) {
     int n = 0;
-    for (size_t i = 0; i < s.length(); ++i)
+    for (Loc i = 0; i < s.length(); ++i)
         if ((size_t)s[i] < TRUE_AA)
             ++n;
     return n;
 }
 
-bool use_seg_masking(const sequence& a, const sequence& b) {
+bool use_seg_masking(const Sequence& a, const Sequence& b) {
     if (config.comp_based_stats != CBS::COMP_BASED_STATS_AND_MATRIX_ADJUST || a.length() != b.length())
         return true;
-    size_t n = 0;
-    for (size_t i = 0; i < a.length(); ++i)
+    Loc n = 0;
+    for (Loc i = 0; i < a.length(); ++i)
         if (a[i] == b[i])
             ++n;
     return n != a.length();
 }
 
-TargetMatrix::TargetMatrix(const Composition& query_comp, int query_len, const sequence& target)
+int TargetMatrix::score_width() const {
+    return (score_max > SCHAR_MAX || score_min < SCHAR_MIN) ? 1 : 0;
+}
+
+TargetMatrix::TargetMatrix(const Composition& query_comp, int query_len, const Sequence& target)
 {
-    if (!CBS::matrix_adjust(config.comp_based_stats))
+    if (!CBS::matrix_adjust(config.comp_based_stats) || target.length() == 0 || query_len == 0)
         return;
     
     vector<Letter> target_seq = target.copy();
     //Masking::get()(target_seq.data(), target_seq.size(), Masking::Algo::SEG);
 
     //auto c = composition(target);
-    auto c = composition(sequence(target_seq.data(), target_seq.size()));
+    auto c = composition(Sequence(target_seq.data(), target_seq.size()));
     EMatrixAdjustRule rule = eUserSpecifiedRelEntropy;
     if (CBS::conditioned(config.comp_based_stats)) {
         rule = s_TestToApplyREAdjustmentConditional(query_len, (int)target.length(), query_comp.data(), c.data(), score_matrix.background_freqs());
@@ -109,6 +113,8 @@ TargetMatrix::TargetMatrix(const Composition& query_comp, int query_len, const s
     
     if (config.comp_based_stats == CBS::COMP_BASED_STATS || rule == eCompoScaleOldMatrix)
         s = CompositionBasedStats(score_matrix.matrix32_scaled_pointers().data(), query_comp, c, score_matrix.ungapped_lambda(), score_matrix.freq_ratios());
+    else if (config.comp_based_stats == CBS::HAUSER_GLOBAL)
+        s = hauser_global(query_comp, c);
     else
         s = CompositionMatrixAdjust(query_len, count_true_aa(target), query_comp.data(), c.data(), config.cbs_matrix_scale, score_matrix.ideal_lambda(), score_matrix.joint_probs(), score_matrix.background_freqs());
     
@@ -119,7 +125,7 @@ TargetMatrix::TargetMatrix(const Composition& query_comp, int query_len, const s
                 scores32[i * 32 + j] = s[j * AMINO_ACID_COUNT + i];
                 score_min = std::min(score_min, s[j * AMINO_ACID_COUNT + i]);
                 score_max = std::max(score_max, s[j * AMINO_ACID_COUNT + i]);
-                //std::cerr << s2[j * 20 + i] << ' ';
+                //std::cerr << s[j * AMINO_ACID_COUNT + i] << ' ';
             }
             else {
                 scores[i * 32 + j] = std::max(score_matrix(i, j) * config.cbs_matrix_scale, SCHAR_MIN);
@@ -127,7 +133,7 @@ TargetMatrix::TargetMatrix(const Composition& query_comp, int query_len, const s
                 score_min = std::min(score_min, scores32[i * 32 + j]);
                 score_max = std::max(score_max, scores32[i * 32 + j]);
             }
-        //std::cerr << endl;
+        //std::cerr << std::endl;
     }
 }
 
