@@ -29,7 +29,7 @@ namespace Stats {
 
 using Composition = std::array<double, TRUE_AA>;
 
-Composition composition(const sequence& s);
+Composition composition(const Sequence& s);
 
 struct TargetMatrix {
 
@@ -38,7 +38,11 @@ struct TargetMatrix {
 
     TargetMatrix(const int16_t* query_matrix, const int16_t* target_matrix);
 
-    TargetMatrix(const Composition& query_comp, int query_len, const sequence& target);
+    TargetMatrix(const Composition& query_comp, int query_len, const Sequence& target);
+    int score_width() const;
+    bool blank() const {
+        return scores.empty();
+    }
 
     std::vector<int8_t> scores;
     std::vector<int32_t> scores32;
@@ -94,8 +98,8 @@ int s_GetMatrixScoreProbs(double** scoreProb, int* obs_min, int* obs_max,
 double s_CalcLambda(double probs[], int min_score, int max_score, double lambda0);
 double ideal_lambda(const int** matrix);
 void s_SetXUOScores(double** M, int alphsize, const double row_probs[], const double col_probs[]);
-int count_true_aa(const sequence& s);
-bool use_seg_masking(const sequence& a, const sequence& b);
+int count_true_aa(const Sequence& s);
+bool use_seg_masking(const Sequence& a, const Sequence& b);
 
 EMatrixAdjustRule
 s_TestToApplyREAdjustmentConditional(int Len_query,
@@ -111,6 +115,7 @@ struct CBS {
         case 4:
         case COMP_BASED_STATS:
         case COMP_BASED_STATS_AND_MATRIX_ADJUST:
+        case HAUSER_GLOBAL:
             return false;
         case 1:
         case 2:
@@ -130,6 +135,7 @@ struct CBS {
         case MATRIX_ADJUST:
         case COMP_BASED_STATS:
         case COMP_BASED_STATS_AND_MATRIX_ADJUST:
+        case HAUSER_GLOBAL:
             return true;
         default:
             throw std::runtime_error("Unknown CBS code.");
@@ -178,6 +184,7 @@ struct CBS {
         case MATRIX_ADJUST:
         case COMP_BASED_STATS:
         case COMP_BASED_STATS_AND_MATRIX_ADJUST:
+        case HAUSER_GLOBAL:
             return 1;
         default:
             return 0;
@@ -191,6 +198,7 @@ struct CBS {
         MATRIX_ADJUST = 4,
         COMP_BASED_STATS = 5,
         COMP_BASED_STATS_AND_MATRIX_ADJUST = 6,
+        HAUSER_GLOBAL = 7,
         COUNT
     };
     CBS(unsigned code, double query_match_distance_threshold, double length_ratio_threshold, double angle);
@@ -202,6 +210,7 @@ struct CBS {
 
 std::vector<int> CompositionMatrixAdjust(int query_len, int target_len, const double* query_comp, const double* target_comp, int scale, double ungapped_lambda, const double* joint_probs, const double* background_freqs);
 std::vector<int> CompositionBasedStats(const int* const* matrix_in, const Composition& queryProb, const Composition& resProb, double lambda, const FreqRatios& freq_ratios);
+std::vector<int> hauser_global(const Composition& query_comp, const Composition& target_comp);
 int Blast_OptimizeTargetFrequencies(double x[],
     int alphsize,
     int* iterations,
