@@ -26,8 +26,6 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../basic/diagonal_segment.h"
 #include "../basic/match.h"
 
-void init_cbs();
-
 struct No_score_correction
 {
 	void operator()(int &score, int i, int query_anchor, int mult) const
@@ -36,12 +34,14 @@ struct No_score_correction
 
 struct Bias_correction : public std::vector<float>
 {
-	Bias_correction(const sequence &seq);
+	Bias_correction() {}
+	Bias_correction(const Sequence &seq);
 	void operator()(float &score, int i, int query_anchor, int mult) const
 	{
 		score += (*this)[query_anchor + i * mult];
 	}
 	int operator()(const Hsp &hsp) const;
 	int operator()(const Diagonal_segment &d) const;
+	static std::vector<int8_t> reverse(const int8_t* p, const size_t len);
 	std::vector<int8_t> int8;
 };
