@@ -18,21 +18,29 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#include "../basic/config.h"
-#include "score_matrix.h"
-#include <string.h>
-#include <assert.h>
-#include <math.h>
-#include <stdlib.h>
-#include "../lib/blast/nlm_linear_algebra.h"
-#include "cbs.h"
-
-namespace Stats {
-
-static constexpr int COMPO_NUM_TRUE_AA = 20;
-static const int kReMatrixAdjustmentPseudocounts = 20;
-/** relative entropy of BLOSUM62 */
-static const double kFixedReBlosum62 = 0.44;
+/* ===========================================================================
+*
+*                            PUBLIC DOMAIN NOTICE
+*               National Center for Biotechnology Information
+*
+*  This software/database is a "United States Government Work" under the
+*  terms of the United States Copyright Act.  It was written as part of
+*  the author's official duties as a United States Government employee and
+*  thus cannot be copyrighted.  This software/database is freely available
+*  to the public for use. The National Library of Medicine and the U.S.
+*  Government have not placed any restriction on its use or reproduction.
+*
+*  Although all reasonable efforts have been taken to ensure the accuracy
+*  and reliability of the software and data, the NLM and the U.S.
+*  Government do not and cannot warrant the performance or results that
+*  may be obtained by using this software or data. The NLM and the U.S.
+*  Government disclaim all warranties, express or implied, including
+*  warranties of performance, merchantability or fitness for any particular
+*  purpose.
+*
+*  Please cite the author in any work or product based on this material.
+*
+* ===========================================================================*/
 
 /**
  * @file optimize_target_freq.c
@@ -95,6 +103,30 @@ static const double kFixedReBlosum62 = 0.44;
  * @author E. Michael Gertz
  */
 
+
+#include "../basic/config.h"
+#include "score_matrix.h"
+#include <string.h>
+#include <assert.h>
+#include <math.h>
+#include <stdlib.h>
+#include "../lib/blast/nlm_linear_algebra.h"
+#include "cbs.h"
+
+namespace Stats {
+
+static constexpr int COMPO_NUM_TRUE_AA = 20;
+static const int kReMatrixAdjustmentPseudocounts = 20;
+/** relative entropy of BLOSUM62 */
+static const double kFixedReBlosum62 = 0.44;
+
+static void print(const double* v, int n, const char* s) {
+    /*std::cout << s << std::endl;
+    for (int i = 0; i < n; ++i)
+        std::cout << v[i] << ' ';
+    std::cout << std::endl;*/
+}
+
 //#include <algo/blast/composition_adjustment/optimize_target_freq.h>
 
 /** bound on error for Newton's method */
@@ -258,6 +290,7 @@ ResidualsLinearConstraints(double rA[], int alphsize, const double x[],
         rA[i + alphsize - 1] = row_sums[i];
     }
     MultiplyByA(1.0, rA, alphsize, -1.0, x);
+    print(rA, 2 * 20 - 1, "ResidualsLinearConstraints rA");
 }
 
 
@@ -296,6 +329,7 @@ DualResiduals(double resids_x[], int alphsize, double** grads,
         }
     }
     MultiplyByAtranspose(1.0, resids_x, alphsize, 1.0, z);
+    print(resids_x, n, "DualResiduals resids_x");
 }
 
 
@@ -355,6 +389,7 @@ CalculateResiduals(double* rnorm,
     }
     *rnorm =
         sqrt(norm_resids_x * norm_resids_x + norm_resids_z * norm_resids_z);
+    print(rnorm, 1, "CalculateResiduals rnorm");
 }
 
 
@@ -529,6 +564,7 @@ FactorReNewtonSystem(ReNewtonSystem* newton_system,
     /* Then we compute J D^{-1} J^T; First fill in the part that corresponds
      * to the linear constraints */
     ScaledSymmetricProductA(W, Dinv, alphsize);
+    print(W[0], 820, "FactorReNewtonSystem W");
 
     if (constrain_rel_entropy) {
         /* Save the gradient of the relative entropy constraint. */
@@ -757,6 +793,7 @@ Blast_OptimizeTargetFrequencies(double x[],
     if (grads == NULL) goto error_return;
 
     ComputeScoresFromProbs(old_scores, alphsize, q, row_sums, col_sums);
+    print(old_scores, n, "old_scores");
 
     /* Use q as the initial value for x */
     memcpy(x, q, n * sizeof(double));
@@ -767,6 +804,8 @@ Blast_OptimizeTargetFrequencies(double x[],
         /* Compute the residuals */
         EvaluateReFunctions(values, grads, alphsize, x, q, old_scores,
             constrain_rel_entropy);
+        print(values, 2, "values");
+        print(grads[0], 2 * n, "grads");
         CalculateResiduals(&rnorm, resids_x, alphsize, resids_z, values,
             grads, row_sums, col_sums, x, z,
             constrain_rel_entropy, relative_entropy);
@@ -801,6 +840,7 @@ Blast_OptimizeTargetFrequencies(double x[],
             }
         }
     }
+    print(x, 20 * 20, "x");
     converged = 0;
     if (its <= maxits && rnorm <= tol) {
         /* Newton's iteration converged */
@@ -857,7 +897,7 @@ Blast_ApplyPseudocounts(double* probs20,
 
 /* Documented in composition_adjustment.h. */
 void
-Blast_TrueAaToStdTargetFreqs(double** StdFreq, int StdAlphsize,
+Blast_TrueAaToStdTargetFreqs(double** StdFreq, size_t StdAlphsize,
     const double* freq)
 {
     /* Note I'm using a rough convention for this routine that uppercase
@@ -867,37 +907,37 @@ Blast_TrueAaToStdTargetFreqs(double** StdFreq, int StdAlphsize,
      */
      /* Shorter names for the sizes of the two alphabets */
     const int small_alphsize = COMPO_NUM_TRUE_AA;
-    int A, B;          /* characters in the std (big) alphabet */
-    int a, b;          /* characters in the small alphabet */
+    size_t A, B;          /* characters in the std (big) alphabet */
+    size_t a, b;          /* characters in the small alphabet */
     double sum;        /* sum of values in target_freq; used to normalize */
     sum = 0.0;
     for (a = 0; a < small_alphsize; a++) {
         for (b = 0; b < small_alphsize; b++) {
-            sum += freq[a * 20  + b];
+            sum += freq[a * TRUE_AA  + b];
         }
     }
     for (A = 0; A < StdAlphsize; A++) {
         /* for all rows */
-        //if (alphaConvert[A] < 0) {
-        //    /* the row corresponds to a nonstandard reside */
-        //    for (B = 0; B < StdAlphsize; B++) {
-        //        StdFreq[A][B] = 0.0;
-        //    }
-        //}
-        //else {
+        if (A >= TRUE_AA) {
+            /* the row corresponds to a nonstandard reside */
+            for (B = 0; B < StdAlphsize; B++) {
+                StdFreq[A][B] = 0.0;
+            }
+        }
+        else {
             /* the row corresponds to a standard reside */
             a = A;
 
             for (B = 0; B < StdAlphsize; B++) {
                 /* for all columns */
-                if (B < 0) {
+                if (B >= TRUE_AA) {
                     /* the column corresponds to a nonstandard reside */
                     StdFreq[A][B] = 0.0;
                 }
                 else {
                     /* the column corresponds to a standard reside */
                     b = B;
-                    StdFreq[A][B] = freq[a * 20 + b] / sum;
+                    StdFreq[A][B] = freq[a * TRUE_AA + b] / sum;
                 }
             }
             /* Set values for two-character ambiguities */
@@ -906,7 +946,7 @@ Blast_TrueAaToStdTargetFreqs(double** StdFreq, int StdAlphsize,
             //if (StdAlphsize > eJchar) {
             //    StdFreq[A][eJchar] = StdFreq[A][eIchar] + StdFreq[A][eLchar];
             //}
-        //}
+        }
     }
     /* Add rows to set values for two-character ambiguities */
     //memcpy(StdFreq[eBchar], StdFreq[eDchar], StdAlphsize * sizeof(double));
@@ -938,8 +978,6 @@ Blast_CalcFreqRatios(double** ratios, int alphsize,
     }
 }
 
-
-
 /**
  * Given a set of target frequencies and two sets of character
  * probabilities for the true amino acids in the ARND alphabet,
@@ -957,7 +995,7 @@ Blast_CalcFreqRatios(double** ratios, int alphsize,
  * @param Lambda        the desired scale of this matrix
  */
 static int
-s_ScoresStdAlphabet(int** Matrix, int Alphsize,
+s_ScoresStdAlphabet(int** Matrix, size_t Alphsize,
     const double* target_freq,
     const double row_prob[], const double col_prob[],
     double Lambda)
@@ -967,7 +1005,6 @@ s_ScoresStdAlphabet(int** Matrix, int Alphsize,
      * and lowercase letters refer to the true amino acid (smaller)
      * alphabet.
      */
-    int i;
     /* row and column probabilities in the NCBIstdaa alphabet */
     //double RowProb[COMPO_LARGEST_ALPHABET];
     //double ColProb[COMPO_LARGEST_ALPHABET];
@@ -983,9 +1020,9 @@ s_ScoresStdAlphabet(int** Matrix, int Alphsize,
     //s_SetPairAmbigProbsToSum(ColProb, Alphsize);
 
     Blast_TrueAaToStdTargetFreqs(Scores, Alphsize, target_freq);
-    Blast_CalcFreqRatios(Scores, Alphsize, row_prob, col_prob);
+    Blast_CalcFreqRatios(Scores, TRUE_AA, row_prob, col_prob);
     Blast_FreqRatioToScore(Scores, Alphsize, Alphsize, Lambda);
-    //s_SetXUOScores(Scores, Alphsize, RowProb, ColProb);
+    s_SetXUOScores(Scores, TRUE_AA, row_prob, col_prob);
 
     s_RoundScoreMatrix(Matrix, Alphsize, Alphsize, Scores);
     Nlm_DenseMatrixFree(&Scores);
@@ -998,9 +1035,8 @@ s_ScoresStdAlphabet(int** Matrix, int Alphsize,
 }
 
 /* Documented in composition_adjustment.h. */
-int
+static int
 Blast_CompositionMatrixAdj(int** matrix,
-    int alphsize,
     EMatrixAdjustRule matrix_adjust_rule,
     int length1,
     int length2,
@@ -1010,11 +1046,17 @@ Blast_CompositionMatrixAdj(int** matrix,
     const double* joint_probs,
     const double* background_freqs)
 {
+    /*for (int i = 0; i < 20; ++i)
+        printf("%.10f ", stdaa_row_probs[i]);
+    printf("\n");
+    for (int i = 0; i < 20; ++i)
+        printf("%.10f ", stdaa_col_probs[i]);
+    printf("\n");*/
     int iteration_count, status;
     double row_probs[COMPO_NUM_TRUE_AA], col_probs[COMPO_NUM_TRUE_AA];
     /* Target RE when optimizing the matrix; zero if the relative
        entropy should not be constrained. */
-    double dummy, desired_re = 0.0;
+    double desired_re = 0.0;
     std::copy(stdaa_row_probs, stdaa_row_probs + 20, row_probs);
     std::copy(stdaa_col_probs, stdaa_col_probs + 20, col_probs);
 
@@ -1051,7 +1093,7 @@ Blast_CompositionMatrixAdj(int** matrix,
     Blast_ApplyPseudocounts(col_probs, length2,
         background_freqs);
 
-    vector<double> mat_final(20 * 20);
+    vector<double> mat_final(TRUE_AA * TRUE_AA);
 
     status =
         Blast_OptimizeTargetFrequencies(mat_final.data(),
@@ -1068,7 +1110,7 @@ Blast_CompositionMatrixAdj(int** matrix,
         return status;
 
     return
-        s_ScoresStdAlphabet(matrix, alphsize, mat_final.data(),
+        s_ScoresStdAlphabet(matrix, AMINO_ACID_COUNT, mat_final.data(),
             row_probs, col_probs,
             lambda);
 }
@@ -1215,9 +1257,9 @@ s_TestToApplyREAdjustmentConditional(int Len_query,
         which_rule = eUserSpecifiedRelEntropy;
     }
     else {
-        if ((D_m_q >= config.query_match_distance_threshold) &&
-            (len_large / len_small > config.length_ratio_threshold) &&
-            (angle > config.cbs_angle)) {
+        if ((D_m_q > comp_based_stats.query_match_distance_threshold) &&
+            (len_large / len_small > comp_based_stats.length_ratio_threshold) &&
+            (angle > comp_based_stats.angle)) {
             which_rule = eCompoScaleOldMatrix;
         }
         else {
@@ -1228,13 +1270,12 @@ s_TestToApplyREAdjustmentConditional(int Len_query,
 }
 
 vector<int> CompositionMatrixAdjust(int query_len, int target_len, const double* query_comp, const double* target_comp, int scale, double ungapped_lambda, const double* joint_probs, const double* background_freqs) {
-    vector<int> v(TRUE_AA * TRUE_AA);
+    vector<int> v(AMINO_ACID_COUNT * AMINO_ACID_COUNT);
     vector<int*> p;
-    p.reserve(TRUE_AA);
-    for (size_t i = 0; i < TRUE_AA; ++i)
-        p.push_back(&v[i * TRUE_AA]);
+    p.reserve(AMINO_ACID_COUNT);
+    for (size_t i = 0; i < AMINO_ACID_COUNT; ++i)
+        p.push_back(&v[i * AMINO_ACID_COUNT]);
     int r = Blast_CompositionMatrixAdj(p.data(),
-        TRUE_AA,
         eUserSpecifiedRelEntropy,
         query_len,
         target_len,
@@ -1244,9 +1285,9 @@ vector<int> CompositionMatrixAdjust(int query_len, int target_len, const double*
         joint_probs,
         background_freqs);
     if (r != 0) {
-        for (size_t i = 0; i < TRUE_AA; ++i)
-            for (size_t j = 0; j < TRUE_AA; ++j)
-                v[i * 20 + j] = score_matrix(i, j) * scale;
+        for (size_t i = 0; i < AMINO_ACID_COUNT; ++i)
+            for (size_t j = 0; j < AMINO_ACID_COUNT; ++j)
+                v[i * AMINO_ACID_COUNT + j] = score_matrix(i, j) * scale;
         //throw std::runtime_error("Error computing composition matrix adjust.");
     }
     return v;
