@@ -28,6 +28,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #include "../util/seq_file_format.h"
 #include "../util/sequence/sequence.h"
 #include "../util/io/output_file.h"
+#include "../stats/score_matrix.h"
 
 using std::vector;
 using std::cout;
@@ -62,7 +63,7 @@ vector<Letter> generate_random_seq(size_t length, std::minstd_rand0 &random_engi
 	vector<Letter> seq;
 	seq.reserve(length);
 	for (size_t i = 0; i < length; ++i)
-		seq.push_back(get_distribution<20>(background_freq, random_engine));
+		seq.push_back(get_distribution<20>(score_matrix.background_freqs(), random_engine));
 	return seq;
 }
 
@@ -71,16 +72,16 @@ void simulate_seqs() {
 	std::minstd_rand0 rand_engine;
 	for (size_t i = 0; i < n; ++i) {
 		cout << ">" << i << endl;
-		cout << sequence(generate_random_seq(l, rand_engine)) << endl;
+		cout << Sequence(generate_random_seq(l, rand_engine)) << endl;
 	}
 }
 
-vector<Letter> simulate_homolog(const sequence &seq, double id, std::minstd_rand0 &random_engine)
+vector<Letter> simulate_homolog(const Sequence &seq, double id, std::minstd_rand0 &random_engine)
 {
 	vector<Letter> out;
 	out.reserve(seq.length());
 	std::uniform_int_distribution<int> dist(0, 3);
-	for (unsigned i = 0; i < seq.length(); ++i)
+	for (Loc i = 0; i < seq.length(); ++i)
 		if ((double)rand() / RAND_MAX < id)
 			out.push_back(seq[i]);
 		else {
@@ -102,7 +103,7 @@ void mutate() {
 	std::uniform_real_distribution<double> id_dist(0.3, 1.0);
 	while (FASTA_format().get_seq(id, seq, in, value_traits)) {
 		const double i = id_dist(random_engine);
-		Util::Sequence::format(sequence(simulate_homolog(sequence(seq), i, random_engine)), std::to_string(int(i*100)).c_str(), nullptr, out, "fasta", nucleotide_traits);
+		Util::Seq::format(Sequence(simulate_homolog(Sequence(seq), i, random_engine)), std::to_string(int(i*100)).c_str(), nullptr, out, "fasta", nucleotide_traits);
 	}
 	out.close();
 	in.close();
