@@ -3,16 +3,16 @@ DIAMOND protein aligner
 Copyright (C) 2013-2017 Benjamin Buchfink <buchfink@gmail.com>
 
 This program is free software: you can redistribute it and/or modify
-it under the terms of the GNU Affero General Public License as
-published by the Free Software Foundation, either version 3 of the
-License, or (at your option) any later version.
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation, either version 3 of the License, or
+(at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-GNU Affero General Public License for more details.
+GNU General Public License for more details.
 
-You should have received a copy of the GNU Affero General Public License
+You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
@@ -66,6 +66,14 @@ struct BinaryBuffer : public vector<char>
 			case 2: read(dst);
 			}
 		}
+		void read_packed(uint8_t length, int32_t& dst)
+		{
+			switch (length) {
+			case 0: uint8_t x; read(x); dst = (int32_t)x; break;
+			case 1: uint16_t y; read(y); dst = (int32_t)y; break;
+			case 2: read(dst);
+			}
+		}
 		void read_varint(uint32_t &dst)
 		{
 			::read_varint(*this, dst);
