@@ -1,6 +1,6 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2020 Max Planck Society for the Advancement of Science e.V.
+Copyright (C) 2013-2021 Max Planck Society for the Advancement of Science e.V.
                         Benjamin Buchfink
                         Eberhard Karls Universitaet Tuebingen
 
@@ -18,9 +18,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef DOUBLE_ARRAY_H_
-#define DOUBLE_ARRAY_H_
-
+#pragma once
 #include <stdint.h>
 #include <stddef.h>
 #include <algorithm>
@@ -138,5 +136,3 @@ private:
 	size_t size_;
 
 };
-
-#endif
\ No newline at end of file
