@@ -1,6 +1,9 @@
 /****
 DIAMOND protein aligner
-Copyright (C) 2013-2018 Benjamin Buchfink <buchfink@gmail.com>
+Copyright (C) 2016-2021 Max Planck Society for the Advancement of Science e.V.
+                        Benjamin Buchfink
+						
+Code developed by Benjamin Buchfink <benjamin.buchfink@tue.mpg.de>
 
 This program is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -16,21 +19,22 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef INPUT_FILE_H_
-#define INPUT_FILE_H_
-
+#pragma once
 #include <vector>
 #include <string>
 #include <stdexcept>
 #include <stdint.h>
 #include "output_file.h"
 #include "deserializer.h"
+#include "temp_file.h"
 
 using std::vector;
 using std::string;
 using std::runtime_error;
 
-struct TempFile;
+const size_t MEGABYTES = 1 << 20;
+const size_t GIGABYTES = 1 << 30;
+const size_t KILOBYTES = 1 << 10;
 
 struct InputFile : public Deserializer
 {
@@ -46,5 +50,3 @@ struct InputFile : public Deserializer
 	bool unlinked;
 	
 };
-
-#endif /* BINARY_FILE_H_ */
