@@ -16,9 +16,7 @@ You should have received a copy of the GNU General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 ****/
 
-#ifndef OUTPUT_FILE_H_
-#define OUTPUT_FILE_H_
-
+#pragma once
 #include <map>
 #include <stdexcept>
 #include <string>
@@ -29,9 +27,11 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 using std::string;
 using std::pair;
 
+enum class Compressor { NONE, ZLIB, ZSTD };
+
 struct OutputFile : public Serializer
 {
-	OutputFile(const string &file_name, bool compressed = false, const char *mode = "wb");
+	OutputFile(const string &file_name, Compressor compressor = Compressor::NONE, const char *mode = "wb");
 #ifndef _MSC_VER
 	OutputFile(pair<string, int> fd, const char *mode);
 #endif
@@ -44,7 +44,7 @@ struct OutputFile : public Serializer
 		TextBuffer buf;
 		for (typename std::map<_k, _v>::const_iterator i = begin; i != end; ++i) {
 			buf << i->first << '\t' << i->second << '\n';
-			write(buf.get_begin(), buf.size());
+			write(buf.data(), buf.size());
 			buf.clear();
 		}
 	}
@@ -59,5 +59,3 @@ protected:
 	string file_name_;
 
 };
-
-#endif
\ No newline at end of file
