@@ -61,6 +61,11 @@ struct Serializer : public Consumer
 		return *this;
 	}
 
+	Serializer& operator<<(const double x) {
+		write(x);
+		return *this;
+	}
+
 	Serializer& operator<<(const std::vector<unsigned> &v)
 	{
 		*this << (unsigned)v.size();
@@ -92,6 +97,13 @@ struct Serializer : public Consumer
 		return *this;
 	}
 
+	template<typename Type1, typename Type2>
+	Serializer& operator<<(const std::pair<Type1, Type2>& p) {
+		*this << p.first;
+		*this << p.second;
+		return *this;
+	}
+
 	template<typename _t>
 	void write(const _t &x)
 	{
