@@ -25,7 +25,9 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 struct TempFile : public OutputFile
 {
 
-	TempFile();
+	TempFile(bool unlink = true);
+	TempFile(const std::string & file_name);
+	virtual void finalize() override {}
 	static std::string get_temp_dir();
 	static unsigned n;
 	static uint64_t hash_key;
@@ -34,9 +36,9 @@ struct TempFile : public OutputFile
 private:
 
 #ifdef _MSC_VER
-	std::string init();
+	std::string init(bool unlink);
 #else
-	std::pair<std::string, int> init();
+	std::pair<std::string, int> init(bool unlink);
 #endif
 
 };
