@@ -1,6 +1,4 @@
-#ifndef UTIL_STRING_STRING_H_
-#define UTIL_STRING_STRING_H_
-
+#pragma once
 #include <string>
 #include <string.h>
 #include <algorithm>
@@ -55,7 +53,8 @@ inline int format_double(double x, char *p) {
 	return sprintf(p, "%lli.%lli", i / 10, i % 10);
 }
 
+std::string replace(const std::string& s, char a, char b);
+std::string ratio_percentage(const double x, const double y);
+std::string ratio_percentage(const size_t x, const size_t y);
 
 }}
-
-#endif
\ No newline at end of file
