@@ -26,14 +26,14 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #ifdef _MSC_VER
 
 #define PACKED_ATTRIBUTE
-#define FORCE_INLINE __forceinline
-#define MSC_INLINE __forceinline
+#define FORCE_INLINE __forceinline static
+#define FLATTEN
 
 #else
 
 #define PACKED_ATTRIBUTE __attribute__((packed))
-#define FORCE_INLINE __attribute__((always_inline))
-#define MSC_INLINE
+#define FORCE_INLINE __attribute__((always_inline)) static inline
+#define FLATTEN __attribute__((flatten))
 
 #endif
 
