@@ -109,4 +109,10 @@ void print_binary(uint64_t x)
 		std::cout << (x & 1);
 		x >>= 1;
 	}
+}
+
+void exit_with_error(const std::exception& e) {
+	std::cerr << "Error: " << e.what() << endl;
+	log_stream << "Error: " << e.what() << endl;
+	exit(EXIT_FAILURE);
 }
\ No newline at end of file
